@@ -250,6 +250,11 @@ afsd_ServiceControlHandlerEx(
     {
     case SERVICE_CONTROL_SHUTDOWN:
     case SERVICE_CONTROL_STOP:
+	if (ctrlCode == SERVICE_CONTROL_SHUTDOWN)
+	    afsi_log("SERVICE_CONTROL_SHUTDOWN");
+	else
+            afsi_log("SERVICE_CONTROL_STOP");
+
         ServiceStatus.dwCurrentState = SERVICE_STOP_PENDING;
         ServiceStatus.dwWin32ExitCode = NO_ERROR;
         ServiceStatus.dwCheckPoint = 1;
@@ -339,7 +344,7 @@ afsd_ServiceControlHandlerEx(
 		    powerStateSuspended = 1;
 		    if (osVersion.dwMajorVersion >= 6) {
                         cm_SuspendSCache();
-			smb_StopListeners();
+			smb_StopListeners(0);
                     }
                     dwRet = NO_ERROR;                       
                     break;                                  
@@ -348,14 +353,14 @@ afsd_ServiceControlHandlerEx(
 		    powerStateSuspended = 1;
 		    if (osVersion.dwMajorVersion >= 6) {
                         cm_SuspendSCache();
-			smb_StopListeners();
+			smb_StopListeners(0);
                     }
                     dwRet = NO_ERROR;                       
                     break;                                  
                 case PBT_APMRESUMECRITICAL:             
                     afsi_log("SERVICE_CONTROL_APMRESUMECRITICAL"); 
 		    if (osVersion.dwMajorVersion >= 6)
-			smb_RestartListeners();
+			smb_RestartListeners(0);
                     dwRet = NO_ERROR;                       
                     break;                                  
                 case PBT_APMRESUMESUSPEND:                                                        
@@ -388,8 +393,9 @@ afsd_ServiceControlHandlerEx(
 		    /* This is the message delivered once all devices are up */
                     afsi_log("SERVICE_CONTROL_APMRESUMEAUTOMATIC"); 
 		    powerStateSuspended = 0;
-		    if (osVersion.dwMajorVersion >= 6)
-			smb_RestartListeners();
+		    if (osVersion.dwMajorVersion >= 6) {
+			smb_SetLanAdapterChangeDetected();
+                    }
                     dwRet = NO_ERROR;                       
                     break;                                  
                 default:                                                                          
@@ -436,7 +442,7 @@ static DWORD __stdcall MountGlobalDrivesThread(void * notUsed)
 
     dwResult = RegOpenKeyEx(HKEY_LOCAL_MACHINE, szKeyName, 0, KEY_QUERY_VALUE, &hKey);
     if (dwResult != ERROR_SUCCESS)
-        return;
+        return 0;
 
     while (dwIndex < MAX_DRIVES) {
         dwDriveSize = sizeof(szDriveToMapTo);
@@ -1000,7 +1006,11 @@ BOOL AFSModulesVerify(void)
         RegCloseKey (parmKey);
     }
 
-    if (verifyServiceSig && cacheSize < 716800) {
+    if (verifyServiceSig 
+#ifndef _WIN64
+         && cacheSize < 716800
+#endif
+         ) {
         trustVerified = VerifyTrust(filename);
     } else {
         afsi_log("Signature Verification disabled");
@@ -1305,6 +1315,9 @@ afsd_Main(DWORD argc, LPTSTR *argv)
         SetServiceStatus(StatusHandle, &ServiceStatus);
 #endif
 
+        /* Notify any volume status handlers that the cache manager has started */
+        cm_VolStatus_Service_Started();
+
 /* the following ifdef chooses the mode of operation for the service.  to enable
  * a runtime flag (instead of compile-time), pioctl() would need to dynamically
  * determine the mode, in order to use the correct ioctl special-file path. */
@@ -1367,9 +1380,6 @@ afsd_Main(DWORD argc, LPTSTR *argv)
 	LogEvent(EVENTLOG_INFORMATION_TYPE, MSG_SERVICE_RUNNING);
     }
 
-    /* Notify any volume status handlers that we have started */
-    cm_VolStatus_Service_Started();
-
     /* allow an exit to be called when started */
     hHookDll = LoadLibrary(AFSD_HOOK_DLL);
     if (hHookDll)
