@@ -143,10 +143,7 @@ void cm_CallbackNotifyChange(cm_scache_t *scp)
         cm_fid_t tfid;
         cm_scache_t *dscp;
 
-        tfid.cell = scp->fid.cell;
-        tfid.volume = scp->fid.volume;
-        tfid.vnode = scp->parentVnode;
-        tfid.unique = scp->parentUnique;
+        cm_SetFid(&tfid, scp->fid.cell, scp->fid.volume, scp->parentVnode, scp->parentUnique);
         dscp = cm_FindSCache(&tfid);
 #ifndef AFSIFS
         if ( dscp &&
@@ -361,8 +358,9 @@ SRXAFSCB_CallBack(struct rx_call *callp, AFSCBFids *fidsArrayp, AFSCBs *cbsArray
     cm_server_t *tsp = NULL;
     cm_cell_t* cellp = NULL;
 
+#ifdef HOLD_CALL_MUTEX
     MUTEX_ENTER(&callp->lock);
-
+#endif
     if ((connp = rx_ConnectionOf(callp)) && (peerp = rx_PeerOf(connp))) {
         host = rx_HostOf(peerp);
         port = rx_PortOf(peerp);
@@ -371,7 +369,7 @@ SRXAFSCB_CallBack(struct rx_call *callp, AFSCBFids *fidsArrayp, AFSCBs *cbsArray
         if (tsp)
             cellp = tsp->cellp;
 
-        if (cellp)
+        if (!cellp)
             osi_Log2(afsd_logp, "SRXAFSCB_CallBack from host 0x%x port %d",
                      ntohl(host),
                      ntohs(port));
@@ -395,8 +393,9 @@ SRXAFSCB_CallBack(struct rx_call *callp, AFSCBFids *fidsArrayp, AFSCBs *cbsArray
         else
             cm_RevokeCallback(callp, cellp, tfidp);
     }
-
+#ifdef HOLD_CALL_MUTEX
     MUTEX_EXIT(&callp->lock);
+#endif
     return 0;
 }
 
@@ -436,8 +435,9 @@ SRXAFSCB_InitCallBackState(struct rx_call *callp)
     unsigned long host = 0;
     unsigned short port = 0;
 
+#ifdef HOLD_CALL_MUTEX
     MUTEX_ENTER(&callp->lock);
-
+#endif
     if ((connp = rx_ConnectionOf(callp)) && (peerp = rx_PeerOf(connp))) {
         host = rx_HostOf(peerp);
         port = rx_PortOf(peerp);
@@ -518,7 +518,9 @@ SRXAFSCB_InitCallBackState(struct rx_call *callp)
             cm_PutServer(tsp);
 	} 
     }
+#ifdef HOLD_CALL_MUTEX
     MUTEX_EXIT(&callp->lock);
+#endif
     return 0;
 }
 
@@ -550,8 +552,9 @@ SRXAFSCB_Probe(struct rx_call *callp)
     unsigned long host = 0;
     unsigned short port = 0;
 
+#ifdef HOLD_CALL_MUTEX
     MUTEX_ENTER(&callp->lock);
-
+#endif
     if ((connp = rx_ConnectionOf(callp)) && (peerp = rx_PeerOf(connp))) {
         host = rx_HostOf(peerp);
         port = rx_PortOf(peerp);
@@ -561,7 +564,9 @@ SRXAFSCB_Probe(struct rx_call *callp)
               ntohl(host),
               ntohs(port));
 
+#ifdef HOLD_CALL_MUTEX
     MUTEX_EXIT(&callp->lock);
+#endif
     return 0;
 }
 
@@ -656,8 +661,9 @@ SRXAFSCB_GetLock(struct rx_call *callp, long index, AFSDBLock *lockp)
     unsigned long host = 0;
     unsigned short port = 0;
 
+#ifdef HOLD_CALL_MUTEX
     MUTEX_ENTER(&callp->lock);
-
+#endif
     if ((connp = rx_ConnectionOf(callp)) && (peerp = rx_PeerOf(connp))) {
         host = rx_HostOf(peerp);
         port = rx_PortOf(peerp);
@@ -700,7 +706,9 @@ SRXAFSCB_GetLock(struct rx_call *callp, long index, AFSDBLock *lockp)
         code = 0;
     }
 
+#ifdef HOLD_CALL_MUTEX
     MUTEX_EXIT(&callp->lock);
+#endif
     return code;
 }
 
@@ -716,8 +724,9 @@ SRXAFSCB_GetCE(struct rx_call *callp, long index, AFSDBCacheEntry *cep)
     unsigned long host = 0;
     unsigned short port = 0;
 
+#ifdef HOLD_CALL_MUTEX
     MUTEX_ENTER(&callp->lock);
-
+#endif
     if ((connp = rx_ConnectionOf(callp)) && (peerp = rx_PeerOf(connp))) {
         host = rx_HostOf(peerp);
         port = rx_PortOf(peerp);
@@ -809,7 +818,9 @@ SRXAFSCB_GetCE(struct rx_call *callp, long index, AFSDBCacheEntry *cep)
   fcnDone:
     lock_ReleaseRead(&cm_scacheLock);
 
+#ifdef HOLD_CALL_MUTEX
     MUTEX_EXIT(&callp->lock);
+#endif
     return (code);
 }
 
@@ -825,8 +836,9 @@ SRXAFSCB_GetCE64(struct rx_call *callp, long index, AFSDBCacheEntry64 *cep)
     unsigned long host = 0;
     unsigned short port = 0;
 
+#ifdef HOLD_CALL_MUTEX
     MUTEX_ENTER(&callp->lock);
-
+#endif
     if ((connp = rx_ConnectionOf(callp)) && (peerp = rx_PeerOf(connp))) {
         host = rx_HostOf(peerp);
         port = rx_PortOf(peerp);
@@ -923,7 +935,9 @@ SRXAFSCB_GetCE64(struct rx_call *callp, long index, AFSDBCacheEntry64 *cep)
   fcnDone:
     lock_ReleaseRead(&cm_scacheLock);
 
+#ifdef HOLD_CALL_MUTEX
     MUTEX_EXIT(&callp->lock);
+#endif
     return (code);
 }
 
@@ -936,8 +950,9 @@ SRXAFSCB_XStatsVersion(struct rx_call *callp, long *vp)
     unsigned long host = 0;
     unsigned short port = 0;
 
+#ifdef HOLD_CALL_MUTEX
     MUTEX_ENTER(&callp->lock);
-
+#endif
     if ((connp = rx_ConnectionOf(callp)) && (peerp = rx_PeerOf(connp))) {
         host = rx_HostOf(peerp);
         port = rx_PortOf(peerp);
@@ -947,7 +962,9 @@ SRXAFSCB_XStatsVersion(struct rx_call *callp, long *vp)
              ntohl(host), ntohs(port));
     *vp = -1;
 
+#ifdef HOLD_CALL_MUTEX
     MUTEX_EXIT(&callp->lock);
+#endif
     return RXGEN_OPCODE;
 }
 
@@ -961,8 +978,9 @@ SRXAFSCB_GetXStats(struct rx_call *callp, long cvn, long coln, long *srvp, long
     unsigned long host = 0;
     unsigned short port = 0;
 
+#ifdef HOLD_CALL_MUTEX
     MUTEX_ENTER(&callp->lock);
-
+#endif
     if ((connp = rx_ConnectionOf(callp)) && (peerp = rx_PeerOf(connp))) {
         host = rx_HostOf(peerp);
         port = rx_PortOf(peerp);
@@ -971,7 +989,9 @@ SRXAFSCB_GetXStats(struct rx_call *callp, long cvn, long coln, long *srvp, long
     osi_Log2(afsd_logp, "SRXAFSCB_GetXStats from host 0x%x port %d - not implemented",
              ntohl(host), ntohs(port));
 
+#ifdef HOLD_CALL_MUTEX
     MUTEX_EXIT(&callp->lock);
+#endif
     return RXGEN_OPCODE;
 }
 
@@ -999,8 +1019,9 @@ SRXAFSCB_WhoAreYou(struct rx_call *callp, struct interfaceAddr* addr)
     unsigned long host = 0;
     unsigned short port = 0;
 
+#ifdef HOLD_CALL_MUTEX
     MUTEX_ENTER(&callp->lock);
-
+#endif
     if ((connp = rx_ConnectionOf(callp)) && (peerp = rx_PeerOf(connp))) {
         host = rx_HostOf(peerp);
         port = rx_PortOf(peerp);
@@ -1026,7 +1047,9 @@ SRXAFSCB_WhoAreYou(struct rx_call *callp, struct interfaceAddr* addr)
             cm_NetMtu[i] : rx_mtu;
     }
 
+#ifdef HOLD_CALL_MUTEX
     MUTEX_EXIT(&callp->lock);
+#endif
     return 0;
 }
 
@@ -1036,7 +1059,7 @@ SRXAFSCB_InitCallBackState3(struct rx_call *callp, afsUUID* serverUuid)
     char *p = NULL;
 
     if (UuidToString((UUID *)serverUuid, &p) == RPC_S_OK) {
-        osi_Log1(afsd_logp, "SRXAFSCB_InitCallBackState3 %s ->",p);
+        osi_Log1(afsd_logp, "SRXAFSCB_InitCallBackState3 %s ->",osi_LogSaveString(afsd_logp,p));
         RpcStringFree(&p);
     } else
         osi_Log0(afsd_logp, "SRXAFSCB_InitCallBackState3 - no server Uuid ->");
@@ -1055,8 +1078,9 @@ SRXAFSCB_ProbeUuid(struct rx_call *callp, afsUUID* clientUuid)
     char *p,*q;
     int code = 0;
 
+#ifdef HOLD_CALL_MUTEX
     MUTEX_ENTER(&callp->lock);
-
+#endif
     if ((connp = rx_ConnectionOf(callp)) && (peerp = rx_PeerOf(connp))) {
         host = rx_HostOf(peerp);
         port = rx_PortOf(peerp);
@@ -1079,40 +1103,23 @@ SRXAFSCB_ProbeUuid(struct rx_call *callp, afsUUID* clientUuid)
                   ntohl(host),
                   ntohs(port));
 
+#ifdef HOLD_CALL_MUTEX
     MUTEX_EXIT(&callp->lock);
+#endif
     return code;
 }
 
 /* debug interface */
-int 
-SRXAFSCB_GetCellByNum(struct rx_call *callp, afs_int32 a_cellnum,
-                      char **a_name, serverList *a_hosts)
+static int 
+GetCellCommon(afs_int32 a_cellnum, char **a_name, serverList *a_hosts)
 {
     afs_int32 sn;
     cm_cell_t * cellp;
     cm_serverRef_t * serverRefp; 
-    struct rx_connection *connp;
-    struct rx_peer *peerp;
-    unsigned long host = 0;
-    unsigned short port = 0;
 
-    MUTEX_ENTER(&callp->lock);
-
-    if ((connp = rx_ConnectionOf(callp)) && (peerp = rx_PeerOf(connp))) {
-        host = rx_HostOf(peerp);
-        port = rx_PortOf(peerp);
-    }
-
-    osi_Log3(afsd_logp, "SRXAFSCB_GetCellByNum(%d) from host 0x%x port %d",
-             a_cellnum, ntohl(host), ntohs(port));
-
-    a_hosts->serverList_val = 0;
-    a_hosts->serverList_len = 0;
-
-    cellp = cm_FindCellByID(a_cellnum);
+    cellp = cm_FindCellByID(a_cellnum, CM_FLAG_NOPROBE);
     if (!cellp) {
         *a_name = strdup("");
-        MUTEX_EXIT(&callp->lock);
         return 0;
     }
 
@@ -1134,10 +1141,43 @@ SRXAFSCB_GetCellByNum(struct rx_call *callp, afs_int32 a_cellnum,
     }
 
     lock_ReleaseRead(&cm_serverLock);
-    MUTEX_EXIT(&callp->lock);
     return 0;
 }
 
+/* debug interface */
+int 
+SRXAFSCB_GetCellByNum(struct rx_call *callp, afs_int32 a_cellnum,
+                      char **a_name, serverList *a_hosts)
+{
+    struct rx_connection *connp;
+    struct rx_peer *peerp;
+    unsigned long host = 0;
+    unsigned short port = 0;
+    int rc;
+
+#ifdef HOLD_CALL_MUTEX
+    MUTEX_ENTER(&callp->lock);
+#endif
+    if ((connp = rx_ConnectionOf(callp)) && (peerp = rx_PeerOf(connp))) {
+        host = rx_HostOf(peerp);
+        port = rx_PortOf(peerp);
+    }
+
+    osi_Log3(afsd_logp, "SRXAFSCB_GetCellByNum(%d) from host 0x%x port %d",
+             a_cellnum, ntohl(host), ntohs(port));
+
+    a_hosts->serverList_val = 0;
+    a_hosts->serverList_len = 0;
+
+
+    rc = GetCellCommon(a_cellnum, a_name, a_hosts);
+
+#ifdef HOLD_CALL_MUTEX
+    MUTEX_EXIT(&callp->lock);
+#endif
+    return rc;
+}
+
 /* debug interface */
 int 
 SRXAFSCB_TellMeAboutYourself( struct rx_call *callp, 
@@ -1158,8 +1198,9 @@ SRXAFSCB_TellMeAboutYourself( struct rx_call *callp,
     unsigned long host = 0;
     unsigned short port = 0;
 
+#ifdef HOLD_CALL_MUTEX
     MUTEX_ENTER(&callp->lock);
-
+#endif
     if ((connp = rx_ConnectionOf(callp)) && (peerp = rx_PeerOf(connp))) {
         host = rx_HostOf(peerp);
         port = rx_PortOf(peerp);
@@ -1191,7 +1232,9 @@ SRXAFSCB_TellMeAboutYourself( struct rx_call *callp,
     capabilities->Capabilities_len = dataBytes / sizeof(afs_int32);
     capabilities->Capabilities_val = dataBuffP;
 
+#ifdef HOLD_CALL_MUTEX
     MUTEX_EXIT(&callp->lock);
+#endif
     return 0;
 }
 
@@ -1228,8 +1271,9 @@ int SRXAFSCB_GetServerPrefs(
     unsigned long host = 0;
     unsigned short port = 0;
 
+#ifdef HOLD_CALL_MUTEX
     MUTEX_ENTER(&callp->lock);
-
+#endif
     if ((connp = rx_ConnectionOf(callp)) && (peerp = rx_PeerOf(connp))) {
         host = rx_HostOf(peerp);
         port = rx_PortOf(peerp);
@@ -1242,7 +1286,9 @@ int SRXAFSCB_GetServerPrefs(
     *a_srvr_addr = 0xffffffff;
     *a_srvr_rank = 0xffffffff;
 
+#ifdef HOLD_CALL_MUTEX
     MUTEX_EXIT(&callp->lock);
+#endif
     return 0;
 }
 
@@ -1271,14 +1317,15 @@ int SRXAFSCB_GetServerPrefs(
 int SRXAFSCB_GetCellServDB(struct rx_call *callp, afs_int32 index, char **a_name, 
                            serverList *a_hosts)
 {
-    char *t_name;
     struct rx_connection *connp;
     struct rx_peer *peerp;
     unsigned long host = 0;
     unsigned short port = 0;
+    int rc;
 
+#ifdef HOLD_CALL_MUTEX
     MUTEX_ENTER(&callp->lock);
-
+#endif
     if ((connp = rx_ConnectionOf(callp)) && (peerp = rx_PeerOf(connp))) {
         host = rx_HostOf(peerp);
         port = rx_PortOf(peerp);
@@ -1287,12 +1334,14 @@ int SRXAFSCB_GetCellServDB(struct rx_call *callp, afs_int32 index, char **a_name
     osi_Log2(afsd_logp, "SRXAFSCB_GetCellServDB from host 0x%x port %d - not implemented",
              ntohl(host), ntohs(port));
 
-    t_name = (char *)malloc(AFSNAMEMAX);
-    t_name[0] = '\0';
-    *a_name = t_name;
-    a_hosts->serverList_len = 0;
-
-    MUTEX_EXIT(&callp->lock);
+#ifdef AFS_FREELANCE_CLIENT
+    if (cm_freelanceEnabled && index == 0) {
+        rc = GetCellCommon(AFS_FAKE_ROOT_CELL_ID, a_name, a_hosts);
+    } else
+#endif
+    {
+        rc = GetCellCommon(index+1, a_name, a_hosts);
+    }
     return 0;
 }
 
@@ -1324,8 +1373,9 @@ int SRXAFSCB_GetLocalCell(struct rx_call *callp, char **a_name)
     unsigned long host = 0;
     unsigned short port = 0;
 
+#ifdef HOLD_CALL_MUTEX
     MUTEX_ENTER(&callp->lock);
-
+#endif
     if ((connp = rx_ConnectionOf(callp)) && (peerp = rx_PeerOf(connp))) {
         host = rx_HostOf(peerp);
         port = rx_PortOf(peerp);
@@ -1343,7 +1393,9 @@ int SRXAFSCB_GetLocalCell(struct rx_call *callp, char **a_name)
     }
     *a_name = t_name;
 
+#ifdef HOLD_CALL_MUTEX
     MUTEX_EXIT(&callp->lock);
+#endif
     return 0;
 }
 
@@ -1423,8 +1475,9 @@ int SRXAFSCB_GetCacheConfig(struct rx_call *callp,
     unsigned long host = 0;
     unsigned short port = 0;
 
+#ifdef HOLD_CALL_MUTEX
     MUTEX_ENTER(&callp->lock);
-
+#endif
     if ((connp = rx_ConnectionOf(callp)) && (peerp = rx_PeerOf(connp))) {
         host = rx_HostOf(peerp);
         port = rx_PortOf(peerp);
@@ -1452,7 +1505,9 @@ int SRXAFSCB_GetCacheConfig(struct rx_call *callp,
     config->cacheConfig_val = t_config;
     config->cacheConfig_len = (*configCount)/sizeof(afs_uint32);
 
+#ifdef HOLD_CALL_MUTEX
     MUTEX_EXIT(&callp->lock);
+#endif
     return 0;
 }
 
@@ -1874,8 +1929,8 @@ void cm_CheckCBExpiration(void)
                 lock_ReleaseMutex(&scp->mx);
                 cm_CallbackNotifyChange(scp);
 
-                cm_ReleaseSCacheNoLock(scp);
                 lock_ObtainWrite(&cm_scacheLock);
+                cm_ReleaseSCacheNoLock(scp);
             }
         }
     }
@@ -1911,9 +1966,10 @@ cm_GiveUpAllCallbacks(cm_server_t *tsp, afs_int32 markDown)
             lock_ObtainMutex(&tsp->mx);
             if (!(tsp->flags & CM_SERVERFLAG_DOWN)) {
                 tsp->flags |= CM_SERVERFLAG_DOWN;
-                tsp->downTime = osi_Time();
+                tsp->downTime = time(NULL);
             }
             cm_ForceNewConnections(tsp);
+
             /* Now update the volume status */
             for (tsrvp = tsp->vols; tsrvp; tsrvp = tsrvp->nextp) {
                 for (i=0; i<NUM_SERVER_VOLS; i++) {
@@ -1923,7 +1979,7 @@ cm_GiveUpAllCallbacks(cm_server_t *tsp, afs_int32 markDown)
                         cm_InitReq(&req);
 
                         code = cm_GetVolumeByID(tsp->cellp, tsrvp->ids[i], cm_rootUserp,
-                                                 &req, CM_GETVOL_FLAG_NO_LRU_UPDATE, &volp);
+                                                 &req, CM_GETVOL_FLAG_NO_LRU_UPDATE | CM_GETVOL_FLAG_NO_RESET, &volp);
                         if (code == 0) {    
                             cm_UpdateVolumeStatus(volp, tsrvp->ids[i]);
                             cm_PutVolume(volp);
