@@ -127,9 +127,9 @@ cm_NameEntries(char *namep, long *lenp)
 {
     long i;
         
-    i = (long)strlen(namep) + 1;
+    i = (long)strlen(namep);
     if (lenp) *lenp = i;
-    return 1 + ((i+15) >> 5);
+    return 1 + ((i+16) >> 5);
 }
 
 /* Create an entry in a file.  Dir is a file representation, while
@@ -1559,10 +1559,7 @@ cm_DirGetPage(cm_dirOp_t * op,
             goto _has_buffer;
         }
 
-        lock_ObtainRead(&op->scp->bufCreateLock);
         code = buf_Get(op->scp, &bufferOffset, &bufferp);
-        lock_ReleaseRead(&op->scp->bufCreateLock);
-
         if (code) {
             osi_Log1(afsd_logp, "    buf_Get returned code 0x%x", code);
             bufferp = NULL;
