@@ -357,10 +357,10 @@ void cm_InitFakeRootDir() {
 
 int cm_FakeRootFid(cm_fid_t *fidp)
 {
-    fidp->cell = AFS_FAKE_ROOT_CELL_ID;            /* root cell */
-    fidp->volume = AFS_FAKE_ROOT_VOL_ID;           /* root.afs ? */
-    fidp->vnode = 0x1;
-    fidp->unique = 0x1;
+    cm_SetFid(fidp, 
+              AFS_FAKE_ROOT_CELL_ID,            /* root cell */
+              AFS_FAKE_ROOT_VOL_ID,            /* root.afs ? */
+              1, 1);
     return 0;
 }
   
@@ -395,10 +395,7 @@ int cm_reInitLocalMountPoints() {
 
     osi_Log0(afsd_logp,"Invalidating local mount point scp...  ");
 
-    aFid.cell = AFS_FAKE_ROOT_CELL_ID;
-    aFid.volume=AFS_FAKE_ROOT_VOL_ID;
-    aFid.unique=0x1;
-    aFid.vnode=0x2;
+    cm_SetFid(&aFid, AFS_FAKE_ROOT_CELL_ID, AFS_FAKE_ROOT_VOL_ID, 1, 2);
 
     lock_ObtainWrite(&cm_scacheLock);
     lock_ObtainMutex(&cm_Freelance_Lock);  /* always scache then freelance lock */
@@ -434,7 +431,7 @@ int cm_reInitLocalMountPoints() {
                 }
             }
         }
-        aFid.vnode = aFid.vnode + 1;
+        cm_SetFid(&aFid, AFS_FAKE_ROOT_CELL_ID, AFS_FAKE_ROOT_VOL_ID, aFid.vnode + 1, 2);
     }
     lock_ReleaseWrite(&cm_scacheLock);
     osi_Log0(afsd_logp,"\tall old scp cleared!");
@@ -477,7 +474,7 @@ long cm_InitLocalMountPoints() {
     char line[512];
     char*t, *t2;
     cm_localMountPoint_t* aLocalMountPoint;
-    char hdir[120];
+    char hdir[260];
     long code;
     char rootCellName[256];
 #if !defined(DJGPP)
@@ -686,7 +683,7 @@ long cm_InitLocalMountPoints() {
     /* What follows is the old code to read freelance mount points 
      * out of a text file modified to copy the data into the registry
      */
-    cm_GetConfigDir(hdir);
+    cm_GetConfigDir(hdir, sizeof(hdir));
     strcat(hdir, AFS_FREELANCE_INI);
     // open the ini file for reading
     fp = fopen(hdir, "r");
@@ -821,7 +818,7 @@ int cm_getNoLocalMountPoints() {
 }
 
 #if !defined(DJGPP)
-long cm_FreelanceMountPointExists(char * filename)
+long cm_FreelanceMountPointExists(char * filename, int prefix_ok)
 {
     char* cp;
     char line[512];
@@ -889,6 +886,11 @@ long cm_FreelanceMountPointExists(char * filename)
                 found = 1;
                 break;
             }
+
+            if (prefix_ok && strlen(shortname) - strlen(filename) == 1 && !strncmp(shortname, filename, strlen(filename))) {
+                found = 1;
+                break;
+            }
         }
         RegCloseKey(hkFreelance);
     }
@@ -898,7 +900,7 @@ long cm_FreelanceMountPointExists(char * filename)
     return found;
 }
 
-long cm_FreelanceSymlinkExists(char * filename)
+long cm_FreelanceSymlinkExists(char * filename, int prefix_ok)
 {
     char* cp;
     char line[512];
@@ -946,6 +948,11 @@ long cm_FreelanceSymlinkExists(char * filename)
                 found = 1;
                 break;
             }
+
+            if (prefix_ok && strlen(shortname) - strlen(filename) == 1 && !strncmp(shortname, filename, strlen(filename))) {
+                found = 1;
+                break;
+            }
         }
         for ( dwIndex = 0; dwIndex < dwSymlinks; dwIndex++ ) {
             TCHAR szValueName[16];
@@ -975,7 +982,7 @@ long cm_FreelanceSymlinkExists(char * filename)
 long cm_FreelanceAddMount(char *filename, char *cellname, char *volume, int rw, cm_fid_t *fidp)
 {
     FILE *fp;
-    char hfile[120];
+    char hfile[260];
     char line[512];
     char fullname[200];
     int n;
@@ -1009,8 +1016,8 @@ long cm_FreelanceAddMount(char *filename, char *cellname, char *volume, int rw,
     }
 
 #if !defined(DJGPP)
-    if ( cm_FreelanceMountPointExists(filename) ||
-         cm_FreelanceSymlinkExists(filename) )
+    if ( cm_FreelanceMountPointExists(filename, 0) ||
+         cm_FreelanceSymlinkExists(filename, 0) )
         return -1;
 #endif
     
@@ -1076,7 +1083,7 @@ long cm_FreelanceAddMount(char *filename, char *cellname, char *volume, int rw,
     } else 
 #endif
     {
-        cm_GetConfigDir(hfile);
+        cm_GetConfigDir(hfile, sizeof(hfile));
         strcat(hfile, AFS_FREELANCE_INI);
         fp = fopen(hfile, "r+");
         if (!fp)
@@ -1096,10 +1103,8 @@ long cm_FreelanceAddMount(char *filename, char *cellname, char *volume, int rw,
     lock_ReleaseMutex(&cm_Freelance_Lock);
 
     /* cm_reInitLocalMountPoints(); */
-    if (fidp) {
-        fidp->unique = 1;
-        fidp->vnode = cm_noLocalMountPoints + 1;   /* vnode value of last mt pt */
-    }
+    if (fidp)
+        cm_SetFid(fidp, fidp->cell, fidp->volume, cm_noLocalMountPoints + 1, 1);
     cm_noteLocalMountPointChange();
     return 0;
 }
@@ -1110,7 +1115,7 @@ long cm_FreelanceRemoveMount(char *toremove)
     char* cp;
     char line[512];
     char shortname[200];
-    char hfile[120], hfile2[120];
+    char hfile[260], hfile2[260];
     FILE *fp1, *fp2;
     int found=0;
 #if !defined(DJGPP)
@@ -1165,7 +1170,7 @@ long cm_FreelanceRemoveMount(char *toremove)
     } else 
 #endif
     {
-        cm_GetConfigDir(hfile);
+        cm_GetConfigDir(hfile, sizeof(hfile));
         strcat(hfile, AFS_FREELANCE_INI);
         strcpy(hfile2, hfile);
         strcat(hfile2, "2");
@@ -1246,8 +1251,8 @@ long cm_FreelanceAddSymlink(char *filename, char *destination, cm_fid_t *fidp)
     }
 
 #if !defined(DJGPP)
-    if ( cm_FreelanceMountPointExists(filename) ||
-         cm_FreelanceSymlinkExists(filename) )
+    if ( cm_FreelanceMountPointExists(filename, 0) ||
+         cm_FreelanceSymlinkExists(filename, 0) )
         return CM_ERROR_EXISTS;
 #endif
 
@@ -1312,10 +1317,8 @@ long cm_FreelanceAddSymlink(char *filename, char *destination, cm_fid_t *fidp)
     lock_ReleaseMutex(&cm_Freelance_Lock);
 
     /* cm_reInitLocalMountPoints(); */
-    if (fidp) {
-        fidp->unique = 1;
-        fidp->vnode = cm_noLocalMountPoints + 1;   /* vnode value of last mt pt */
-    }
+    if (fidp)
+        cm_SetFid(fidp, fidp->cell, fidp->volume, cm_noLocalMountPoints + 1, 1);
     cm_noteLocalMountPointChange();
     return 0;
 }
