@@ -6,7 +6,6 @@ typedef struct cm_localMountPoint {
     char*                       namep;
     char*                       mountPointStringp;
     unsigned int                fileType;
-    struct cm_localMountPoint*  next;
 } cm_localMountPoint_t;
 
 extern int cm_getNoLocalMountPoints();
@@ -14,10 +13,14 @@ extern long cm_InitLocalMountPoints();
 extern int cm_getLocalMountPointChange();
 extern int cm_reInitLocalMountPoints();
 extern void cm_InitFreelance();
+extern void cm_FreelanceShutdown(void);
+extern int cm_noteLocalMountPointChange(void);
 extern long cm_FreelanceRemoveMount(char *toremove);
 extern long cm_FreelanceAddMount(char *filename, char *cellname, char *volume, int rw, cm_fid_t *fidp);
 extern long cm_FreelanceRemoveSymlink(char *toremove);
 extern long cm_FreelanceAddSymlink(char *filename, char *destination, cm_fid_t *fidp);
+extern long cm_FreelanceMountPointExists(char * filename, int prefix_ok);
+extern long cm_FreelanceSymlinkExists(char * filename, int prefix_ok);
 extern int cm_clearLocalMountPointChange();
 extern int cm_FakeRootFid(cm_fid_t *fidp);
 
