@@ -71,14 +71,12 @@ long cm_CleanFile(cm_scache_t *scp, cm_user_t *userp, cm_req_t *reqp)
 {
     long code;
 
-    lock_ObtainWrite(&scp->bufCreateLock);
     code = buf_CleanVnode(scp, userp, reqp);
         
     lock_ObtainMutex(&scp->mx);
     cm_DiscardSCache(scp);
     lock_ReleaseMutex(&scp->mx);
 
-    lock_ReleaseWrite(&scp->bufCreateLock);
     osi_Log2(afsd_logp,"cm_CleanFile scp 0x%x returns error: [%x]",scp, code);
     return code;
 }
@@ -94,7 +92,6 @@ long cm_FlushFile(cm_scache_t *scp, cm_user_t *userp, cm_req_t *reqp)
     }
 #endif
 
-    lock_ObtainWrite(&scp->bufCreateLock);
     code = buf_FlushCleanPages(scp, userp, reqp);
         
     lock_ObtainMutex(&scp->mx);
@@ -102,7 +99,6 @@ long cm_FlushFile(cm_scache_t *scp, cm_user_t *userp, cm_req_t *reqp)
 
     lock_ReleaseMutex(&scp->mx);
 
-    lock_ReleaseWrite(&scp->bufCreateLock);
     osi_Log2(afsd_logp,"cm_FlushFile scp 0x%x returns error: [%x]",scp, code);
     return code;
 }
@@ -211,8 +207,10 @@ void TranslateExtendedChars(char *str)
 /* parse the passed-in file name and do a namei on it.  If we fail,
  * return an error code, otherwise return the vnode located in *scpp.
  */
+#define CM_PARSE_FLAG_LITERAL 1
+
 long cm_ParseIoctlPath(smb_ioctl_t *ioctlp, cm_user_t *userp, cm_req_t *reqp,
-	cm_scache_t **scpp)
+	cm_scache_t **scpp, afs_uint32 flags)
 {
     long code;
 #ifndef AFSIFS
@@ -221,7 +219,7 @@ long cm_ParseIoctlPath(smb_ioctl_t *ioctlp, cm_user_t *userp, cm_req_t *reqp,
 #endif
     char * relativePath;
     char * lastComponent = NULL;
-    afs_uint32 follow = 0;
+    afs_uint32 follow = (flags & CM_PARSE_FLAG_LITERAL ? CM_FLAG_NOMOUNTCHASE : CM_FLAG_FOLLOW);
 
     relativePath = ioctlp->inDatap;
     /* setup the next data value for the caller to use */
@@ -236,10 +234,16 @@ long cm_ParseIoctlPath(smb_ioctl_t *ioctlp, cm_user_t *userp, cm_req_t *reqp,
      *    \\netbios-name\submount\path\.
      *    \\netbios-name\submount\path\file
      */
-    TranslateExtendedChars(relativePath);
+
+	/* We do not perform path name translation on the ioctl path data 
+	 * because these paths were not translated by Windows through the
+	 * file system API.  Therefore, they are not OEM characters but 
+	 * whatever the display character set is.
+	 */
+    // TranslateExtendedChars(relativePath);
 
     /* This is usually nothing, but for StatMountPoint it is the file name. */
-    TranslateExtendedChars(ioctlp->inDatap);
+    // TranslateExtendedChars(ioctlp->inDatap);
 
 #ifdef AFSIFS
     /* we have passed the whole path, including the afs prefix.
@@ -247,7 +251,7 @@ long cm_ParseIoctlPath(smb_ioctl_t *ioctlp, cm_user_t *userp, cm_req_t *reqp,
        and it returns the correct (full) path.  therefore, there is
        no drive letter, and the path is absolute. */
     code = cm_NameI(cm_data.rootSCachep, relativePath,
-                     CM_FLAG_CASEFOLD | follow,
+                     CM_FLAG_CASEFOLD,
                      userp, "", reqp, scpp);
 
     if (code) {
@@ -300,7 +304,7 @@ long cm_ParseIoctlPath(smb_ioctl_t *ioctlp, cm_user_t *userp, cm_req_t *reqp,
 		code = cm_NameI(substRootp, p, CM_FLAG_CASEFOLD | CM_FLAG_FOLLOW,
 				 userp, NULL, reqp, &iscp);
 		if (code == 0)
-		    code = cm_NameI(iscp, lastComponent, CM_FLAG_CASEFOLD | CM_FLAG_NOMOUNTCHASE,
+		    code = cm_NameI(iscp, lastComponent, CM_FLAG_CASEFOLD | follow,
 				    userp, NULL, reqp, scpp);
 		if (iscp)
 		    cm_ReleaseSCache(iscp);
@@ -347,7 +351,7 @@ long cm_ParseIoctlPath(smb_ioctl_t *ioctlp, cm_user_t *userp, cm_req_t *reqp,
 		code = cm_NameI(substRootp, p, CM_FLAG_CASEFOLD | CM_FLAG_FOLLOW,
 				 userp, NULL, reqp, &iscp);
 		if (code == 0)
-		    code = cm_NameI(iscp, lastComponent, CM_FLAG_CASEFOLD | CM_FLAG_NOMOUNTCHASE,
+		    code = cm_NameI(iscp, lastComponent, CM_FLAG_CASEFOLD | follow,
 				    userp, NULL, reqp, scpp);
 		if (iscp)
 		    cm_ReleaseSCache(iscp);
@@ -379,12 +383,12 @@ long cm_ParseIoctlPath(smb_ioctl_t *ioctlp, cm_user_t *userp, cm_req_t *reqp,
 	    code = cm_NameI(substRootp, relativePath, CM_FLAG_CASEFOLD | CM_FLAG_FOLLOW,
 			     userp, NULL, reqp, &iscp);
 	    if (code == 0)
-		code = cm_NameI(iscp, lastComponent, CM_FLAG_CASEFOLD | CM_FLAG_NOMOUNTCHASE,
+		code = cm_NameI(iscp, lastComponent, CM_FLAG_CASEFOLD | follow,
 				 userp, NULL, reqp, scpp);
 	    if (iscp)
 		cm_ReleaseSCache(iscp);
 	} else {
-	    code = cm_NameI(substRootp, relativePath, CM_FLAG_CASEFOLD,
+	    code = cm_NameI(substRootp, relativePath, CM_FLAG_CASEFOLD | follow,
 			     userp, NULL, reqp, scpp);
 	}
         if (code) {
@@ -405,12 +409,42 @@ long cm_ParseIoctlPath(smb_ioctl_t *ioctlp, cm_user_t *userp, cm_req_t *reqp,
 
 void cm_SkipIoctlPath(smb_ioctl_t *ioctlp)
 {
-    long temp;
+    size_t temp;
         
-    temp = (long) strlen(ioctlp->inDatap) + 1;
+    temp = strlen(ioctlp->inDatap) + 1;
     ioctlp->inDatap += temp;
 }       
 
+/* 
+ * Must be called before cm_ParseIoctlPath or cm_SkipIoctlPath 
+ */
+static cm_ioctlQueryOptions_t * 
+cm_IoctlGetQueryOptions(struct smb_ioctl *ioctlp, struct cm_user *userp)
+{
+    afs_uint32 pathlen = strlen(ioctlp->inDatap) + 1;
+    char *p = ioctlp->inDatap + pathlen;
+    cm_ioctlQueryOptions_t * optionsp = NULL;
+
+    if (ioctlp->inCopied > p - ioctlp->inAllocp) {
+        optionsp = (cm_ioctlQueryOptions_t *)p;
+        if (optionsp->size < 12 /* minimum size of struct */)
+            optionsp = NULL;
+    }
+
+    return optionsp;
+}
+
+/* 
+ * Must be called after cm_ParseIoctlPath or cm_SkipIoctlPath
+ * or any other time that ioctlp->inDatap points at the 
+ * cm_ioctlQueryOptions_t object.
+ */
+static void
+cm_IoctlSkipQueryOptions(struct smb_ioctl *ioctlp, struct cm_user *userp)
+{
+    cm_ioctlQueryOptions_t * optionsp = (cm_ioctlQueryOptions_t *)ioctlp->inDatap;
+    ioctlp->inDatap += optionsp->size;
+}
 
 /* format the specified path to look like "/afs/<cellname>/usr", by
  * adding "/afs" (if necessary) in front, changing any \'s to /'s, and
@@ -576,11 +610,24 @@ long cm_IoctlGetACL(smb_ioctl_t *ioctlp, cm_user_t *userp)
     int tlen;
     cm_req_t req;
     struct rx_connection * callp;
+    cm_ioctlQueryOptions_t *optionsp;
+    afs_uint32 flags = 0;
 
     cm_InitReq(&req);
 
-    code = cm_ParseIoctlPath(ioctlp, userp, &req, &scp);
-    if (code) return code;
+    optionsp = cm_IoctlGetQueryOptions(ioctlp, userp);
+    if (optionsp && CM_IOCTL_QOPTS_HAVE_LITERAL(optionsp))
+        flags |= (optionsp->literal ? CM_PARSE_FLAG_LITERAL : 0);
+
+    if (optionsp && CM_IOCTL_QOPTS_HAVE_FID(optionsp)) {
+        cm_SkipIoctlPath(ioctlp);
+
+        code = cm_GetSCache(&optionsp->fid, &scp, userp, &req);
+    } else {
+        code = cm_ParseIoctlPath(ioctlp, userp, &req, &scp, flags);
+    }
+    if (code) 
+        return code;
 
     /* now make the get acl call */
 #ifdef AFS_FREELANCE_CLIENT
@@ -623,11 +670,23 @@ long cm_IoctlGetFileCellName(struct smb_ioctl *ioctlp, struct cm_user *userp)
     cm_scache_t *scp;
     cm_cell_t *cellp;
     cm_req_t req;
+    cm_ioctlQueryOptions_t *optionsp;
+    afs_uint32 flags = 0;
 
     cm_InitReq(&req);
 
-    code = cm_ParseIoctlPath(ioctlp, userp, &req, &scp);
-    if (code) return code;
+    optionsp = cm_IoctlGetQueryOptions(ioctlp, userp);
+    if (optionsp && CM_IOCTL_QOPTS_HAVE_LITERAL(optionsp))
+        flags |= (optionsp->literal ? CM_PARSE_FLAG_LITERAL : 0);
+
+    if (optionsp && CM_IOCTL_QOPTS_HAVE_FID(optionsp)) {
+        cm_SkipIoctlPath(ioctlp);
+        code = cm_GetSCache(&optionsp->fid, &scp, userp, &req);
+    } else {
+        code = cm_ParseIoctlPath(ioctlp, userp, &req, &scp, flags);
+    }
+    if (code) 
+        return code;
 
 #ifdef AFS_FREELANCE_CLIENT
     if ( cm_freelanceEnabled && 
@@ -640,7 +699,7 @@ long cm_IoctlGetFileCellName(struct smb_ioctl *ioctlp, struct cm_user *userp)
     } else 
 #endif /* AFS_FREELANCE_CLIENT */
     {
-        cellp = cm_FindCellByID(scp->fid.cell);
+        cellp = cm_FindCellByID(scp->fid.cell, CM_FLAG_NOPROBE);
         if (cellp) {
             StringCbCopyA(ioctlp->outDatap, SMB_IOCTL_MAXDATA - (ioctlp->outDatap - ioctlp->outAllocp), cellp->name);
             ioctlp->outDatap += strlen(ioctlp->outDatap) + 1;
@@ -668,7 +727,7 @@ long cm_IoctlSetACL(struct smb_ioctl *ioctlp, struct cm_user *userp)
 
     cm_InitReq(&req);
 
-    code = cm_ParseIoctlPath(ioctlp, userp, &req, &scp);
+    code = cm_ParseIoctlPath(ioctlp, userp, &req, &scp, 0);
     if (code) return code;
 	
 #ifdef AFS_FREELANCE_CLIENT
@@ -715,6 +774,8 @@ long cm_IoctlFlushAllVolumes(struct smb_ioctl *ioctlp, struct cm_user *userp)
 
     cm_InitReq(&req);
 
+    cm_SkipIoctlPath(ioctlp);	/* we don't care about the path */
+
     lock_ObtainWrite(&cm_scacheLock);
     for (i=0; i<cm_data.scacheHashTableSize; i++) {
         for (scp = cm_data.scacheHashTablep[i]; scp; scp = scp->nextp) {
@@ -739,12 +800,24 @@ long cm_IoctlFlushVolume(struct smb_ioctl *ioctlp, struct cm_user *userp)
     unsigned long volume;
     unsigned long cell;
     cm_req_t req;
+    cm_ioctlQueryOptions_t *optionsp;
+    afs_uint32 flags = 0;
 
     cm_InitReq(&req);
 
-    code = cm_ParseIoctlPath(ioctlp, userp, &req, &scp);
-    if (code) return code;
-        
+    optionsp = cm_IoctlGetQueryOptions(ioctlp, userp);
+    if (optionsp && CM_IOCTL_QOPTS_HAVE_LITERAL(optionsp))
+        flags |= (optionsp->literal ? CM_PARSE_FLAG_LITERAL : 0);
+
+    if (optionsp && CM_IOCTL_QOPTS_HAVE_FID(optionsp)) {
+        cm_SkipIoctlPath(ioctlp);
+        code = cm_GetSCache(&optionsp->fid, &scp, userp, &req);
+    } else {
+        code = cm_ParseIoctlPath(ioctlp, userp, &req, &scp, flags);
+    }
+    if (code) 
+        return code;
+
 #ifdef AFS_FREELANCE_CLIENT
     if ( scp->fid.cell == AFS_FAKE_ROOT_CELL_ID && scp->fid.volume == AFS_FAKE_ROOT_VOL_ID ) {
 	code = CM_ERROR_NOACCESS;
@@ -765,12 +838,24 @@ long cm_IoctlFlushFile(struct smb_ioctl *ioctlp, struct cm_user *userp)
     long code;
     cm_scache_t *scp;
     cm_req_t req;
+    cm_ioctlQueryOptions_t *optionsp;
+    afs_uint32 flags = 0;
 
     cm_InitReq(&req);
 
-    code = cm_ParseIoctlPath(ioctlp, userp, &req, &scp);
-    if (code) return code;
-        
+    optionsp = cm_IoctlGetQueryOptions(ioctlp, userp);
+    if (optionsp && CM_IOCTL_QOPTS_HAVE_LITERAL(optionsp))
+        flags |= (optionsp->literal ? CM_PARSE_FLAG_LITERAL : 0);
+
+    if (optionsp && CM_IOCTL_QOPTS_HAVE_FID(optionsp)) {
+        cm_SkipIoctlPath(ioctlp);
+        code = cm_GetSCache(&optionsp->fid, &scp, userp, &req);
+    } else {
+        code = cm_ParseIoctlPath(ioctlp, userp, &req, &scp, flags);
+    }
+    if (code) 
+        return code;
+
 #ifdef AFS_FREELANCE_CLIENT
     if ( scp->fid.cell == AFS_FAKE_ROOT_CELL_ID && scp->fid.volume == AFS_FAKE_ROOT_VOL_ID ) {
 	code = CM_ERROR_NOACCESS;
@@ -802,7 +887,7 @@ long cm_IoctlSetVolumeStatus(struct smb_ioctl *ioctlp, struct cm_user *userp)
 
     cm_InitReq(&req);
 
-    code = cm_ParseIoctlPath(ioctlp, userp, &req, &scp);
+    code = cm_ParseIoctlPath(ioctlp, userp, &req, &scp, 0);
     if (code) return code;
 
 #ifdef AFS_FREELANCE_CLIENT
@@ -811,7 +896,7 @@ long cm_IoctlSetVolumeStatus(struct smb_ioctl *ioctlp, struct cm_user *userp)
     } else
 #endif
     {
-        cellp = cm_FindCellByID(scp->fid.cell);
+        cellp = cm_FindCellByID(scp->fid.cell, 0);
         osi_assertx(cellp, "null cm_cell_t");
 
         if (scp->flags & CM_SCACHEFLAG_RO) {
@@ -900,11 +985,23 @@ long cm_IoctlGetVolumeStatus(struct smb_ioctl *ioctlp, struct cm_user *userp)
     char *MOTD;
     cm_req_t req;
     struct rx_connection * callp;
+    cm_ioctlQueryOptions_t *optionsp;
+    afs_uint32 flags = 0;
 
     cm_InitReq(&req);
 
-    code = cm_ParseIoctlPath(ioctlp, userp, &req, &scp);
-    if (code) return code;
+    optionsp = cm_IoctlGetQueryOptions(ioctlp, userp);
+    if (optionsp && CM_IOCTL_QOPTS_HAVE_LITERAL(optionsp))
+        flags |= (optionsp->literal ? CM_PARSE_FLAG_LITERAL : 0);
+
+    if (optionsp && CM_IOCTL_QOPTS_HAVE_FID(optionsp)) {
+        cm_SkipIoctlPath(ioctlp);
+        code = cm_GetSCache(&optionsp->fid, &scp, userp, &req);
+    } else {
+        code = cm_ParseIoctlPath(ioctlp, userp, &req, &scp, flags);
+    }
+    if (code) 
+        return code;
 
 #ifdef AFS_FREELANCE_CLIENT
     if ( scp->fid.cell == AFS_FAKE_ROOT_CELL_ID && scp->fid.volume == AFS_FAKE_ROOT_VOL_ID ) {
@@ -963,13 +1060,20 @@ long cm_IoctlGetFid(struct smb_ioctl *ioctlp, struct cm_user *userp)
     register char *cp;
     cm_fid_t fid;
     cm_req_t req;
+    cm_ioctlQueryOptions_t * optionsp;
+    afs_uint32 flags = 0;
 
     cm_InitReq(&req);
 
-    code = cm_ParseIoctlPath(ioctlp, userp, &req, &scp);
+    optionsp = cm_IoctlGetQueryOptions(ioctlp, userp);
+    if (optionsp && CM_IOCTL_QOPTS_HAVE_LITERAL(optionsp))
+        flags |= (optionsp->literal ? CM_PARSE_FLAG_LITERAL : 0);
+
+    code = cm_ParseIoctlPath(ioctlp, userp, &req, &scp, flags);
     if (code) return code;
 
     memset(&fid, 0, sizeof(cm_fid_t));
+    fid.cell   = scp->fid.cell;
     fid.volume = scp->fid.volume;
     fid.vnode  = scp->fid.vnode;
     fid.unique = scp->fid.unique;
@@ -987,17 +1091,68 @@ long cm_IoctlGetFid(struct smb_ioctl *ioctlp, struct cm_user *userp)
     return 0;
 }
 
+long cm_IoctlGetFileType(struct smb_ioctl *ioctlp, struct cm_user *userp)
+{
+    cm_scache_t *scp;
+    register long code;
+    register char *cp;
+    afs_uint32 fileType = 0;
+    cm_req_t req;
+    cm_ioctlQueryOptions_t * optionsp;
+    afs_uint32 flags = 0;
+
+    cm_InitReq(&req);
+
+    optionsp = cm_IoctlGetQueryOptions(ioctlp, userp);
+    if (optionsp && CM_IOCTL_QOPTS_HAVE_LITERAL(optionsp))
+        flags |= (optionsp->literal ? CM_PARSE_FLAG_LITERAL : 0);
+
+    if (optionsp && CM_IOCTL_QOPTS_HAVE_FID(optionsp)) {
+        cm_SkipIoctlPath(ioctlp);
+        code = cm_GetSCache(&optionsp->fid, &scp, userp, &req);
+    } else {
+        code = cm_ParseIoctlPath(ioctlp, userp, &req, &scp, flags);
+    }
+    if (code) 
+        return code;
+
+    fileType = scp->fileType;
+    cm_ReleaseSCache(scp);
+
+    /* Copy all this junk into msg->im_data, keeping track of the lengths. */
+    cp = ioctlp->outDatap;
+    memcpy(cp, (char *)&fileType, sizeof(fileType));
+    cp += sizeof(fileType);
+
+    /* return new size */
+    ioctlp->outDatap = cp;
+
+    return 0;
+}
+
 long cm_IoctlGetOwner(struct smb_ioctl *ioctlp, struct cm_user *userp)
 {
     cm_scache_t *scp;
     register long code;
     register char *cp;
     cm_req_t req;
+    cm_ioctlQueryOptions_t *optionsp;
+    afs_uint32 flags = 0;
 
     cm_InitReq(&req);
 
-    code = cm_ParseIoctlPath(ioctlp, userp, &req, &scp);
-    if (code) return code;
+    optionsp = cm_IoctlGetQueryOptions(ioctlp, userp);
+    if (optionsp && CM_IOCTL_QOPTS_HAVE_LITERAL(optionsp))
+        flags |= (optionsp->literal ? CM_PARSE_FLAG_LITERAL : 0);
+
+    if (optionsp && CM_IOCTL_QOPTS_HAVE_FID(optionsp)) {
+        cm_SkipIoctlPath(ioctlp);
+        code = cm_GetSCache(&optionsp->fid, &scp, userp, &req);
+    } else {
+        code = cm_ParseIoctlPath(ioctlp, userp, &req, &scp, flags);
+    }
+    if (code) 
+        return code;
 
     /* Copy all this junk into msg->im_data, keeping track of the lengths. */
     cp = ioctlp->outDatap;
@@ -1025,46 +1180,82 @@ long cm_IoctlWhereIs(struct smb_ioctl *ioctlp, struct cm_user *userp)
     unsigned long volume;
     char *cp;
     cm_req_t req;
+    cm_ioctlQueryOptions_t *optionsp;
+    afs_uint32 flags = 0;
 
     cm_InitReq(&req);
 
-    code = cm_ParseIoctlPath(ioctlp, userp, &req, &scp);
-    if (code) return code;
-        
+    optionsp = cm_IoctlGetQueryOptions(ioctlp, userp);
+    if (optionsp && CM_IOCTL_QOPTS_HAVE_LITERAL(optionsp))
+        flags |= (optionsp->literal ? CM_PARSE_FLAG_LITERAL : 0);
+
+    if (optionsp && CM_IOCTL_QOPTS_HAVE_FID(optionsp)) {
+        cm_SkipIoctlPath(ioctlp);
+        code = cm_GetSCache(&optionsp->fid, &scp, userp, &req);
+    } else {
+        code = cm_ParseIoctlPath(ioctlp, userp, &req, &scp, flags);
+    }
+    if (code) 
+        return code;
+
     volume = scp->fid.volume;
 
-    cellp = cm_FindCellByID(scp->fid.cell);
+    cellp = cm_FindCellByID(scp->fid.cell, 0);
 
     cm_ReleaseSCache(scp);
 
     if (!cellp)
 	return CM_ERROR_NOSUCHCELL;
 
-    code = cm_GetVolumeByID(cellp, volume, userp, &req, CM_GETVOL_FLAG_CREATE, &tvp);
-    if (code) 
-        return code;
-	
-    cp = ioctlp->outDatap;
+#ifdef AFS_FREELANCE_CLIENT
+    if ( cellp->cellID == AFS_FAKE_ROOT_CELL_ID) {
+        struct in_addr addr;
+
+        addr.s_net = 127;
+        addr.s_host = 0;
+        addr.s_lh = 0;
+        addr.s_impno = 1;
+
+        cp = ioctlp->outDatap;
         
-    lock_ObtainMutex(&tvp->mx);
-    tsrpp = cm_GetVolServers(tvp, volume);
-    lock_ObtainRead(&cm_serverLock);
-    for (current = *tsrpp; current; current = current->next) {
-        tsp = current->server;
-        memcpy(cp, (char *)&tsp->addr.sin_addr.s_addr, sizeof(long));
+        memcpy(cp, (char *)&addr, sizeof(addr));
+        cp += sizeof(addr);
+
+        /* still room for terminating NULL, add it on */
+        addr.s_addr = 0;
+        memcpy(cp, (char *)&addr, sizeof(addr));
+        cp += sizeof(addr);
+
+        ioctlp->outDatap = cp;
+    } else 
+#endif
+    {
+        code = cm_GetVolumeByID(cellp, volume, userp, &req, CM_GETVOL_FLAG_CREATE, &tvp);
+        if (code) 
+            return code;
+	
+        cp = ioctlp->outDatap;
+        
+        lock_ObtainMutex(&tvp->mx);
+        tsrpp = cm_GetVolServers(tvp, volume);
+        lock_ObtainRead(&cm_serverLock);
+        for (current = *tsrpp; current; current = current->next) {
+            tsp = current->server;
+            memcpy(cp, (char *)&tsp->addr.sin_addr.s_addr, sizeof(long));
+            cp += sizeof(long);
+        }
+        lock_ReleaseRead(&cm_serverLock);
+        cm_FreeServerList(tsrpp, 0);
+        lock_ReleaseMutex(&tvp->mx);
+
+        /* still room for terminating NULL, add it on */
+        volume = 0;	/* reuse vbl */
+        memcpy(cp, (char *)&volume, sizeof(long));
         cp += sizeof(long);
+
+        ioctlp->outDatap = cp;
+        cm_PutVolume(tvp);
     }
-    lock_ReleaseRead(&cm_serverLock);
-    cm_FreeServerList(tsrpp, 0);
-    lock_ReleaseMutex(&tvp->mx);
-
-    /* still room for terminating NULL, add it on */
-    volume = 0;	/* reuse vbl */
-    memcpy(cp, (char *)&volume, sizeof(long));
-    cp += sizeof(long);
-
-    ioctlp->outDatap = cp;
-    cm_PutVolume(tvp);
     return 0;
 }       
 
@@ -1078,7 +1269,7 @@ long cm_IoctlStatMountPoint(struct smb_ioctl *ioctlp, struct cm_user *userp)
 
     cm_InitReq(&req);
 
-    code = cm_ParseIoctlPath(ioctlp, userp, &req, &dscp);
+    code = cm_ParseIoctlPath(ioctlp, userp, &req, &dscp, 0);
     if (code) return code;
 
     cp = ioctlp->inDatap;
@@ -1119,7 +1310,7 @@ long cm_IoctlDeleteMountPoint(struct smb_ioctl *ioctlp, struct cm_user *userp)
 
     cm_InitReq(&req);
 
-    code = cm_ParseIoctlPath(ioctlp, userp, &req, &dscp);
+    code = cm_ParseIoctlPath(ioctlp, userp, &req, &dscp, 0);
     if (code) return code;
 
     cp = ioctlp->inDatap;
@@ -1214,14 +1405,14 @@ long cm_IoctlCheckServers(struct smb_ioctl *ioctlp, struct cm_user *userp)
      */
     if (haveCell) {
         /* have cell name, too */
-        cellp = cm_GetCell(cp, 0);
+        cellp = cm_GetCell(cp, (temp & 1) ? CM_FLAG_NOPROBE : 0);
         if (!cellp) 
             return CM_ERROR_NOSUCHCELL;
     }
     else cellp = (cm_cell_t *) 0;
     if (!cellp && (temp & 2)) {
         /* use local cell */
-        cellp = cm_FindCellByID(1);
+        cellp = cm_FindCellByID(1, 0);
     }
     if (!(temp & 1)) {	/* if not fast, call server checker routine */
         /* check down servers */
@@ -1403,8 +1594,6 @@ long cm_IoctlGetCell(struct smb_ioctl *ioctlp, struct cm_user *userp)
         return CM_ERROR_NOMORETOKENS;	/* mapped to EDOM */
 }
 
-extern long cm_AddCellProc(void *rockp, struct sockaddr_in *addrp, char *namep);
-
 long cm_IoctlNewCell(struct smb_ioctl *ioctlp, struct cm_user *userp)
 {
     /* NT cache manager will read cell information from CellServDB each time
@@ -1417,6 +1606,8 @@ long cm_IoctlNewCell(struct smb_ioctl *ioctlp, struct cm_user *userp)
      */  
   
     cm_cell_t *cp;
+    cm_cell_rock_t rock;
+
   
     cm_SkipIoctlPath(ioctlp);
     lock_ObtainWrite(&cm_cellLock);
@@ -1428,12 +1619,14 @@ long cm_IoctlNewCell(struct smb_ioctl *ioctlp, struct cm_user *userp)
         /* delete all previous server lists - cm_FreeServerList will ask for write on cm_ServerLock*/
         cm_FreeServerList(&cp->vlServersp, CM_FREESERVERLIST_DELETE);
         cp->vlServersp = NULL;
-        code = cm_SearchCellFile(cp->name, cp->name, cm_AddCellProc, cp);
+        rock.cellp = cp;
+        rock.flags = 0;
+        code = cm_SearchCellFile(cp->name, cp->name, cm_AddCellProc, &rock);
 #ifdef AFS_AFSDB_ENV
         if (code) {
             if (cm_dnsEnabled) {
                 int ttl;
-                code = cm_SearchCellByDNS(cp->name, cp->name, &ttl, cm_AddCellProc, cp);
+                code = cm_SearchCellByDNS(cp->name, cp->name, &ttl, cm_AddCellProc, &rock);
                 if ( code == 0 ) { /* got cell from DNS */
                     cp->flags |= CM_CELLFLAG_DNS;
                     cp->flags &= ~CM_CELLFLAG_VLSERVER_INVALID;
@@ -1666,7 +1859,7 @@ long cm_IoctlSetSPrefs(struct smb_ioctl *ioctlp, struct cm_user *userp)
         }
         else	/* add a new server without a cell */
         {
-            tsp = cm_NewServer(&tmp, type, NULL); /* refcount = 1 */
+            tsp = cm_NewServer(&tmp, type, NULL, CM_FLAG_NOPROBE); /* refcount = 1 */
             tsp->ipRank = rank;
         }
 	lock_ObtainMutex(&tsp->mx);
@@ -1885,7 +2078,7 @@ long cm_IoctlListlink(struct smb_ioctl *ioctlp, struct cm_user *userp)
 
     cm_InitReq(&req);
 
-    code = cm_ParseIoctlPath(ioctlp, userp, &req, &dscp);
+    code = cm_ParseIoctlPath(ioctlp, userp, &req, &dscp, 0);
     if (code) return code;
 
     cp = ioctlp->inDatap;
@@ -1943,7 +2136,7 @@ long cm_IoctlIslink(struct smb_ioctl *ioctlp, struct cm_user *userp)
 
     cm_InitReq(&req);
 
-    code = cm_ParseIoctlPath(ioctlp, userp, &req, &dscp);
+    code = cm_ParseIoctlPath(ioctlp, userp, &req, &dscp, 0);
     if (code) return code;
 
     cp = ioctlp->inDatap;
@@ -1972,7 +2165,7 @@ long cm_IoctlDeletelink(struct smb_ioctl *ioctlp, struct cm_user *userp)
 
     cm_InitReq(&req);
 
-    code = cm_ParseIoctlPath(ioctlp, userp, &req, &dscp);
+    code = cm_ParseIoctlPath(ioctlp, userp, &req, &dscp, 0);
     if (code) return code;
 
     cp = ioctlp->inDatap;
@@ -2164,7 +2357,7 @@ long cm_IoctlSetToken(struct smb_ioctl *ioctlp, struct cm_user *userp)
         tp += sizeof(int);
 
         /* cell name */
-        cellp = cm_GetCell(tp, CM_FLAG_CREATE);
+        cellp = cm_GetCell(tp, CM_FLAG_CREATE | CM_FLAG_NOPROBE);
         if (!cellp) 
             return CM_ERROR_NOSUCHCELL;
         tp += strlen(tp) + 1;
@@ -2934,10 +3127,21 @@ long cm_IoctlPathAvailability(struct smb_ioctl *ioctlp, struct cm_user *userp)
     cm_vol_state_t *statep;
     afs_uint32 volume;
     cm_req_t req;
+    cm_ioctlQueryOptions_t *optionsp;
+    afs_uint32 flags = 0;
 
     cm_InitReq(&req);
 
-    code = cm_ParseIoctlPath(ioctlp, userp, &req, &scp);
+    optionsp = cm_IoctlGetQueryOptions(ioctlp, userp);
+    if (optionsp && CM_IOCTL_QOPTS_HAVE_LITERAL(optionsp))
+        flags |= (optionsp->literal ? CM_PARSE_FLAG_LITERAL : 0);
+
+    if (optionsp && CM_IOCTL_QOPTS_HAVE_FID(optionsp)) {
+        cm_SkipIoctlPath(ioctlp);
+        code = cm_GetSCache(&optionsp->fid, &scp, userp, &req);
+    } else {
+        code = cm_ParseIoctlPath(ioctlp, userp, &req, &scp, flags);
+    }
     if (code) 
         return code;
         
@@ -2950,7 +3154,7 @@ long cm_IoctlPathAvailability(struct smb_ioctl *ioctlp, struct cm_user *userp)
     {
         volume = scp->fid.volume;
 
-        cellp = cm_FindCellByID(scp->fid.cell);
+        cellp = cm_FindCellByID(scp->fid.cell, 0);
 
         cm_ReleaseSCache(scp);
 
@@ -3042,7 +3246,7 @@ long cm_IoctlVolStatTest(struct smb_ioctl *ioctlp, struct cm_user *userp)
     }
 
     if (testp->fid.cell > 0) {
-        cellp = cm_FindCellByID(testp->fid.cell);
+        cellp = cm_FindCellByID(testp->fid.cell, 0);
     }
 
     if (!cellp)
