@@ -10,7 +10,8 @@
 #ifndef CM_MEMMAP_H
 #define CM_MEMMAP_H 1
 
-#define CM_CONFIG_DATA_MAGIC            ('A' | 'F'<<8 | 'S'<<16 | 'D'<<24)
+#define CM_CONFIG_DATA_VERSION  1
+#define CM_CONFIG_DATA_MAGIC            ('A' | 'F'<<8 | 'S'<<16 | CM_CONFIG_DATA_VERSION<<24)
 
 typedef struct cm_config_data {
     afs_uint32          size;
@@ -46,7 +47,7 @@ typedef struct cm_config_data {
     cm_fid_t            rootFid;
     cm_scache_t *       rootSCachep;
     cm_scache_t         fakeSCache;
-    afs_uint32          fakeDirVersion;
+    afs_uint64          fakeDirVersion;
 
     cm_aclent_t *       aclLRUp;
     cm_aclent_t	*       aclLRUEndp;
