@@ -17,6 +17,13 @@
 #endif /* !DJGPP */
 #include <osi.h>
 
+/* this value is set to 1022 in order to  */
+#define NUM_SERVER_VOLS         (32 - sizeof(void *) / 4)
+typedef struct cm_server_vols {
+    afs_uint32             ids[NUM_SERVER_VOLS];
+    struct cm_server_vols *nextp;
+} cm_server_vols_t;
+
 /* pointed to by volumes and cells without holds; cm_serverLock is obtained
  * at the appropriate times to change the pointers to these servers.
  */
@@ -32,15 +39,18 @@ typedef struct cm_server {
     unsigned long refCount;		/* locked by cm_serverLock */
     osi_mutex_t mx;
     unsigned short ipRank;		/* server priority */
+    cm_server_vols_t *  vols;           /* by mx */
+    time_t downTime;                    /* by mx */
 } cm_server_t;
 
-enum repstate {not_busy, busy, offline};
+enum repstate {srv_not_busy, srv_busy, srv_offline, srv_deleted};
 
 typedef struct cm_serverRef {
     struct cm_serverRef *next;      /* locked by cm_serverLock */
     struct cm_server *server;       /* locked by cm_serverLock */
     enum repstate status;           /* locked by cm_serverLock */
     unsigned long refCount;         /* locked by cm_serverLock */
+    afs_uint32 volID;               /* locked by cm_serverLock */
 } cm_serverRef_t;
 
 /* types */
@@ -48,13 +58,18 @@ typedef struct cm_serverRef {
 #define CM_SERVER_FILE		2	/* a file server */
 
 /* flags */
-#define CM_SERVERFLAG_DOWN	1	/* server is down */
-#define CM_SERVERFLAG_PREF_SET	2       /* server preference set by user */
-#define CM_SERVERFLAG_PINGING 	4 	/* a ping against this server in progress */
+#define CM_SERVERFLAG_DOWN	0x1	/* server is down */
+#define CM_SERVERFLAG_PREF_SET	0x2     /* server preference set by user */
+#define CM_SERVERFLAG_PINGING 	0x4 	/* a ping against this server in progress */
+#define CM_SERVERFLAG_NO64BIT   0x8     /* server has no support for
+                                           64-bit operations. */
+#define CM_SERVERFLAG_NOINLINEBULK 0x10	/* server has no support for inline bulk */
 
 /* flags for procedures */
 #define CM_FLAG_CHECKUPSERVERS		1	/* check working servers */
 #define CM_FLAG_CHECKDOWNSERVERS	2	/* check down servers */
+#define CM_FLAG_CHECKVLDBSERVERS        4       /* check only vldb servers */
+#define CM_FLAG_CHECKFILESERVERS        8       /* check only file servers */
 
 /* values for ipRank */
 #define CM_IPRANK_TOP	5000	/* on same machine */
@@ -67,9 +82,9 @@ typedef struct cm_serverRef {
 #define CM_MAXINTERFACE_ADDR          16
 
 extern cm_server_t *cm_NewServer(struct sockaddr_in *addrp, int type,
-	struct cm_cell *cellp);
+	struct cm_cell *cellp, afs_uint32 flags);
 
-extern cm_serverRef_t *cm_NewServerRef(struct cm_server *serverp);
+extern cm_serverRef_t *cm_NewServerRef(struct cm_server *serverp, afs_uint32 volID);
 
 extern LONG_PTR cm_ChecksumServerList(cm_serverRef_t *serversp);
 
@@ -87,7 +102,7 @@ extern osi_rwlock_t cm_serverLock;
 
 extern void cm_InitServer(void);
 
-extern void cm_CheckServers(long flags, struct cm_cell *cellp);
+extern void cm_CheckServers(afs_uint32 flags, struct cm_cell *cellp);
 
 extern cm_server_t *cm_allServersp;
 
@@ -101,8 +116,16 @@ extern void cm_RandomizeServer(cm_serverRef_t** list);
 
 extern void cm_FreeServer(cm_server_t* server);
 
-extern void cm_FreeServerList(cm_serverRef_t** list);
+#define CM_FREESERVERLIST_DELETE 1
+
+extern void cm_FreeServerList(cm_serverRef_t** list, afs_uint32 flags);
 
 extern void cm_ForceNewConnectionsAllServers(void);
 
+extern void cm_SetServerNo64Bit(cm_server_t * serverp, int no64bit);
+
+extern void cm_SetServerNoInlineBulk(cm_server_t * serverp, int no);
+
+extern cm_server_t * cm_FindServerByIP(afs_uint32 addr, int type);
+
 #endif /*  __CM_SERVER_H_ENV__ */
