@@ -213,6 +213,9 @@ long cm_UpdateVolume(struct cm_cell *cellp, cm_user_t *userp, cm_req_t *reqp,
     } else
 #endif
     {
+        if (cellp->flags & CM_CELLFLAG_VLSERVER_INVALID)
+            cm_UpdateCell(cellp, 0);
+
         /* now we have volume structure locked and held; make RPC to fill it */
 	osi_Log2(afsd_logp, "CALL VL_GetEntryByName{UNO} name %s:%s", volp->cellp->name, volp->namep);
         do {
@@ -253,7 +256,8 @@ long cm_UpdateVolume(struct cm_cell *cellp, cm_user_t *userp, cm_req_t *reqp,
         snprintf(name, VL_MAXNAMELEN, "%s.readonly", volp->namep);
                 
         /* now we have volume structure locked and held; make RPC to fill it */
-	osi_Log2(afsd_logp, "CALL VL_GetEntryByName{UNO} name %s:%s", volp->cellp->name, name);
+	osi_Log2(afsd_logp, "CALL VL_GetEntryByName{UNO} name %s:%s", volp->cellp->name, 
+                 osi_LogSaveString(afsd_logp,name));
         do {
             code = cm_ConnByMServers(cellp->vlServersp, userp, reqp, &connp);
             if (code) 
@@ -275,10 +279,10 @@ long cm_UpdateVolume(struct cm_cell *cellp, cm_user_t *userp, cm_req_t *reqp,
         code = cm_MapVLRPCError(code, reqp);
 	if ( code )
 	    osi_Log3(afsd_logp, "CALL VL_GetEntryByName{UNO} name %s:%s FAILURE, code 0x%x", 
-		      volp->cellp->name, name, code);
+		      volp->cellp->name, osi_LogSaveString(afsd_logp,name), code);
 	else
 	    osi_Log2(afsd_logp, "CALL VL_GetEntryByName{UNO} name %s:%s SUCCESS", 
-		      volp->cellp->name, name);
+		      volp->cellp->name, osi_LogSaveString(afsd_logp,name));
     }
 
     if (code == 0) {
@@ -451,7 +455,7 @@ long cm_UpdateVolume(struct cm_cell *cellp, cm_user_t *userp, cm_req_t *reqp,
             tsp = cm_FindServer(&tsockAddr, CM_SERVER_FILE);
             if (!tsp)
                 tsp = cm_NewServer(&tsockAddr, CM_SERVER_FILE,
-                                    cellp);
+                                    cellp, 0);
 
             /* if this server was created by fs setserverprefs */
             if ( !tsp->cellp ) 
@@ -617,25 +621,27 @@ long cm_GetVolumeByID(cm_cell_t *cellp, afs_uint32 volumeID, cm_user_t *userp,
         
     /* return it held */
     if (volp) {
-	lock_ObtainMutex(&volp->mx);
+        lock_ObtainMutex(&volp->mx);
         
-	code = 0;
-	if (volp->flags & CM_VOLUMEFLAG_RESET) {
-	    code = cm_UpdateVolume(cellp, userp, reqp, volp);
-	    if (code == 0)
-		volp->flags &= ~CM_VOLUMEFLAG_RESET;
-	}
-	lock_ReleaseMutex(&volp->mx);
-	if (code == 0) {
-	    *outVolpp = volp;
+        code = 0;
+        if ((volp->flags & CM_VOLUMEFLAG_RESET) && !(flags & CM_GETVOL_FLAG_NO_RESET)) {
+            code = cm_UpdateVolume(cellp, userp, reqp, volp);
+            if (code == 0)
+                volp->flags &= ~CM_VOLUMEFLAG_RESET;
+        }
+        lock_ReleaseMutex(&volp->mx);
+        if (code == 0) {
+            *outVolpp = volp;
 
-            lock_ObtainWrite(&cm_volumeLock);
-            cm_AdjustVolumeLRU(volp);
-            lock_ReleaseWrite(&cm_volumeLock);
+            if (!(flags & CM_GETVOL_FLAG_NO_LRU_UPDATE)) {
+                lock_ObtainWrite(&cm_volumeLock);
+                cm_AdjustVolumeLRU(volp);
+                lock_ReleaseWrite(&cm_volumeLock);
+            }
         } else
-	    cm_PutVolume(volp);
+            cm_PutVolume(volp);
 
-	return code;
+        return code;
     }
         
     /* otherwise, we didn't find it so consult the VLDB */
@@ -750,6 +756,7 @@ long cm_GetVolumeByName(struct cm_cell *cellp, char *volumeNamep,
 	    volp->dotdotFid.volume = 0;
 	    volp->dotdotFid.unique = 0;
 	    volp->dotdotFid.vnode = 0;
+            volp->dotdotFid.hash = 0;
 	} else {
 	    volp = &cm_data.volumeBaseAddress[cm_data.currentVolumes++];
 	    memset(volp, 0, sizeof(cm_volume_t));
@@ -784,10 +791,10 @@ long cm_GetVolumeByName(struct cm_cell *cellp, char *volumeNamep,
     }
 
     /* if we get here we are holding the mutex */
-    if (volp->flags & CM_VOLUMEFLAG_RESET) {
-	code = cm_UpdateVolume(cellp, userp, reqp, volp);
-	if (code == 0)
-	    volp->flags &= ~CM_VOLUMEFLAG_RESET;
+    if ((volp->flags & CM_VOLUMEFLAG_RESET) && !(flags & CM_GETVOL_FLAG_NO_RESET)) {
+        code = cm_UpdateVolume(cellp, userp, reqp, volp);
+        if (code == 0)
+            volp->flags &= ~CM_VOLUMEFLAG_RESET;
     }	
     lock_ReleaseMutex(&volp->mx);
 
@@ -796,13 +803,15 @@ long cm_GetVolumeByName(struct cm_cell *cellp, char *volumeNamep,
         code = CM_ERROR_NOSUCHVOLUME;
 
     if (code == 0) {
-	*outVolpp = volp;
-
-        lock_ObtainWrite(&cm_volumeLock);
-        cm_AdjustVolumeLRU(volp);
-        lock_ReleaseWrite(&cm_volumeLock);
+		*outVolpp = volp;
+		
+		if (!(flags & CM_GETVOL_FLAG_NO_LRU_UPDATE)) {
+	        lock_ObtainWrite(&cm_volumeLock);
+			cm_AdjustVolumeLRU(volp);
+			lock_ReleaseWrite(&cm_volumeLock);
+		}
     } else
-	cm_PutVolume(volp);
+		cm_PutVolume(volp);
 
     return code;
 }	
@@ -818,7 +827,7 @@ void cm_ForceUpdateVolume(cm_fid_t *fidp, cm_user_t *userp, cm_req_t *reqp)
 
     if (!fidp) return;
 
-    cellp = cm_FindCellByID(fidp->cell);
+    cellp = cm_FindCellByID(fidp->cell, 0);
     if (!cellp) return;
 
     /* search for the volume */
@@ -1009,8 +1018,16 @@ cm_CheckOfflineVolume(cm_volume_t *volp, afs_uint32 volID)
 
     lock_ObtainMutex(&volp->mx);
 
+    if (volp->flags & CM_VOLUMEFLAG_RESET) {
+        cm_InitReq(&req);
+        code = cm_UpdateVolume(volp->cellp, cm_rootUserp, &req, volp);
+        if (code == 0)
+            volp->flags &= ~CM_VOLUMEFLAG_RESET;
+    }
+
     if (volp->rw.ID != 0 && (!volID || volID == volp->rw.ID) &&
-         (volp->rw.state == vl_busy || volp->rw.state == vl_offline)) {
+		volp->rw.serversp &&
+         (volp->rw.state == vl_busy || volp->rw.state == vl_offline || volp->rw.state == vl_unknown)) {
         cm_InitReq(&req);
 
         for (serversp = volp->rw.serversp; serversp; serversp = serversp->next) {
@@ -1018,6 +1035,7 @@ cm_CheckOfflineVolume(cm_volume_t *volp, afs_uint32 volID)
                 serversp->status = srv_not_busy;
         }
 
+        lock_ReleaseMutex(&volp->mx);
         do {
             code = cm_ConnFromVolume(volp, volp->rw.ID, cm_rootUserp, &req, &connp);
             if (code) 
@@ -1031,6 +1049,7 @@ cm_CheckOfflineVolume(cm_volume_t *volp, afs_uint32 volID)
         } while (cm_Analyze(connp, cm_rootUserp, &req, NULL, NULL, NULL, NULL, code));
         code = cm_MapRPCError(code, &req);
 
+        lock_ObtainMutex(&volp->mx);
         if (code == 0 && volStat.Online) {
             cm_VolumeStatusNotification(volp, volp->rw.ID, volp->rw.state, vl_online);
             volp->rw.state = vl_online;
@@ -1043,7 +1062,8 @@ cm_CheckOfflineVolume(cm_volume_t *volp, afs_uint32 volID)
     }
 
     if (volp->ro.ID != 0 && (!volID || volID == volp->ro.ID) &&
-         (volp->ro.state == vl_busy || volp->ro.state == vl_offline)) {
+		volp->ro.serversp &&
+         (volp->ro.state == vl_busy || volp->ro.state == vl_offline || volp->ro.state == vl_unknown)) {
         cm_InitReq(&req);
 
         for (serversp = volp->ro.serversp; serversp; serversp = serversp->next) {
@@ -1051,6 +1071,7 @@ cm_CheckOfflineVolume(cm_volume_t *volp, afs_uint32 volID)
                 serversp->status = srv_not_busy;
         }
 
+        lock_ReleaseMutex(&volp->mx);
         do {
             code = cm_ConnFromVolume(volp, volp->ro.ID, cm_rootUserp, &req, &connp);
             if (code) 
@@ -1064,6 +1085,7 @@ cm_CheckOfflineVolume(cm_volume_t *volp, afs_uint32 volID)
         } while (cm_Analyze(connp, cm_rootUserp, &req, NULL, NULL, NULL, NULL, code));
         code = cm_MapRPCError(code, &req);
 
+        lock_ObtainMutex(&volp->mx);
         if (code == 0 && volStat.Online) {
             cm_VolumeStatusNotification(volp, volp->ro.ID, volp->ro.state, vl_online);
             volp->ro.state = vl_online;
@@ -1076,7 +1098,8 @@ cm_CheckOfflineVolume(cm_volume_t *volp, afs_uint32 volID)
     }
 
     if (volp->bk.ID != 0 && (!volID || volID == volp->bk.ID) &&
-         (volp->bk.state == vl_busy || volp->bk.state == vl_offline)) {
+		volp->bk.serversp &&
+         (volp->bk.state == vl_busy || volp->bk.state == vl_offline || volp->bk.state == vl_unknown)) {
         cm_InitReq(&req);
 
         for (serversp = volp->bk.serversp; serversp; serversp = serversp->next) {
@@ -1084,6 +1107,7 @@ cm_CheckOfflineVolume(cm_volume_t *volp, afs_uint32 volID)
                 serversp->status = srv_not_busy;
         }
 
+        lock_ReleaseMutex(&volp->mx);
         do {
             code = cm_ConnFromVolume(volp, volp->bk.ID, cm_rootUserp, &req, &connp);
             if (code) 
@@ -1097,6 +1121,7 @@ cm_CheckOfflineVolume(cm_volume_t *volp, afs_uint32 volID)
         } while (cm_Analyze(connp, cm_rootUserp, &req, NULL, NULL, NULL, NULL, code));
         code = cm_MapRPCError(code, &req);
 
+        lock_ObtainMutex(&volp->mx);
         if (code == 0 && volStat.Online) {
             cm_VolumeStatusNotification(volp, volp->bk.ID, volp->bk.state, vl_online);
             volp->bk.state = vl_online;
@@ -1546,10 +1571,7 @@ cm_VolumeRenewROCallbacks(void)
             cm_fid_t      fid;
             cm_scache_t * scp;
 
-            fid.cell = volp->cellp->cellID;
-            fid.volume = volp->ro.ID;
-            fid.vnode = 1;
-            fid.unique = 1;
+            cm_SetFid(&fid, volp->cellp->cellID, volp->ro.ID, 1, 1);
 
             cm_InitReq(&req);
 
