@@ -14,6 +14,10 @@
 #include "netbios95.h"
 #endif /* DJGPP */
 
+#if _WIN32_WINNT < 0x0501
+#undef _WIN32_WINNT
+#define _WIN32_WINNT 0x0501
+#endif
 #include <ntsecapi.h>
 
 /* Support largefiles by default */
@@ -219,10 +223,10 @@ typedef struct myncb {
 typedef struct smb_vc {
     struct smb_vc *nextp;		/* not used */
     afs_uint32 magic;			/* a magic value to detect bad entries */
-    unsigned long refCount;		/* the reference count */
-    long flags;			        /* the flags, if any; locked by mx */
+    afs_int32 refCount;		        /* the reference count */
+    afs_uint32 flags;		        /* the flags, if any; locked by mx */
     osi_mutex_t mx;			/* the mutex */
-    long vcID;			        /* VC id */
+    afs_uint32 vcID;		        /* VC id */
     unsigned short lsn;		        /* the NCB LSN associated with this */
     unsigned short uidCounter;	        /* session ID counter */
     unsigned short tidCounter;	        /* tree ID counter */
@@ -255,7 +259,7 @@ typedef struct smb_vc {
 /* one per user session */
 typedef struct smb_user {
     struct smb_user *nextp;		/* next sibling */
-    unsigned long refCount;		/* ref count */
+    afs_int32 refCount;		        /* ref count */
     afs_uint32 flags;		        /* flags; locked by mx */
     osi_mutex_t mx;
     unsigned short userID;		/* the session identifier */
@@ -268,7 +272,7 @@ typedef struct smb_user {
 
 typedef struct smb_username {
     struct smb_username *nextp;		/* next sibling */
-    unsigned long refCount;		/* ref count */
+    afs_int32 refCount;		        /* ref count */
     long flags;			        /* flags; locked by mx */
     osi_mutex_t mx;
     struct cm_user *userp;		/* CM user structure */
@@ -299,7 +303,7 @@ typedef struct smb_username {
 /* one per tree-connect */
 typedef struct smb_tid {
     struct smb_tid *nextp;		/* next sibling */
-    unsigned long refCount;
+    afs_int32 refCount;
     afs_uint32 flags;			/* protected by mx */
     osi_mutex_t mx;			/* for non-tree-related stuff */
     unsigned short tid;		        /* the tid */
@@ -315,7 +319,7 @@ typedef struct smb_tid {
 /* one per process ID */
 typedef struct smb_pid {
     struct smb_pid *nextp;		/* next sibling */
-    unsigned long refCount;
+    afs_int32 refCount;
     long flags;
     osi_mutex_t mx;			/* for non-tree-related stuff */
     unsigned short pid;		        /* the pid */
@@ -328,7 +332,7 @@ typedef struct smb_ioctl {
     char *inDatap;			/* ioctl func's current position
 					 * in input parameter block */
     char *inAllocp;			/* allocated input parameter block */
-    long inCopied;			/* # of input bytes copied in so far
+    afs_uint32 inCopied;			/* # of input bytes copied in so far
 					 * by write calls */
     cm_space_t *prefix;		        /* prefix for subst drives */
     char *tidPathp;			/* Pathname associated with Tree ID */
@@ -336,11 +340,11 @@ typedef struct smb_ioctl {
     /* output side */
     char *outDatap;			/* output results assembled so far */
     char *outAllocp;		        /* output results assembled so far */
-    long outCopied;			/* # of output bytes copied back so far
+    afs_uint32 outCopied;		/* # of output bytes copied back so far
                                          * by read calls */
 	
     /* flags */
-    long flags;
+    afs_uint32 flags;
 
     /* fid pointer */
     struct smb_fid *fidp;
@@ -356,7 +360,7 @@ typedef struct smb_ioctl {
 /* one per file ID; these are really file descriptors */
 typedef struct smb_fid {
     osi_queue_t q;
-    unsigned long refCount;
+    afs_int32 refCount;
     afs_uint32 flags;			/* protected by mx */
     osi_mutex_t mx;			/* for non-tree-related stuff */
     unsigned short fid;		        /* the file ID */
@@ -383,7 +387,7 @@ typedef struct smb_fid {
     afs_uint32	delete;			/* ok to del: locked by smb_rctLock */
 } smb_fid_t;
 
-#define SMB_FID_OPENREAD		1	/* open for reading */
+#define SMB_FID_OPENREAD_LISTDIR	1	/* open for reading / listing directory */
 #define SMB_FID_OPENWRITE		2	/* open for writing */
 #define SMB_FID_CREATED                 4       /* a new file */
 #define SMB_FID_IOCTL			8	/* a file descriptor for the
@@ -400,6 +404,7 @@ typedef struct smb_fid {
 #define SMB_FID_NTOPEN			0x100	/* have dscp and pathp */
 #define SMB_FID_SEQUENTIAL		0x200
 #define SMB_FID_RANDOM			0x400
+#define SMB_FID_EXECUTABLE              0x800
 
 #define SMB_FID_SHARE_READ              0x1000
 #define SMB_FID_SHARE_WRITE             0x2000
@@ -447,7 +452,7 @@ typedef struct smb_fid {
 typedef struct smb_dirSearch {
     osi_queue_t q;			/* queue of all outstanding cookies */
     osi_mutex_t mx;			/* just in case the caller screws up */
-    unsigned long refCount;		/* reference count */
+    afs_int32 refCount;		        /* reference count */
     long cookie;			/* value returned to the caller */
     struct cm_scache *scp;		/* vnode of the dir we're searching */
     time_t lastTime;			/* last time we used this (osi_Time) */
@@ -455,6 +460,8 @@ typedef struct smb_dirSearch {
 					 * locked by smb_globalLock */
     unsigned short attribute;	        /* search attribute
 					 * (used for extended protocol) */
+    char tidPath[256];                  /* tid path */
+    char relPath[1024];                 /* relative path */        
     char mask[256];			/* search mask for V3 */
 } smb_dirSearch_t;
 
@@ -519,11 +526,11 @@ typedef struct smb_dispatch {
 						 * the response was already
 						 * sent.
                                                  */
-#define SMB_MAX_PATH                    256     /* max path length */
+#define SMB_MAX_PATH                    260     /* max path length */
 
 /* prototypes */
 
-extern void smb_Init(osi_log_t *logp, char *smbNamep, int useV3, int LANadapt,
+extern void smb_Init(osi_log_t *logp, int useV3,
 	int nThreads
 #ifndef DJGPP
         , void *aMBfunc
@@ -556,7 +563,7 @@ extern smb_tid_t *smb_FindTID(smb_vc_t *vcp, unsigned short tid, int flags);
 
 extern void smb_HoldTIDNoLock(smb_tid_t *tidp);
 
-extern void smb_ReleaseTID(smb_tid_t *tidp);
+extern void smb_ReleaseTID(smb_tid_t *tidp, afs_uint32 locked);
 
 extern smb_user_t *smb_FindUID(smb_vc_t *vcp, unsigned short uid, int flags);
 
@@ -664,6 +671,8 @@ extern int smb_maxMpxRequests; /* max # of mpx requests */
 extern int smb_StoreAnsiFilenames;
 extern int smb_hideDotFiles;
 extern unsigned int smb_IsDotFile(char *lastComp);
+extern afs_uint32 smb_AsyncStore;
+extern afs_uint32 smb_AsyncStoreSize;
 
 /* the following are used for smb auth */
 extern int smb_authType; /* Type of SMB authentication to be used. One from below. */
@@ -748,9 +757,13 @@ extern DWORD smb_ServerExceptionFilter(void);
 extern void smb_UpdateServerPriority(void);
 extern void smb_SetRequestStartTime(void);
 extern void smb_ResetServerPriority(void);
-extern void smb_RestartListeners(void);
-extern void smb_StopListeners(void);
-extern void smb_StopListener(NCB *ncbp, int lana);
+extern void smb_RestartListeners(int);
+extern void smb_StopListeners(int);
+extern void smb_StopListener(NCB *ncbp, int lana, int wait);
+extern long smb_IsNetworkStarted(void);
+extern void smb_LanAdapterChange(int);
+extern void smb_SetLanAdapterChangeDetected(void);
+
 
 #define SMB_LISTENER_UNINITIALIZED -1
 #define SMB_LISTENER_STOPPED 0
