@@ -18,7 +18,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_lookup.c,v 1.50.2.3 2004/10/18 17:43:51 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_lookup.c,v 1.72.2.4 2008/02/11 04:00:49 shadow Exp $");
 
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
 #include "afsincludes.h"	/* Afs-based standard headers */
@@ -27,13 +27,11 @@ RCSID
 #include "afs/nfsclient.h"
 #include "afs/exporter.h"
 #include "afs/afs_osidnlc.h"
+#include "afs/afs_dynroot.h"
 
 
 extern struct DirEntry *afs_dir_GetBlob();
-
-#ifdef AFS_LINUX22_ENV
-extern struct inode_operations afs_symlink_iops, afs_dir_iops;
-#endif
+extern struct vcache *afs_globalVp;
 
 
 afs_int32 afs_bkvolpref = 0;
@@ -56,45 +54,45 @@ int afs_fakestat_enable = 0;	/* 1: fakestat-all, 2: fakestat-crosscell */
  *
  * NOTE: this function returns a held volume structure in *volpp if it returns 0!
  */
-int
-EvalMountPoint(register struct vcache *avc, struct vcache *advc,
-	       struct volume **avolpp, register struct vrequest *areq)
+static int
+EvalMountData(char type, char *data, afs_uint32 states, afs_uint32 cellnum,
+              struct volume **avolpp, register struct vrequest *areq,
+	      afs_uint32 *acellidxp, afs_uint32 *avolnump, afs_uint32 *avnoidp)
 {
-    afs_int32 code;
     struct volume *tvp = 0;
     struct VenusFid tfid;
     struct cell *tcell;
-    char *cpos, *volnamep;
-    char type, *buf;
+    char *cpos, *volnamep, *x;
+    char *buf;
     afs_int32 prefetch;		/* 1=>None  2=>RO  3=>BK */
-    afs_int32 mtptCell, assocCell, hac = 0;
+    afs_int32 mtptCell, assocCell = 0, hac = 0;
     afs_int32 samecell, roname, len;
+    afs_uint32 volid, cellidx, vnoid = 0;
 
-    AFS_STATCNT(EvalMountPoint);
-#ifdef notdef
-    if (avc->mvid && (avc->states & CMValid))
-	return 0;		/* done while racing */
-#endif
-    *avolpp = NULL;
-    code = afs_HandleLink(avc, areq);
-    if (code)
-	return code;
-
-    /* Determine which cell and volume the mointpoint goes to */
-    type = avc->linkData[0];	/* '#'=>Regular '%'=>RW */
-    cpos = afs_strchr(&avc->linkData[1], ':');	/* if cell name present */
+    cpos = afs_strchr(data, ':');	/* if cell name present */
     if (cpos) {
+	cellnum = 0;
 	volnamep = cpos + 1;
 	*cpos = 0;
-	tcell = afs_GetCellByName(&avc->linkData[1], READ_LOCK);
+	for (x = data; *x >= '0' && *x <= '9'; x++)
+	    cellnum = (cellnum * 10) + (*x - '0');
+	if (cellnum && !*x)
+	    tcell = afs_GetCell(cellnum, READ_LOCK);
+	else {
+	    tcell = afs_GetCellByName(data, READ_LOCK);
+	               cellnum = 0;
+	}
 	*cpos = ':';
+    } else if (cellnum) {
+	volnamep = data;
+	tcell = afs_GetCell(cellnum, READ_LOCK);
     } else {
-	volnamep = &avc->linkData[1];
-	tcell = afs_GetCell(avc->fid.Cell, READ_LOCK);
+	return ENODEV;
     }
     if (!tcell)
 	return ENODEV;
 
+    cellidx = tcell->cellIndex;
     mtptCell = tcell->cellNum;	/* The cell for the mountpoint */
     if (tcell->lcellp) {
 	hac = 1;		/* has associated cell */
@@ -102,15 +100,61 @@ EvalMountPoint(register struct vcache *avc, struct vcache *advc,
     }
     afs_PutCell(tcell, READ_LOCK);
 
+    cpos = afs_strrchr(volnamep, ':'); /* if vno present */
+    if (cpos) 
+	*cpos = 0;
+    /* Look for an all-numeric volume ID */
+    volid = 0;
+    for (x = volnamep; *x >= '0' && *x <= '9'; x++)
+	volid = (volid * 10) + (*x - '0');
+    if (cpos) {
+	*cpos = ':';
+	vnoid = 0;
+	if (!*x) /* allow vno with numeric volid only */
+	    for (x = (cpos + 1); *x >= '0' && *x <= '9'; x++)
+		vnoid = (vnoid * 10) + (*x - '0');
+	if (*x)
+	    vnoid = 0;
+    }
+
+    /*
+     * If the volume ID was all-numeric, and they didn't ask for a
+     * pointer to the volume structure, then just return the number
+     * as-is.  This is currently only used for handling name lookups
+     * in the dynamic mount directory.
+     */
+    if (!*x && !avolpp) {
+	if (acellidxp)
+	    *acellidxp = cellidx;
+	if (avolnump)
+	    *avolnump = volid;
+	if (avnoidp)
+	    *avnoidp = vnoid;
+	return 0;
+    }
+
+    /*
+     * If the volume ID was all-numeric, and the type was '%', then
+     * assume whoever made the mount point knew what they were doing,
+     * and don't second-guess them by forcing use of a RW volume when
+     * they gave the ID of something else.
+     */
+    if (!*x && type == '%') {
+	tfid.Fid.Volume = volid;	/* remember BK volume */
+	tfid.Cell = mtptCell;
+	tvp = afs_GetVolume(&tfid, areq, WRITE_LOCK);	/* get the new one */
+	if (!tvp)
+	    return ENODEV;	/* oops, can't do it */
+	goto done;
+    }
+
     /* Is volume name a "<n>.backup" or "<n>.readonly" name */
     len = strlen(volnamep);
     roname = ((len > 9) && (strcmp(&volnamep[len - 9], ".readonly") == 0))
 	|| ((len > 7) && (strcmp(&volnamep[len - 7], ".backup") == 0));
 
     /* When we cross mountpoint, do we stay in the same cell */
-    samecell = (avc->fid.Cell == mtptCell) || (hac
-					       && (avc->fid.Cell ==
-						   assocCell));
+    samecell = (cellnum == mtptCell) || (hac && (cellnum == assocCell));
 
     /* Decide whether to prefetch the BK, or RO.  Also means we want the BK or
      * RO.
@@ -121,9 +165,9 @@ EvalMountPoint(register struct vcache *avc, struct vcache *advc,
      *   want to prefetch the RO volume.
      */
     if ((type == '#') && !roname) {
-	if (afs_bkvolpref && samecell && (avc->states & CBackup))
+	if (afs_bkvolpref && samecell && (states & CBackup))
 	    prefetch = 3;	/* Prefetch the BK */
-	else if (!samecell || (avc->states & CRO))
+	else if (!samecell || (states & CRO))
 	    prefetch = 2;	/* Prefetch the RO */
 	else
 	    prefetch = 1;	/* Do not prefetch */
@@ -167,7 +211,7 @@ EvalMountPoint(register struct vcache *avc, struct vcache *advc,
 	return ENODEV;		/* Couldn't find the volume */
 
     /* Don't cross mountpoint from a BK to a BK volume */
-    if ((avc->states & CBackup) && (tvp->states & VBackup)) {
+    if ((states & CBackup) && (tvp->states & VBackup)) {
 	afs_PutVolume(tvp, WRITE_LOCK);
 	return ENODEV;
     }
@@ -194,12 +238,52 @@ EvalMountPoint(register struct vcache *avc, struct vcache *advc,
 	    return ENODEV;	/* oops, can't do it */
     }
 
+done:
+    if (acellidxp)
+	*acellidxp = cellidx;
+    if (avolnump)
+	*avolnump = tvp->volume;
+    if (avnoidp)
+	*avnoidp = vnoid;
+    if (avolpp)
+	*avolpp = tvp;
+    else
+	afs_PutVolume(tvp, WRITE_LOCK);
+    return 0;
+}
+
+int
+EvalMountPoint(register struct vcache *avc, struct vcache *advc,
+	       struct volume **avolpp, register struct vrequest *areq)
+{
+    afs_int32 code;
+    afs_uint32 avnoid;
+
+    AFS_STATCNT(EvalMountPoint);
+#ifdef notdef
+    if (avc->mvid && (avc->states & CMValid))
+	return 0;		/* done while racing */
+#endif
+    *avolpp = NULL;
+    code = afs_HandleLink(avc, areq);
+    if (code)
+	return code;
+
+    /* Determine which cell and volume the mointpoint goes to */
+    code = EvalMountData(avc->linkData[0], avc->linkData + 1,
+                         avc->states, avc->fid.Cell, avolpp, areq, 0, 0,
+			 &avnoid);
+    if (code) return code;
+
+    if (!avnoid)
+	avnoid = 1;
+
     if (avc->mvid == 0)
 	avc->mvid =
 	    (struct VenusFid *)osi_AllocSmallSpace(sizeof(struct VenusFid));
-    avc->mvid->Cell = tvp->cell;
-    avc->mvid->Fid.Volume = tvp->volume;
-    avc->mvid->Fid.Vnode = 1;
+    avc->mvid->Cell = (*avolpp)->cell;
+    avc->mvid->Fid.Volume = (*avolpp)->volume;
+    avc->mvid->Fid.Vnode = avnoid;
     avc->mvid->Fid.Unique = 1;
     avc->states |= CMValid;
 
@@ -218,11 +302,10 @@ EvalMountPoint(register struct vcache *avc, struct vcache *advc,
      * cd'ing via a new path to a volume will reset the ".." pointer
      * to the new path.
      */
-    tvp->mtpoint = avc->fid;	/* setup back pointer to mtpoint */
+    (*avolpp)->mtpoint = avc->fid;	/* setup back pointer to mtpoint */
     if (advc)
-	tvp->dotdot = advc->fid;
+	(*avolpp)->dotdot = advc->fid;
 
-    *avolpp = tvp;
     return 0;
 }
 
@@ -253,7 +336,7 @@ afs_InitFakeStat(struct afs_fakestat_state *state)
  *
  * Only issues RPCs if canblock is non-zero.
  */
-int
+static int
 afs_EvalFakeStat_int(struct vcache **avcp, struct afs_fakestat_state *state,
 		     struct vrequest *areq, int canblock)
 {
@@ -295,7 +378,7 @@ afs_EvalFakeStat_int(struct vcache **avcp, struct afs_fakestat_state *state,
 	    do {
 		retry = 0;
 		ObtainWriteLock(&afs_xvcache, 597);
-		root_vp = afs_FindVCache(tvc->mvid, &retry, 0);
+		root_vp = afs_FindVCache(tvc->mvid, &retry, IS_WLOCK);
 		if (root_vp && retry) {
 		    ReleaseWriteLock(&afs_xvcache);
 		    afs_PutVCache(root_vp);
@@ -309,7 +392,15 @@ afs_EvalFakeStat_int(struct vcache **avcp, struct afs_fakestat_state *state,
 	    code = canblock ? ENOENT : 0;
 	    goto done;
 	}
-	if (tvolp) {
+#ifdef AFS_DARWIN80_ENV
+        root_vp->m.Type = VDIR;
+        AFS_GUNLOCK();
+        code = afs_darwin_finalizevnode(root_vp, NULL, NULL, 0);
+        AFS_GLOCK();
+        if (code) goto done;
+        vnode_ref(AFSTOV(root_vp));
+#endif
+	if (tvolp && !afs_InReadDir(root_vp)) {
 	    /* Is this always kosher?  Perhaps we should instead use
 	     * NBObtainWriteLock to avoid potential deadlock.
 	     */
@@ -416,7 +507,7 @@ afs_getsysname(register struct vrequest *areq, register struct vcache *adp,
     else {
 	au = afs_GetUser(areq->uid, adp->fid.Cell, 0);
 	if (au->exporter) {
-	    error = EXP_SYSNAME(au->exporter, (char *)0, sysnamelist, num);
+	    error = EXP_SYSNAME(au->exporter, (char *)0, sysnamelist, num, 0);
 	    if (error) {
 		strcpy(bufp, "@sys");
 		afs_PutUser(au, 0);
@@ -432,15 +523,15 @@ afs_getsysname(register struct vrequest *areq, register struct vcache *adp,
 }
 
 void
-Check_AtSys(register struct vcache *avc, char *aname,
+Check_AtSys(register struct vcache *avc, const char *aname,
 	    struct sysname_info *state, struct vrequest *areq)
 {
     int num = 0;
-    char **sysnamelist[MAXSYSNAME];
+    char **sysnamelist[MAXNUMSYSNAMES];
 
     if (AFS_EQ_ATSYS(aname)) {
 	state->offset = 0;
-	state->name = (char *)osi_AllocLargeSpace(AFS_SMALLOCSIZ);
+	state->name = (char *)osi_AllocLargeSpace(MAXSYSNAME);
 	state->allocked = 1;
 	state->index =
 	    afs_getsysname(areq, avc, state->name, &num, sysnamelist);
@@ -448,7 +539,7 @@ Check_AtSys(register struct vcache *avc, char *aname,
 	state->offset = -1;
 	state->allocked = 0;
 	state->index = 0;
-	state->name = aname;
+	state->name = (char *)aname;
     }
 }
 
@@ -457,7 +548,7 @@ Next_AtSys(register struct vcache *avc, struct vrequest *areq,
 	   struct sysname_info *state)
 {
     int num = afs_sysnamecount;
-    char **sysnamelist[MAXSYSNAME];
+    char **sysnamelist[MAXNUMSYSNAMES];
 
     if (state->index == -1)
 	return 0;		/* No list */
@@ -493,35 +584,22 @@ Next_AtSys(register struct vcache *avc, struct vrequest *areq,
 	    au = afs_GetUser(areq->uid, avc->fid.Cell, 0);
 	    if (au->exporter) {
 		error =
-		    EXP_SYSNAME(au->exporter, (char *)0, sysnamelist, num);
+		    EXP_SYSNAME(au->exporter, (char *)0, sysnamelist, &num, 0);
 		if (error) {
+		    afs_PutUser(au, 0);
 		    return 0;
 		}
 	    }
 	    afs_PutUser(au, 0);
 	}
-	if (++(state->index) >= num || !(*sysnamelist)[state->index])
+	if (++(state->index) >= num || !(*sysnamelist)[(unsigned int)state->index])
 	    return 0;		/* end of list */
     }
-    strcpy(state->name + state->offset, (*sysnamelist)[state->index]);
+    strcpy(state->name + state->offset, (*sysnamelist)[(unsigned int)state->index]);
     return 1;
 }
 
-#if (defined(AFS_SGI62_ENV) || defined(AFS_SUN57_64BIT_ENV))
-extern int BlobScan(ino64_t * afile, afs_int32 ablob);
-#else
-#if defined(AFS_HPUX1123_ENV)
-/* DEE should use the new afs_inode_t  for all */
-extern int BlobScan(ino_t * afile, afs_int32 ablob);
-#else
-#if defined AFS_LINUX_64BIT_KERNEL
-extern int BlobScan(long *afile, afs_int32 ablob);
-#else
-extern int BlobScan(afs_int32 * afile, afs_int32 ablob);
-#endif
-#endif
-#endif
-
+extern int BlobScan(struct dcache * afile, afs_int32 ablob);
 
 /* called with an unlocked directory and directory cookie.  Areqp
  * describes who is making the call.
@@ -560,14 +638,14 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
     struct afs_q *tq;		/* temp queue variable */
     AFSCBFids fidParm;		/* file ID parm for bulk stat */
     AFSBulkStats statParm;	/* stat info parm for bulk stat */
-    int fidIndex;		/* which file were stating */
-    struct conn *tcp;		/* conn for call */
+    int fidIndex = 0;		/* which file were stating */
+    struct conn *tcp = 0;	/* conn for call */
     AFSCBs cbParm;		/* callback parm for bulk stat */
     struct server *hostp = 0;	/* host we got callback from */
     long startTime;		/* time we started the call,
 				 * for callback expiration base
 				 */
-    afs_size_t statSeqNo;	/* Valued of file size to detect races */
+    afs_size_t statSeqNo = 0;	/* Valued of file size to detect races */
     int code;			/* error code */
     long newIndex;		/* new index in the dir */
     struct DirEntry *dirEntryp;	/* dir entry we are examining */
@@ -578,9 +656,12 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
     long volStates;		/* flags from vol structure */
     struct volume *volp = 0;	/* volume ptr */
     struct VenusFid dotdot;
-    int flagIndex;		/* First file with bulk fetch flag set */
+    int flagIndex = 0;		/* First file with bulk fetch flag set */
     int inlinebulk = 0;		/* Did we use InlineBulk RPC or not? */
     XSTATS_DECLS;
+#ifdef AFS_DARWIN80_ENV
+    panic("bulkstatus doesn't work on AFS_DARWIN80_ENV. don't call it");
+#endif
     /* first compute some basic parameters.  We dont want to prefetch more
      * than a fraction of the cache in any given call, and we want to preserve
      * a portion of the LRU queue in any event, so as to avoid thrashing
@@ -620,12 +701,12 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
   tagain:
     code = afs_VerifyVCache(adp, areqp);
     if (code)
-	goto done;
+	goto done2;
 
     dcp = afs_GetDCache(adp, (afs_size_t) 0, areqp, &temp, &temp, 1);
     if (!dcp) {
 	code = ENOENT;
-	goto done;
+	goto done2;
     }
 
     /* lock the directory cache entry */
@@ -674,7 +755,7 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
 	/* look for first safe entry to examine in the directory.  BlobScan
 	 * looks for a the 1st allocated dir after the dirCookie slot.
 	 */
-	newIndex = BlobScan(&dcp->f, (dirCookie >> 5));
+	newIndex = BlobScan(dcp, (dirCookie >> 5));
 	if (newIndex == 0)
 	    break;
 
@@ -683,7 +764,7 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
 
 	/* get a ptr to the dir entry */
 	dirEntryp =
-	    (struct DirEntry *)afs_dir_GetBlob(&dcp->f, newIndex);
+	    (struct DirEntry *)afs_dir_GetBlob(dcp, newIndex);
 	if (!dirEntryp)
 	    break;
 
@@ -708,7 +789,7 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
 	    do {
 		retry = 0;
 		ObtainWriteLock(&afs_xvcache, 130);
-		tvcp = afs_FindVCache(&tfid, &retry, 0 /* no stats | LRU */ );
+		tvcp = afs_FindVCache(&tfid, &retry, IS_WLOCK /* no stats | LRU */ );
 		if (tvcp && retry) {
 		    ReleaseWriteLock(&afs_xvcache);
 		    afs_PutVCache(tvcp);
@@ -716,16 +797,36 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
 	    } while (tvcp && retry);
 	    if (!tvcp) {	/* otherwise, create manually */
 		tvcp = afs_NewVCache(&tfid, hostp);
-		ObtainWriteLock(&tvcp->lock, 505);
-		ReleaseWriteLock(&afs_xvcache);
-		afs_RemoveVCB(&tfid);
-		ReleaseWriteLock(&tvcp->lock);
+		if (tvcp)
+		{
+			ObtainWriteLock(&tvcp->lock, 505);
+			ReleaseWriteLock(&afs_xvcache);
+			afs_RemoveVCB(&tfid);
+			ReleaseWriteLock(&tvcp->lock);
+		} else {
+			ReleaseWriteLock(&afs_xvcache);
+		}
 	    } else {
 		ReleaseWriteLock(&afs_xvcache);
 	    }
 	    if (!tvcp)
-		goto done;	/* can't happen at present, more's the pity */
+	    {
+		DRelease((struct buffer *)dirEntryp, 0);
+		ReleaseReadLock(&dcp->lock);
+		ReleaseReadLock(&adp->lock);
+		afs_PutDCache(dcp);
+		goto done;	/* can happen if afs_NewVCache fails */
+	    }
 
+#ifdef AFS_DARWIN80_ENV
+            if (tvcp->states & CVInit) {
+                 /* XXX don't have status yet, so creating the vnode is
+                    not yet useful. we would get CDeadVnode set, and the
+                    upcoming PutVCache will cause the vcache to be flushed &
+                    freed, which in turn means the bulkstatus results won't 
+                    be used */
+            }
+#endif
 	    /* WARNING: afs_DoBulkStat uses the Length field to store a
 	     * sequence number for each bulk status request. Under no
 	     * circumstances should afs_DoBulkStat store a sequence number
@@ -988,12 +1089,8 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
 	 * We only do this if the entry looks clear.
 	 */
 	afs_ProcessFS(tvcp, &statsp[i], areqp);
-#ifdef AFS_LINUX22_ENV
-	/* overwrite the ops if it's a directory or symlink. */
-	if (vType(tvcp) == VDIR)
-	    tvcp->v.v_op = &afs_dir_iops;
-	else if (vType(tvcp) == VLNK)
-	    tvcp->v.v_op = &afs_symlink_iops;
+#if defined(AFS_LINUX22_ENV)
+	afs_fill_inode(AFSTOV(tvcp), NULL);	/* reset inode operations */
 #endif
 
 	/* do some accounting for bulk stats: mark this entry as
@@ -1081,45 +1178,30 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
     } else {
 	code = 0;
     }
+  done2:
     osi_FreeLargeSpace(statMemp);
     osi_FreeLargeSpace(cbfMemp);
     return code;
 }
 
 /* was: (AFS_DEC_ENV) || defined(AFS_OSF30_ENV) || defined(AFS_NCR_ENV) */
+#ifdef AFS_DARWIN80_ENV
+#define AFSDOBULK 0
+#else
 static int AFSDOBULK = 1;
+#endif
 
 int
-#ifdef	AFS_OSF_ENV
-afs_lookup(adp, ndp)
-     struct vcache *adp;
-     struct nameidata *ndp;
-{
-    char aname[MAXNAMLEN + 1];	/* XXX */
-    struct vcache **avcp = (struct vcache **)&(ndp->ni_vp);
-    struct ucred *acred = ndp->ni_cred;
-    int wantparent = ndp->ni_nameiop & WANTPARENT;
-    int opflag = ndp->ni_nameiop & OPFLAG;
-#else				/* AFS_OSF_ENV */
-#if	defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV)
-afs_lookup(OSI_VC_ARG(adp), aname, avcp, pnp, flags, rdir, acred)
-     struct pathname *pnp;
-     int flags;
-     struct vnode *rdir;
+#ifdef AFS_OSF_ENV
+afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED *acred, int opflag, int wantparent)
+#elif defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV)
+afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct pathname *pnp, int flags, struct vnode *rdir, struct AFS_UCRED *acred)
+#elif defined(UKERNEL)
+afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED *acred, int flags)
 #else
-#if defined(UKERNEL)
-afs_lookup(adp, aname, avcp, acred, flags)
-     int flags;
-#else
-afs_lookup(adp, aname, avcp, acred)
-#endif				/* UKERNEL */
-#endif				/* SUN5 || SGI */
-     OSI_VC_DECL(adp);
-     struct vcache **avcp;
-     char *aname;
-     struct AFS_UCRED *acred;
-{
+afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED *acred)
 #endif
+{
     struct vrequest treq;
     char *tname = NULL;
     register struct vcache *tvc = 0;
@@ -1144,8 +1226,6 @@ afs_lookup(adp, aname, avcp, acred)
 
 #ifdef	AFS_OSF_ENV
     ndp->ni_dvp = AFSTOV(adp);
-    memcpy(aname, ndp->ni_ptr, ndp->ni_namelen);
-    aname[ndp->ni_namelen] = '\0';
 #endif /* AFS_OSF_ENV */
 
 #if defined(AFS_DARWIN_ENV)
@@ -1178,7 +1258,7 @@ afs_lookup(adp, aname, avcp, acred)
     *avcp = NULL;		/* Since some callers don't initialize it */
     bulkcode = 0;
 
-    if (!(adp->states & CStatd)) {
+    if (!(adp->states & CStatd) && !afs_InReadDir(adp)) {
 	if ((code = afs_VerifyVCache2(adp, &treq))) {
 	    goto done;
 	}
@@ -1190,7 +1270,6 @@ afs_lookup(adp, aname, avcp, acred)
 	/* looking up ".." in root via special hacks */
 	if (adp->mvid == (struct VenusFid *)0 || adp->mvid->Fid.Volume == 0) {
 #ifdef	AFS_OSF_ENV
-	    extern struct vcache *afs_globalVp;
 	    if (adp == afs_globalVp) {
 		struct vnode *rvp = AFSTOV(adp);
 /*
@@ -1212,7 +1291,7 @@ afs_lookup(adp, aname, avcp, acred)
 	*avcp = tvc;
 	code = (tvc ? 0 : ENOENT);
 	hit = 1;
-	if (tvc && !VREFCOUNT(tvc)) {
+	if (tvc && !VREFCOUNT_GT(tvc, 0)) {
 	    osi_Panic("TT1");
 	}
 	if (code) {
@@ -1233,7 +1312,7 @@ afs_lookup(adp, aname, avcp, acred)
 
     /* Check for read access as well.  We need read access in order to
      * stat files, but not to stat subdirectories. */
-    if (!afs_AccessOK(adp, PRSFS_LOOKUP, &treq, CHECK_MODE_BITS))
+    if (!afs_AccessOK(adp, PRSFS_READ, &treq, CHECK_MODE_BITS))
 	no_read_access = 1;
 
     /* special case lookup of ".".  Can we check for it sooner in this code,
@@ -1245,15 +1324,75 @@ afs_lookup(adp, aname, avcp, acred)
 	ObtainReadLock(&afs_xvcache);
 	osi_vnhold(adp, 0);
 	ReleaseReadLock(&afs_xvcache);
+#ifdef AFS_DARWIN80_ENV
+        vnode_get(AFSTOV(adp));
+#endif
 	code = 0;
 	*avcp = tvc = adp;
 	hit = 1;
-	if (adp && !VREFCOUNT(adp)) {
+	if (adp && !VREFCOUNT_GT(adp, 0)) {
 	    osi_Panic("TT2");
 	}
 	goto done;
     }
 
+    /*
+     * Special case lookup of ".." in the dynamic mount directory.
+     * The parent of this directory is _always_ the AFS root volume.
+     */
+    if (afs_IsDynrootMount(adp) &&
+	aname[0] == '.' && aname[1] == '.' && !aname[2]) {
+
+	ObtainReadLock(&afs_xvcache);
+	osi_vnhold(afs_globalVp, 0);
+	ReleaseReadLock(&afs_xvcache);
+#ifdef AFS_DARWIN80_ENV
+        vnode_get(AFSTOV(afs_globalVp));
+#endif
+	code = 0;
+	*avcp = tvc = afs_globalVp;
+	hit = 1;
+	goto done;
+    }
+
+    /*
+     * Special case lookups in the dynamic mount directory.
+     * The names here take the form cell:volume, similar to a mount point.
+     * EvalMountData parses that and returns a cell and volume ID, which
+     * we use to construct the appropriate dynroot Fid.
+     */
+    if (afs_IsDynrootMount(adp)) {
+	struct VenusFid tfid;
+	afs_uint32 cellidx, volid, vnoid;
+
+	code = EvalMountData('%', aname, 0, 0, NULL, &treq, &cellidx, &volid, &vnoid);
+	if (code)
+	    goto done;
+	afs_GetDynrootMountFid(&tfid);
+	tfid.Fid.Vnode = VNUM_FROM_TYPEID(VN_TYPE_MOUNT, cellidx << 2);
+	tfid.Fid.Unique = volid;
+	*avcp = tvc = afs_GetVCache(&tfid, &treq, NULL, NULL);
+	hit = 1;
+	goto done;
+    }
+
+#ifdef AFS_LINUX26_ENV
+    /*
+     * Special case of the dynamic mount volume in a static root.
+     * This is really unfortunate, but we need this for the translator.
+     */
+    if (adp == afs_globalVp && !afs_GetDynrootEnable() &&
+	!strcmp(aname, AFS_DYNROOT_MOUNTNAME)) {
+	struct VenusFid tfid;
+
+	afs_GetDynrootMountFid(&tfid);
+	*avcp = tvc = afs_GetVCache(&tfid, &treq, NULL, NULL);
+	code = 0;
+	hit = 1;
+	goto done;
+    }
+#endif
+
     Check_AtSys(adp, aname, &sysState, &treq);
     tname = sysState.name;
 
@@ -1276,7 +1415,7 @@ afs_lookup(adp, aname, avcp, acred)
 	}
 #ifdef AFS_LINUX22_ENV
 	if (tvc->mvstat == 2) {	/* we don't trust the dnlc for root vcaches */
-	    AFS_RELE(tvc);
+	    AFS_RELE(AFSTOV(tvc));
 	    *avcp = 0;
 	} else {
 	    code = 0;
@@ -1293,12 +1432,14 @@ afs_lookup(adp, aname, avcp, acred)
     {				/* sub-block just to reduce stack usage */
 	register struct dcache *tdc;
 	afs_size_t dirOffset, dirLen;
-	struct fcache *theDir;
 	struct VenusFid tfid;
 
 	/* now we have to lookup the next fid */
-	tdc =
-	    afs_GetDCache(adp, (afs_size_t) 0, &treq, &dirOffset, &dirLen, 1);
+	if (afs_InReadDir(adp))
+	    tdc = adp->dcreaddir;
+	else
+	    tdc = afs_GetDCache(adp, (afs_size_t) 0, &treq,
+				&dirOffset, &dirLen, 1);
 	if (!tdc) {
 	    *avcp = NULL;	/* redundant, but harmless */
 	    code = EIO;
@@ -1315,24 +1456,30 @@ afs_lookup(adp, aname, avcp, acred)
 	 * cases we need to worry about:
 	 * 1. The cache data is being fetched by another process.
 	 * 2. The cache data is no longer valid
+	 *
+	 * If a readdir is in progress _in this thread_, it has a shared
+	 * lock on the vcache and has obtained current data, so we just
+	 * use that.  This eliminates several possible deadlocks.  
 	 */
-	while ((adp->states & CStatd)
-	       && (tdc->dflags & DFFetching)
-	       && hsame(adp->m.DataVersion, tdc->f.versionNo)) {
-	    ReleaseReadLock(&tdc->lock);
-	    ReleaseReadLock(&adp->lock);
-	    afs_osi_Sleep(&tdc->validPos);
-	    ObtainReadLock(&adp->lock);
-	    ObtainReadLock(&tdc->lock);
-	}
-	if (!(adp->states & CStatd)
-	    || !hsame(adp->m.DataVersion, tdc->f.versionNo)) {
-	    ReleaseReadLock(&tdc->lock);
-	    ReleaseReadLock(&adp->lock);
-	    afs_PutDCache(tdc);
-	    if (tname && tname != aname)
-		osi_FreeLargeSpace(tname);
-	    goto redo;
+	if (!afs_InReadDir(adp)) {
+	    while ((adp->states & CStatd)
+		   && (tdc->dflags & DFFetching)
+		   && hsame(adp->m.DataVersion, tdc->f.versionNo)) {
+		ReleaseReadLock(&tdc->lock);
+		ReleaseReadLock(&adp->lock);
+		afs_osi_Sleep(&tdc->validPos);
+		ObtainReadLock(&adp->lock);
+		ObtainReadLock(&tdc->lock);
+	    }
+	    if (!(adp->states & CStatd)
+		|| !hsame(adp->m.DataVersion, tdc->f.versionNo)) {
+		ReleaseReadLock(&tdc->lock);
+		ReleaseReadLock(&adp->lock);
+		afs_PutDCache(tdc);
+		if (tname && tname != aname)
+		    osi_FreeLargeSpace(tname);
+		goto redo;
+	    }
 	}
 
 	/* Save the version number for when we call osi_dnlc_enter */
@@ -1351,20 +1498,20 @@ afs_lookup(adp, aname, avcp, acred)
 
 	/* lookup the name in the appropriate dir, and return a cache entry
 	 * on the resulting fid */
-	theDir = &tdc->f;
 	code =
-	    afs_dir_LookupOffset(theDir, sysState.name, &tfid.Fid,
+	    afs_dir_LookupOffset(tdc, sysState.name, &tfid.Fid,
 				 &dirCookie);
 
 	/* If the first lookup doesn't succeed, maybe it's got @sys in the name */
 	while (code == ENOENT && Next_AtSys(adp, &treq, &sysState))
 	    code =
-		afs_dir_LookupOffset(theDir, sysState.name, &tfid.Fid,
+		afs_dir_LookupOffset(tdc, sysState.name, &tfid.Fid,
 				     &dirCookie);
 	tname = sysState.name;
 
 	ReleaseReadLock(&tdc->lock);
-	afs_PutDCache(tdc);
+	if (!afs_InReadDir(adp))
+	    afs_PutDCache(tdc);
 
 	if (code == ENOENT && afs_IsDynroot(adp) && dynrootRetry) {
 	    ReleaseReadLock(&adp->lock);
@@ -1398,7 +1545,7 @@ afs_lookup(adp, aname, avcp, acred)
 	 * dirCookie tells us where to start prefetching from.
 	 */
 	if (AFSDOBULK && adp->opens > 0 && !(adp->states & CForeign)
-	    && !afs_IsDynroot(adp)) {
+	    && !afs_IsDynroot(adp) && !afs_InReadDir(adp)) {
 	    afs_int32 retry;
 	    /* if the entry is not in the cache, or is in the cache,
 	     * but hasn't been statd, then do a bulk stat operation.
@@ -1534,7 +1681,7 @@ afs_lookup(adp, aname, avcp, acred)
 		}
 	    }
 	*avcp = tvc;
-	if (tvc && !VREFCOUNT(tvc)) {
+	if (tvc && !VREFCOUNT_GT(tvc, 0)) {
 	    osi_Panic("TT3");
 	}
 	code = 0;
@@ -1605,8 +1752,8 @@ afs_lookup(adp, aname, avcp, acred)
     }
     if (bulkcode)
 	code = bulkcode;
-    else
-	code = afs_CheckCode(code, &treq, 19);
+
+    code = afs_CheckCode(code, &treq, 19);
     if (code) {
 	/* If there is an error, make sure *avcp is null.
 	 * Alphas panic otherwise - defect 10719.
