@@ -19,7 +19,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_read.c,v 1.25 2003/08/27 21:43:19 rees Exp $");
+    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_read.c,v 1.34.2.1 2007/11/29 18:34:32 shadow Exp $");
 
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
 #include "afsincludes.h"	/* Afs-based standard headers */
@@ -27,6 +27,7 @@ RCSID
 #include "afs/afs_cbqueue.h"
 #include "afs/nfsclient.h"
 #include "afs/afs_osidnlc.h"
+#include "afs/afs_osi.h"
 
 
 extern char afs_zeros[AFS_ZEROS];
@@ -59,8 +60,13 @@ afs_MemRead(register struct vcache *avc, struct uio *auio,
     afs_int32 trimlen;
     struct dcache *tdc = 0;
     afs_int32 error, trybusy = 1;
+#ifdef AFS_DARWIN80_ENV
+    uio_t tuiop = NULL;
+#else
     struct uio tuio;
+    struct uio *tuiop = &tuio;
     struct iovec *tvec;
+#endif
     afs_int32 code;
     struct vrequest treq;
 
@@ -88,9 +94,11 @@ afs_MemRead(register struct vcache *avc, struct uio *auio,
     }
 #endif
 
+#ifndef AFS_DARWIN80_ENV
     tvec = (struct iovec *)osi_AllocSmallSpace(sizeof(struct iovec));
-    totalLength = auio->afsio_resid;
-    filePos = auio->afsio_offset;
+#endif
+    totalLength = AFS_UIO_RESID(auio);
+    filePos = AFS_UIO_OFFSET(auio);
     afs_Trace4(afs_iclSetp, CM_TRACE_READ, ICL_TYPE_POINTER, avc,
 	       ICL_TYPE_OFFSET, ICL_HANDLE_OFFSET(filePos), ICL_TYPE_INT32,
 	       totalLength, ICL_TYPE_OFFSET,
@@ -109,7 +117,21 @@ afs_MemRead(register struct vcache *avc, struct uio *auio,
      * Locks held:
      * avc->lock(R)
      */
-    while (totalLength > 0) {
+    if (filePos >= avc->m.Length) {
+	if (len > AFS_ZEROS)
+	    len = sizeof(afs_zeros);	/* and in 0 buffer */
+#ifdef AFS_DARWIN80_ENV
+	trimlen = len;
+	tuiop = afsio_darwin_partialcopy(auio, trimlen);
+#else
+	afsio_copy(auio, &tuio, tvec);
+	trimlen = len;
+	afsio_trim(&tuio, trimlen);
+#endif
+	AFS_UIOMOVE(afs_zeros, trimlen, UIO_READ, tuiop, code);
+    }
+
+    while (avc->m.Length > 0 && totalLength > 0) {
 	/* read all of the cached info */
 	if (filePos >= avc->m.Length)
 	    break;		/* all done */
@@ -300,10 +322,15 @@ afs_MemRead(register struct vcache *avc, struct uio *auio,
 		len = tlen;
 	    if (len > AFS_ZEROS)
 		len = sizeof(afs_zeros);	/* and in 0 buffer */
+#ifdef AFS_DARWIN80_ENV
+	    trimlen = len;
+            tuiop = afsio_darwin_partialcopy(auio, trimlen);
+#else
 	    afsio_copy(auio, &tuio, tvec);
 	    trimlen = len;
 	    afsio_trim(&tuio, trimlen);
-	    AFS_UIOMOVE(afs_zeros, trimlen, UIO_READ, &tuio, code);
+#endif
+	    AFS_UIOMOVE(afs_zeros, trimlen, UIO_READ, tuiop, code);
 	    if (code) {
 		error = code;
 		break;
@@ -312,12 +339,18 @@ afs_MemRead(register struct vcache *avc, struct uio *auio,
 	    /* get the data from the mem cache */
 
 	    /* mung uio structure to be right for this transfer */
+#ifdef AFS_DARWIN80_ENV
+	    trimlen = len;
+            tuiop = afsio_darwin_partialcopy(auio, trimlen);
+	    uio_setoffset(tuiop, offset);
+#else
 	    afsio_copy(auio, &tuio, tvec);
 	    trimlen = len;
 	    afsio_trim(&tuio, trimlen);
 	    tuio.afsio_offset = offset;
+#endif
 
-	    code = afs_MemReadUIO(tdc->f.inode, &tuio);
+	    code = afs_MemReadUIO(tdc->f.inode, tuiop);
 
 	    if (code) {
 		error = code;
@@ -325,7 +358,7 @@ afs_MemRead(register struct vcache *avc, struct uio *auio,
 	    }
 	}
 	/* otherwise we've read some, fixup length, etc and continue with next seg */
-	len = len - tuio.afsio_resid;	/* compute amount really transferred */
+	len = len - AFS_UIO_RESID(tuiop);	/* compute amount really transferred */
 	trimlen = len;
 	afsio_skip(auio, trimlen);	/* update input uio structure */
 	totalLength -= len;
@@ -334,6 +367,12 @@ afs_MemRead(register struct vcache *avc, struct uio *auio,
 
 	if (len <= 0)
 	    break;		/* surprise eof */
+#ifdef AFS_DARWIN80_ENV
+	if (tuiop) {
+	    uio_free(tuiop);
+	    tuiop = 0;
+	}
+#endif
     }				/* the whole while loop */
 
     /*
@@ -358,7 +397,12 @@ afs_MemRead(register struct vcache *avc, struct uio *auio,
     }
     if (!noLock)
 	ReleaseReadLock(&avc->lock);
+#ifdef AFS_DARWIN80_ENV
+    if (tuiop)
+       uio_free(tuiop);
+#else
     osi_FreeSmallSpace(tvec);
+#endif
     error = afs_CheckCode(error, &treq, 10);
     return error;
 }
@@ -447,8 +491,13 @@ afs_UFSRead(register struct vcache *avc, struct uio *auio,
     afs_int32 trimlen;
     struct dcache *tdc = 0;
     afs_int32 error;
+#ifdef AFS_DARWIN80_ENV
+    uio_t tuiop=NULL;
+#else
     struct uio tuio;
+    struct uio *tuiop = &tuio;
     struct iovec *tvec;
+#endif
     struct osi_file *tfile;
     afs_int32 code;
     int trybusy = 1;
@@ -482,9 +531,11 @@ afs_UFSRead(register struct vcache *avc, struct uio *auio,
     }
 #endif
 
+#ifndef AFS_DARWIN80_ENV
     tvec = (struct iovec *)osi_AllocSmallSpace(sizeof(struct iovec));
-    totalLength = auio->afsio_resid;
-    filePos = auio->afsio_offset;
+#endif
+    totalLength = AFS_UIO_RESID(auio);
+    filePos = AFS_UIO_OFFSET(auio);
     afs_Trace4(afs_iclSetp, CM_TRACE_READ, ICL_TYPE_POINTER, avc,
 	       ICL_TYPE_OFFSET, ICL_HANDLE_OFFSET(filePos), ICL_TYPE_INT32,
 	       totalLength, ICL_TYPE_OFFSET,
@@ -499,7 +550,22 @@ afs_UFSRead(register struct vcache *avc, struct uio *auio,
     }
 #endif
 
-    while (totalLength > 0) {
+    if (filePos >= avc->m.Length) {
+	if (len > AFS_ZEROS)
+	    len = sizeof(afs_zeros);	/* and in 0 buffer */
+	len = 0;
+#ifdef AFS_DARWIN80_ENV
+	trimlen = len;
+	tuiop = afsio_darwin_partialcopy(auio, trimlen);
+#else
+	afsio_copy(auio, &tuio, tvec);
+	trimlen = len;
+	afsio_trim(&tuio, trimlen);
+#endif
+	AFS_UIOMOVE(afs_zeros, trimlen, UIO_READ, tuiop, code);
+    }
+
+    while (avc->m.Length > 0 && totalLength > 0) {
 	/* read all of the cached info */
 	if (filePos >= avc->m.Length)
 	    break;		/* all done */
@@ -685,10 +751,15 @@ afs_UFSRead(register struct vcache *avc, struct uio *auio,
 		len = tlen;
 	    if (len > AFS_ZEROS)
 		len = sizeof(afs_zeros);	/* and in 0 buffer */
+#ifdef AFS_DARWIN80_ENV
+	    trimlen = len;
+            tuiop = afsio_darwin_partialcopy(auio, trimlen);
+#else
 	    afsio_copy(auio, &tuio, tvec);
 	    trimlen = len;
 	    afsio_trim(&tuio, trimlen);
-	    AFS_UIOMOVE(afs_zeros, trimlen, UIO_READ, &tuio, code);
+#endif
+	    AFS_UIOMOVE(afs_zeros, trimlen, UIO_READ, tuiop, code);
 	    if (code) {
 		error = code;
 		break;
@@ -711,11 +782,18 @@ afs_UFSRead(register struct vcache *avc, struct uio *auio,
 #endif /* IHINT */
 
 		tfile = (struct osi_file *)osi_UFSOpen(tdc->f.inode);
+#ifdef AFS_DARWIN80_ENV
+	    trimlen = len;
+            tuiop = afsio_darwin_partialcopy(auio, trimlen);
+	    uio_setoffset(tuiop, offset);
+#else
 	    /* mung uio structure to be right for this transfer */
 	    afsio_copy(auio, &tuio, tvec);
 	    trimlen = len;
 	    afsio_trim(&tuio, trimlen);
 	    tuio.afsio_offset = offset;
+#endif
+
 #if defined(AFS_AIX41_ENV)
 	    AFS_GUNLOCK();
 	    code =
@@ -755,9 +833,19 @@ afs_UFSRead(register struct vcache *avc, struct uio *auio,
 			  &tuio, NULL, NULL, -1);
 #elif defined(AFS_SUN5_ENV)
 	    AFS_GUNLOCK();
+#ifdef AFS_SUN510_ENV
+	    {
+		caller_context_t ct;
+
+		VOP_RWLOCK(tfile->vnode, 0, &ct);
+		code = VOP_READ(tfile->vnode, &tuio, 0, afs_osi_credp, &ct);
+		VOP_RWUNLOCK(tfile->vnode, 0, &ct);
+	    }
+#else
 	    VOP_RWLOCK(tfile->vnode, 0);
 	    code = VOP_READ(tfile->vnode, &tuio, 0, afs_osi_credp);
 	    VOP_RWUNLOCK(tfile->vnode, 0);
+#endif
 	    AFS_GLOCK();
 #elif defined(AFS_SGI_ENV)
 	    AFS_GUNLOCK();
@@ -771,15 +859,17 @@ afs_UFSRead(register struct vcache *avc, struct uio *auio,
 	    AFS_GUNLOCK();
 	    VOP_READ(tfile->vnode, &tuio, 0, afs_osi_credp, code);
 	    AFS_GLOCK();
-#elif defined(AFS_SUN_ENV)
-	    code = VOP_RDWR(tfile->vnode, &tuio, UIO_READ, 0, afs_osi_credp);
 #elif defined(AFS_HPUX100_ENV)
 	    AFS_GUNLOCK();
 	    code = VOP_RDWR(tfile->vnode, &tuio, UIO_READ, 0, afs_osi_credp);
 	    AFS_GLOCK();
 #elif defined(AFS_LINUX20_ENV)
 	    AFS_GUNLOCK();
-	    code = osi_file_uio_rdwr(tfile, &tuio, UIO_READ);
+	    code = osi_rdwr(tfile, &tuio, UIO_READ);
+	    AFS_GLOCK();
+#elif defined(AFS_DARWIN80_ENV)
+	    AFS_GUNLOCK();
+	    code = VNOP_READ(tfile->vnode, tuiop, 0, afs_osi_ctxtp);
 	    AFS_GLOCK();
 #elif defined(AFS_DARWIN_ENV)
 	    AFS_GUNLOCK();
@@ -817,7 +907,7 @@ afs_UFSRead(register struct vcache *avc, struct uio *auio,
 	    }
 	}
 	/* otherwise we've read some, fixup length, etc and continue with next seg */
-	len = len - tuio.afsio_resid;	/* compute amount really transferred */
+	len = len - AFS_UIO_RESID(tuiop);	/* compute amount really transferred */
 	trimlen = len;
 	afsio_skip(auio, trimlen);	/* update input uio structure */
 	totalLength -= len;
@@ -825,6 +915,12 @@ afs_UFSRead(register struct vcache *avc, struct uio *auio,
 	filePos += len;
 	if (len <= 0)
 	    break;		/* surprise eof */
+#ifdef AFS_DARWIN80_ENV
+	if (tuiop) {
+	    uio_free(tuiop);
+	    tuiop = 0;
+	}
+#endif
     }
 
     /* if we make it here with tdc non-zero, then it is the last chunk we
@@ -845,7 +941,12 @@ afs_UFSRead(register struct vcache *avc, struct uio *auio,
     if (!noLock)
 	ReleaseReadLock(&avc->lock);
 
+#ifdef AFS_DARWIN80_ENV
+    if (tuiop)
+       uio_free(tuiop);
+#else
     osi_FreeSmallSpace(tvec);
+#endif
     error = afs_CheckCode(error, &treq, 13);
     return error;
 }
