@@ -11,7 +11,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/afs_daemons.c,v 1.43.2.1 2007/10/23 00:02:50 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/afs_daemons.c,v 1.43.2.2 2007/11/08 14:40:12 shadow Exp $");
 
 #ifdef AFS_AIX51_ENV
 #define __FULL_PROTO
@@ -123,12 +123,9 @@ afs_CheckServerDaemon(void)
     }
     afs_CheckServerDaemonStarted = 0;
 }
-#define RECURSIVE_VFS_CONTEXT 1
-#if RECURSIVE_VFS_CONTEXT
+
 extern int vfs_context_ref;
-#else
-#define vfs_context_ref 1
-#endif
+
 void
 afs_Daemon(void)
 {
@@ -153,10 +150,8 @@ afs_Daemon(void)
         osi_Panic("vfs context already initialized");
     while (afs_osi_ctxtp && vfs_context_ref)
         afs_osi_Sleep(&afs_osi_ctxtp);
-#if RECURSIVE_VFS_CONTEXT
     if (afs_osi_ctxtp && !vfs_context_ref)
        vfs_context_rele(afs_osi_ctxtp);
-#endif
     afs_osi_ctxtp = vfs_context_create(NULL);
     afs_osi_ctxtp_initialized = 1;
 #endif
