@@ -45,7 +45,7 @@
 typedef struct timeval osi_timeval_t;
 #endif /* !KERNEL */
 
-#define XSTATS_DECLS struct afs_stats_opTimingData *opP; \
+#define XSTATS_DECLS struct afs_stats_opTimingData *opP = NULL; \
     osi_timeval_t opStartTime, opStopTime, elapsedTime
 
 #define XSTATS_START_TIME(arg) \
@@ -802,10 +802,14 @@ struct afs_stats_CMPerf {
     afs_uint32 cbloops;
     afs_uint32 osiread_efaults;
     afs_int32 cacheBlocksDiscarded;	/*# cache blocks free but not truncated */
+    afs_int32 cacheBucket0_Discarded;  
+    afs_int32 cacheBucket1_Discarded;  
+    afs_int32 cacheBucket2_Discarded;  
+
     /*
      * Spares for future expansion.
      */
-    afs_int32 spare[13];	/*Spares */
+    afs_int32 spare[10];	/*Spares */
 };
 
 
