@@ -15,7 +15,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/bucoord/dump.c,v 1.12 2003/12/07 22:49:19 jaltman Exp $");
+    ("$Header: /cvs/openafs/src/bucoord/dump.c,v 1.13.6.3 2007/11/26 21:08:41 shadow Exp $");
 
 #include <sys/types.h>
 #include <afs/cmd.h>
@@ -23,7 +23,6 @@ RCSID
 #include <winsock2.h>
 #else
 #include <sys/param.h>
-#include <strings.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <netdb.h>
@@ -52,11 +51,7 @@ extern afs_int32 bc_jobNumber();
 
 extern afs_int32 lastTaskCode;
 
-#ifdef AFS_DEC_ENV
-#define HOSTADDR(sockaddr) (sockaddr)->sin_addr.S_un.S_addr
-#else
 #define HOSTADDR(sockaddr) (sockaddr)->sin_addr.s_addr
-#endif
 
 /* bc_Dumper
  *	called (indirectly) to make a dump
@@ -99,7 +94,7 @@ bc_Dumper(aindex)
     volDesc =
 	(struct tc_dumpDesc *)malloc(count * sizeof(struct tc_dumpDesc));
     if (!volDesc) {
-	com_err(whoami, BC_NOMEM, "");
+	afs_com_err(whoami, BC_NOMEM, "");
 	ERROR(BC_NOMEM);
     }
 
@@ -149,11 +144,11 @@ bc_Dumper(aindex)
     printf("Starting dump\n");
     code = TC_PerformDump(tconn, tcdiPtr, &volArray, &dumpTaskPtr->dumpID);
     if (code) {
-	com_err(whoami, code, "; Failed to start dump");
+	afs_com_err(whoami, code, "; Failed to start dump");
 	ERROR(code);
     }
 
-    com_err(whoami, 0, "Task %u: Dump (%s)", dumpTaskPtr->dumpID,
+    afs_com_err(whoami, 0, "Task %u: Dump (%s)", dumpTaskPtr->dumpID,
 	    tcdiPtr->dumpName);
 
     /* create status monitor block */
@@ -183,6 +178,7 @@ bc_Dumper(aindex)
  *	free the list of volumes used for dumps
  */
 
+void
 freeDumpTaskVolumeList(vdptr)
      struct bc_volumeDump *vdptr;
 {
@@ -271,7 +267,7 @@ bc_StartDmpRst(aconfig, adname, avname, avolsToDump, adestServer,
 	    break;
 
     if (i >= BC_MAXSIMDUMPS) {
-	com_err(whoami, BC_NOTLOCKED,
+	afs_com_err(whoami, BC_NOTLOCKED,
 		"All of the dump/restore slots are in use, try again later");
 	return (BC_NOTLOCKED);
     }
@@ -310,7 +306,7 @@ bc_StartDmpRst(aconfig, adname, avname, avolsToDump, adestServer,
 			  (void *)i, "helper", &junk);
     if (code) {
 	bc_HandleMisc(code);
-	com_err(whoami, code, "; Can't start thread");
+	afs_com_err(whoami, code, "; Can't start thread");
 
 	/* Cleanup allocated data structures */
 	freeDumpTaskVolumeList(bc_dumpTasks[i].volumes);
@@ -389,7 +385,7 @@ bc_LabelTape(afsname, pname, size, config, port)
 
     code = TC_LabelTape(tconn, &label, &taskId);
     if (code) {
-	com_err(whoami, code, "; Failed to start labeltape");
+	afs_com_err(whoami, code, "; Failed to start labeltape");
 	return (code);
     }
 
@@ -435,7 +431,7 @@ bc_ReadLabel(config, port)
 	    printf("Tape read was unlabelled\n");
 	    return 0;
 	}
-	com_err(whoami, code, "; Failed to start readlabel");
+	afs_com_err(whoami, code, "; Failed to start readlabel");
 	return (code);
     }
 
@@ -473,7 +469,7 @@ bc_ScanDumps(config, dbAddFlag, port)
 
     code = TC_ScanDumps(tconn, dbAddFlag, &taskId);
     if (code) {
-	com_err(whoami, code, "; Failed to start scantape");
+	afs_com_err(whoami, code, "; Failed to start scantape");
 	return (code);
     }
 
@@ -564,7 +560,7 @@ ConnectButc(config, port, tconn)
 
     code = bc_GetConn(config, port, tconn);
     if (code) {
-	com_err(whoami, code,
+	afs_com_err(whoami, code,
 		"; Can't connect to tape coordinator at port %d", port);
 	return (code);
     }
@@ -574,10 +570,10 @@ ConnectButc(config, port, tconn)
 	rx_DestroyConnection(*tconn);
 
 	if (code == BC_VERSIONFAIL)
-	    com_err(whoami, code,
+	    afs_com_err(whoami, code,
 		    "; Backup and butc are not the same version");
 	else
-	    com_err(whoami, code,
+	    afs_com_err(whoami, code,
 		    "; Can't access tape coordinator at port %d", port);
 
 	return (code);
