@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/budb/database.c,v 1.7 2003/07/15 23:14:48 shadow Exp $");
+    ("$Header: /cvs/openafs/src/budb/database.c,v 1.7.14.3 2007/10/30 15:16:37 shadow Exp $");
 
 #ifdef AFS_NT40_ENV
 #include <winsock2.h>
@@ -21,21 +21,12 @@ RCSID
 #include <sys/types.h>
 #include <afs/stds.h>
 #include <ubik.h>
-#include <afs/auth.h>
 #include <afs/bubasics.h>
 #include "budb_errs.h"
 #include "database.h"
 #include "error_macros.h"
 #include "afs/audit.h"
-
-#ifdef HAVE_STRING_H
 #include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
-
 
 int pollCount;
 struct memoryDB db;		/* really allocate it here */
@@ -75,11 +66,7 @@ InitDB()
  */
 
 afs_int32
-dbwrite(ut, pos, buff, len)
-     struct ubik_trans *ut;
-     afs_int32 pos;
-     char *buff;
-     afs_int32 len;
+dbwrite(struct ubik_trans *ut, afs_int32 pos, void *buff, afs_int32 len)
 {
     afs_int32 code = 0;
 
@@ -111,11 +98,7 @@ dbwrite(ut, pos, buff, len)
 /* same thing for read */
 
 afs_int32
-dbread(ut, pos, buff, len)
-     struct ubik_trans *ut;
-     afs_int32 pos;
-     char *buff;
-     afs_int32 len;
+dbread(struct ubik_trans *ut, afs_int32 pos, void *buff, afs_int32 len)
 {
     afs_int32 code = 0;
 
@@ -146,12 +129,7 @@ dbread(ut, pos, buff, len)
 
 /* Same as dbread excepts it does checking */
 afs_int32
-cdbread(ut, type, pos, buff, len)
-     struct ubik_trans *ut;
-     int type;
-     afs_int32 pos;
-     char *buff;
-     afs_int32 len;
+cdbread(struct ubik_trans *ut, int type, afs_int32 pos, void *buff, afs_int32 len)
 {
     afs_int32 code = 0;
 
