@@ -10,7 +10,8 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID("$Header: /cvs/openafs/src/budb/struct_ops.c,v 1.5 2001/08/08 00:03:40 shadow Exp $");
+RCSID
+    ("$Header: /cvs/openafs/src/budb/struct_ops.c,v 1.8.2.1 2007/11/26 21:08:41 shadow Exp $");
 
 #include <stdio.h>
 #ifdef AFS_NT40_ENV
@@ -35,58 +36,67 @@ RCSID("$Header: /cvs/openafs/src/budb/struct_ops.c,v 1.5 2001/08/08 00:03:40 sha
  * ----------------------------------
  */
 
-printDbHeader(ptr)
-     struct DbHeader *ptr;
+void
+printDbHeader(struct DbHeader *ptr)
 {
+    time_t created = ptr->created;
     printf("version = %d\n", ptr->dbversion);
-    printf("created = %s", ctime((time_t *)&ptr->created));
-    printf("cell = %s\n",ptr->cell);
+    printf("created = %s", ctime(&created));
+    printf("cell = %s\n", ptr->cell);
     printf("lastDumpId = %u\n", ptr->lastDumpId);
     printf("lastInstanceId = %d\n", ptr->lastInstanceId);
     printf("lastTapeId = %d\n", ptr->lastTapeId);
 }
 
-printDump(fid, dptr)
-     FILE *fid;
-     struct dump *dptr;
+void
+printDump(FILE *fid, struct dump *dptr)
 {
-    fprintf(fid,"id = %u\n", dptr->id);
-    fprintf(fid,"idHashChain = %d\n", dptr->idHashChain);
-    fprintf(fid,"name = %s\n", dptr->dumpName);
-    fprintf(fid,"vsname = %s\n", dptr->volumeSet);
-    fprintf(fid,"dumpPath = %s\n", dptr->dumpPath);
-    fprintf(fid,"nameHashChain = %d\n", dptr->nameHashChain);
-    fprintf(fid,"flags = 0x%x\n", dptr->flags);
-    fprintf(fid,"parent = %u\n", dptr->parent);
-    fprintf(fid,"created = %s", ctime((time_t *)&dptr->created));
-    fprintf(fid,"nVolumes = %d\n", dptr->nVolumes);
+    time_t created = dptr->created;
+    fprintf(fid, "id = %u\n", dptr->id);
+    fprintf(fid, "idHashChain = %d\n", dptr->idHashChain);
+    fprintf(fid, "name = %s\n", dptr->dumpName);
+    fprintf(fid, "vsname = %s\n", dptr->volumeSet);
+    fprintf(fid, "dumpPath = %s\n", dptr->dumpPath);
+    fprintf(fid, "nameHashChain = %d\n", dptr->nameHashChain);
+    fprintf(fid, "flags = 0x%x\n", dptr->flags);
+    fprintf(fid, "parent = %u\n", dptr->parent);
+    fprintf(fid, "created = %s", ctime(&created));
+    fprintf(fid, "nVolumes = %d\n", dptr->nVolumes);
     /* printTapeSet(&dptr->tapes); */
-    fprintf(fid,"firstTape = %d\n", dptr->firstTape);
+    fprintf(fid, "firstTape = %d\n", dptr->firstTape);
     /* printKtcPrincipal(&dptr->dumper); */
 
 }
 
-printDumpEntry(deptr)
-     struct budb_dumpEntry *deptr;
+void 
+printDumpEntry(struct budb_dumpEntry *deptr)
 {
+    time_t created = deptr->created;
     printf("id = %u\n", deptr->id);
     printf("Initial id = %u\n", deptr->initialDumpID);
     printf("Appended id = %u\n", deptr->appendedDumpID);
     printf("parent = %u\n", deptr->parent);
     printf("level = %d\n", deptr->level);
     printf("flags = 0x%x", deptr->flags);
-       if (deptr->flags == 0)                   printf(": Successful");
-       if (deptr->flags & BUDB_DUMP_INCOMPLETE) printf(": Incomplete");
-       if (deptr->flags & BUDB_DUMP_TAPEERROR)  printf(": Tape error");
-       if (deptr->flags & BUDB_DUMP_INPROGRESS) printf(": In progress");
-       if (deptr->flags & BUDB_DUMP_ABORTED)    printf(": Aborted");
-       if (deptr->flags & BUDB_DUMP_ADSM)       printf(": (ADSM)");       /* XBSA interface to ADSM */
-       if (deptr->flags & BUDB_DUMP_BUTA)       printf(": (BUTA)");       /* buta dump */
-       printf("\n");
+    if (deptr->flags == 0)
+	printf(": Successful");
+    if (deptr->flags & BUDB_DUMP_INCOMPLETE)
+	printf(": Incomplete");
+    if (deptr->flags & BUDB_DUMP_TAPEERROR)
+	printf(": Tape error");
+    if (deptr->flags & BUDB_DUMP_INPROGRESS)
+	printf(": In progress");
+    if (deptr->flags & BUDB_DUMP_ABORTED)
+	printf(": Aborted");
+    if (deptr->flags & BUDB_DUMP_ADSM)
+	printf(": (ADSM)");	/* XBSA interface to ADSM */
+    if (deptr->flags & BUDB_DUMP_BUTA)
+	printf(": (BUTA)");	/* buta dump */
+    printf("\n");
     printf("volumeSet = %s\n", deptr->volumeSetName);
     printf("dump path = %s\n", deptr->dumpPath);
     printf("name = %s\n", deptr->name);
-    printf("created = %s", ctime((time_t *)&deptr->created));
+    printf("created = %s", ctime(&created));
     printf("nVolumes = %d\n", deptr->nVolumes);
 
     printTapeSet(&deptr->tapes, (deptr->flags & BUDB_DUMP_XBSA_NSS));
@@ -97,84 +107,101 @@ printDumpEntry(deptr)
  *	print the hash table structure, i.e. the header structure.
  */
 
+void
 printHashTable(fid, htptr)
      FILE *fid;
      struct hashTable *htptr;
 {
-    fprintf(fid,"functionType = %d\n", htptr->functionType);
-    fprintf(fid,"threadOffset = %d\n", htptr->threadOffset);
-    fprintf(fid,"entries = %d\n", htptr->entries);
-    fprintf(fid,"length = %d\n", htptr->length);
-    fprintf(fid,"table = %d\n", htptr->table);
-    fprintf(fid,"progress = %d\n", htptr->progress);
-    fprintf(fid,"oldLength = %d\n", htptr->oldLength);
-    fprintf(fid,"oldTable = %d\n", htptr->oldTable);
+    fprintf(fid, "functionType = %d\n", htptr->functionType);
+    fprintf(fid, "threadOffset = %d\n", htptr->threadOffset);
+    fprintf(fid, "entries = %d\n", htptr->entries);
+    fprintf(fid, "length = %d\n", htptr->length);
+    fprintf(fid, "table = %d\n", htptr->table);
+    fprintf(fid, "progress = %d\n", htptr->progress);
+    fprintf(fid, "oldLength = %d\n", htptr->oldLength);
+    fprintf(fid, "oldTable = %d\n", htptr->oldTable);
 }
 
 /* printMemoryHashTable
  *	print the hash table structure, i.e. the header structure.
  */
+int
 printMemoryHashTable(fid, mhtptr)
      FILE *fid;
      struct memoryHashTable *mhtptr;
 {
-    fprintf(fid,"threadOffset = %d\n", mhtptr->threadOffset);
-    fprintf(fid,"length = %d\n", mhtptr->length);
-    fprintf(fid,"progress = %d\n", mhtptr->progress);
-    fprintf(fid,"size = %d\n", mhtptr->size);
-    fprintf(fid,"oldsize = %d\n", mhtptr->oldSize);
+    fprintf(fid, "threadOffset = %d\n", mhtptr->threadOffset);
+    fprintf(fid, "length = %d\n", mhtptr->length);
+    fprintf(fid, "progress = %d\n", mhtptr->progress);
+    fprintf(fid, "size = %d\n", mhtptr->size);
+    fprintf(fid, "oldsize = %d\n", mhtptr->oldSize);
+    return 0;
 }
 
+int
 printPrincipal(ptr)
      struct ktc_principal *ptr;
 {
     printf("name = %s\n", ptr->name);
     printf("instance = %s\n", ptr->instance);
     printf("cell = %s\n", ptr->cell);
+    return 0;
 }
 
+int
 printStructDumpHeader(ptr)
      struct structDumpHeader *ptr;
 {
     printf("type = %d\n", ptr->type);
     printf("structure version = %d\n", ptr->structversion);
     printf("size = %d bytes\n", ptr->size);
+    return 0;
 }
 
-printTape(fid, tptr)
-     FILE *fid;
-     struct tape *tptr;
+int
+printTape(FILE *fid, struct tape *tptr)
 {
-    fprintf(fid,"name = %s\n", tptr->name);
-    fprintf(fid,"nameHashChain = %d\n", tptr->nameHashChain);
-    fprintf(fid,"flags = 0x%x\n", tptr->flags);
-    fprintf(fid,"written = %s", ctime((time_t *)&tptr->written));
-    fprintf(fid,"nMBytes = %d\n", tptr->nMBytes);
-    fprintf(fid,"nBytes = %d\n", tptr->nBytes);
-    fprintf(fid,"nFiles = %d\n", tptr->nFiles);
-    fprintf(fid,"nVolumes = %d\n", tptr->nVolumes);
-    fprintf(fid,"seq = %d\n", tptr->seq);
-    fprintf(fid,"dump = %d\n", tptr->dump);
-    fprintf(fid,"nextTape = %d\n", tptr->nextTape);
-    fprintf(fid,"firstVol = %d\n", tptr->firstVol);
-    fprintf(fid,"labelPos = %d\n", tptr->labelpos);
-    fprintf(fid,"useCount = %d\n", tptr->useCount);
+    time_t written = tptr->written;
+    fprintf(fid, "name = %s\n", tptr->name);
+    fprintf(fid, "nameHashChain = %d\n", tptr->nameHashChain);
+    fprintf(fid, "flags = 0x%x\n", tptr->flags);
+    fprintf(fid, "written = %s", ctime(&written));
+    fprintf(fid, "nMBytes = %d\n", tptr->nMBytes);
+    fprintf(fid, "nBytes = %d\n", tptr->nBytes);
+    fprintf(fid, "nFiles = %d\n", tptr->nFiles);
+    fprintf(fid, "nVolumes = %d\n", tptr->nVolumes);
+    fprintf(fid, "seq = %d\n", tptr->seq);
+    fprintf(fid, "dump = %d\n", tptr->dump);
+    fprintf(fid, "nextTape = %d\n", tptr->nextTape);
+    fprintf(fid, "firstVol = %d\n", tptr->firstVol);
+    fprintf(fid, "labelPos = %d\n", tptr->labelpos);
+    fprintf(fid, "useCount = %d\n", tptr->useCount);
+    return 0;
 }
 
-printTapeEntry(teptr)
-     struct budb_tapeEntry *teptr;
+int
+printTapeEntry(struct budb_tapeEntry *teptr)
 {
+    time_t written = teptr->written;
+    time_t expires = teptr->expires;
+
     printf("name = %s\n", teptr->name);
     printf("flags = 0x%x", teptr->flags);
-       if (teptr->flags & BUDB_TAPE_TAPEERROR)    printf(": Error");
-       if (teptr->flags & BUDB_TAPE_DELETED)      printf(": Deleted");
-       if (teptr->flags & BUDB_TAPE_BEINGWRITTEN) printf(": In progress");
-       if (teptr->flags & BUDB_TAPE_ABORTED)      printf(": Aborted");
-       if (teptr->flags & BUDB_TAPE_STAGED)       printf(": Staged");
-       if (teptr->flags & BUDB_TAPE_WRITTEN)      printf(": Successful");
-       printf("\n");
-    printf("written = %s", ctime((time_t *)&teptr->written));
-    printf("expires = %s", cTIME((time_t *)&teptr->expires));
+    if (teptr->flags & BUDB_TAPE_TAPEERROR)
+	printf(": Error");
+    if (teptr->flags & BUDB_TAPE_DELETED)
+	printf(": Deleted");
+    if (teptr->flags & BUDB_TAPE_BEINGWRITTEN)
+	printf(": In progress");
+    if (teptr->flags & BUDB_TAPE_ABORTED)
+	printf(": Aborted");
+    if (teptr->flags & BUDB_TAPE_STAGED)
+	printf(": Staged");
+    if (teptr->flags & BUDB_TAPE_WRITTEN)
+	printf(": Successful");
+    printf("\n");
+    printf("written = %s", ctime(&written));
+    printf("expires = %s", cTIME(&expires));
     printf("kBytes Tape Used = %u\n", teptr->useKBytes);
     printf("nMBytes Data = %d\n", teptr->nMBytes);
     printf("nBytes  Data = %d\n", teptr->nBytes);
@@ -184,80 +211,95 @@ printTapeEntry(teptr)
     printf("labelPos = %d\n", teptr->labelpos);
     printf("useCount = %d\n", teptr->useCount);
     printf("dump = %d\n", teptr->dump);
+    return 0;
 }
 
+int
 printTapeSet(tsptr, nss)
      struct budb_tapeSet *tsptr;
-     afs_int32 nss;                 /* is the tapeserver name an accurate name */
+     afs_int32 nss;		/* is the tapeserver name an accurate name */
 {
-    printf("Group id  = %d\n",tsptr->id );
-    printf("tapeServer = %s%s\n",
-	   tsptr->tapeServer, (nss?" (single server)":""));
-    printf("format = %s\n",tsptr->format);
-    printf("maxTapes = %d\n",tsptr->maxTapes);
+    printf("Group id  = %d\n", tsptr->id);
+    printf("tapeServer = %s%s\n", tsptr->tapeServer,
+	   (nss ? " (single server)" : ""));
+    printf("format = %s\n", tsptr->format);
+    printf("maxTapes = %d\n", tsptr->maxTapes);
 /*  printf("a  = %d\n",tsptr->a ); */
 /*  printf("b = %d\n",tsptr->b);   */
-    printf("Start Tape Seq = %d\n",tsptr->b);
+    printf("Start Tape Seq = %d\n", tsptr->b);
+    return 0;
 }
 
-printVolumeEntry(veptr)
-     struct budb_volumeEntry *veptr;
+int
+printVolumeEntry(struct budb_volumeEntry *veptr)
 {
+    time_t clone = veptr->clone;
     printf("name = %s\n", veptr->name);
     printf("flags = 0x%x", veptr->flags);
-       if (veptr->flags & BUDB_VOL_TAPEERROR) printf(": Tape Error");
-       if (veptr->flags & BUDB_VOL_FILEERROR) printf(": File Error");
-       if (veptr->flags & BUDB_VOL_BEINGWRITTEN) printf(": In progress");
-       if (veptr->flags & BUDB_VOL_FIRSTFRAG)    printf(": First fragment");
-       if (veptr->flags & BUDB_VOL_LASTFRAG)     printf(": Last fragment");
-       if (veptr->flags & BUDB_VOL_ABORTED)      printf(": Aborted");
-       printf("\n");
+    if (veptr->flags & BUDB_VOL_TAPEERROR)
+	printf(": Tape Error");
+    if (veptr->flags & BUDB_VOL_FILEERROR)
+	printf(": File Error");
+    if (veptr->flags & BUDB_VOL_BEINGWRITTEN)
+	printf(": In progress");
+    if (veptr->flags & BUDB_VOL_FIRSTFRAG)
+	printf(": First fragment");
+    if (veptr->flags & BUDB_VOL_LASTFRAG)
+	printf(": Last fragment");
+    if (veptr->flags & BUDB_VOL_ABORTED)
+	printf(": Aborted");
+    printf("\n");
     printf("id = %d\n", veptr->id);
     printf("server = %s\n", veptr->server);
     printf("partition = %d\n", veptr->partition);
     printf("tapeSeq = %d\n", veptr->tapeSeq);
 
     printf("position = %d\n", veptr->position);
-    printf("clone = %s", ctime((time_t *)&veptr->clone));
+    printf("clone = %s", ctime(&clone));
     printf("startByte = %d\n", veptr->startByte);
     printf("nBytes = %d\n", veptr->nBytes);
     printf("seq = %d\n", veptr->seq);
 
     printf("dump = %d\n", veptr->dump);
     printf("tape = %s\n", veptr->tape);
-}
-  
-printVolFragment(fid, vfptr)
-     FILE *fid;
-     struct volFragment *vfptr;
-{
-    fprintf(fid,"vol = %d\n", vfptr->vol);
-    fprintf(fid,"sameNameChain = %d\n", vfptr->sameNameChain);
-    fprintf(fid,"tape = %d\n", vfptr->tape);
-    fprintf(fid,"sameTapeChain = %d\n", vfptr->sameTapeChain);
-    fprintf(fid,"position = %d\n", vfptr->position);
-    fprintf(fid,"clone = %s", ctime((time_t *)&vfptr->clone));
-    fprintf(fid,"incTime = %s", ctime((time_t *)&vfptr->incTime));
-    fprintf(fid,"startByte = %d\n", vfptr->startByte);
-    fprintf(fid,"nBytes = %d\n", vfptr->nBytes);
-    fprintf(fid,"flags = %d\n", vfptr->flags);
-    fprintf(fid,"sequence = %d\n", vfptr->sequence);
+    return 0;
 }
 
+int
+printVolFragment(FILE *fid, struct volFragment *vfptr)
+{
+    time_t clone = vfptr->clone;
+    time_t incTime = vfptr->incTime;
+    fprintf(fid, "vol = %d\n", vfptr->vol);
+    fprintf(fid, "sameNameChain = %d\n", vfptr->sameNameChain);
+    fprintf(fid, "tape = %d\n", vfptr->tape);
+    fprintf(fid, "sameTapeChain = %d\n", vfptr->sameTapeChain);
+    fprintf(fid, "position = %d\n", vfptr->position);
+    fprintf(fid, "clone = %s", ctime(&clone));
+    fprintf(fid, "incTime = %s", ctime(&incTime));
+    fprintf(fid, "startByte = %d\n", vfptr->startByte);
+    fprintf(fid, "nBytes = %d\n", vfptr->nBytes);
+    fprintf(fid, "flags = %d\n", vfptr->flags);
+    fprintf(fid, "sequence = %d\n", vfptr->sequence);
+    return 0;
+}
+
+int
 printVolInfo(fid, viptr)
      FILE *fid;
      struct volInfo *viptr;
 {
-    fprintf(fid,"name = %s\n", viptr->name);
-    fprintf(fid,"nameHashChain = %d\n", viptr->nameHashChain);
-    fprintf(fid,"id = %d\n", viptr->id);
-    fprintf(fid,"server = %s\n", viptr->server);
-    fprintf(fid,"partition = %d\n", viptr->partition);
-    fprintf(fid,"flags = 0x%x\n", viptr->flags);
-    fprintf(fid,"sameNameHead = %d\n", viptr->sameNameHead);
-    fprintf(fid,"sameNameChain = %d\n", viptr->sameNameChain);
-    fprintf(fid,"firstFragment = %d\n", viptr->firstFragment);
-    fprintf(fid,"nFrags = %d\n", viptr->nFrags);
+    fprintf(fid, "name = %s\n", viptr->name);
+    fprintf(fid, "nameHashChain = %d\n", viptr->nameHashChain);
+    fprintf(fid, "id = %d\n", viptr->id);
+    fprintf(fid, "server = %s\n", viptr->server);
+    fprintf(fid, "partition = %d\n", viptr->partition);
+    fprintf(fid, "flags = 0x%x\n", viptr->flags);
+    fprintf(fid, "sameNameHead = %d\n", viptr->sameNameHead);
+    fprintf(fid, "sameNameChain = %d\n", viptr->sameNameChain);
+    fprintf(fid, "firstFragment = %d\n", viptr->firstFragment);
+    fprintf(fid, "nFrags = %d\n", viptr->nFrags);
+    return 0;
 }
 
 
@@ -270,6 +312,7 @@ printVolInfo(fid, viptr)
  *	currently used for debug only
  */
 
+void
 volFragment_ntoh(netVfPtr, hostVfPtr)
      struct volFragment *netVfPtr, *hostVfPtr;
 {
@@ -286,6 +329,7 @@ volFragment_ntoh(netVfPtr, hostVfPtr)
     hostVfPtr->sequence = ntohs(netVfPtr->sequence);
 }
 
+void
 volInfo_ntoh(netViPtr, hostViPtr)
      struct volInfo *netViPtr, *hostViPtr;
 {
@@ -301,6 +345,7 @@ volInfo_ntoh(netViPtr, hostViPtr)
     hostViPtr->nFrags = ntohl(netViPtr->nFrags);
 }
 
+void
 tape_ntoh(netTapePtr, hostTapePtr)
      struct tape *netTapePtr, *hostTapePtr;
 {
@@ -322,10 +367,11 @@ tape_ntoh(netTapePtr, hostTapePtr)
     hostTapePtr->useCount = ntohl(netTapePtr->useCount);
 }
 
+void
 dump_ntoh(netDumpPtr, hostDumpPtr)
      struct dump *netDumpPtr, *hostDumpPtr;
 {
-    hostDumpPtr->id = ntohl(netDumpPtr->id);    
+    hostDumpPtr->id = ntohl(netDumpPtr->id);
     hostDumpPtr->idHashChain = ntohl(netDumpPtr->idHashChain);
     strcpy(hostDumpPtr->dumpName, netDumpPtr->dumpName);
     strcpy(hostDumpPtr->dumpPath, netDumpPtr->dumpPath);
@@ -344,6 +390,7 @@ dump_ntoh(netDumpPtr, hostDumpPtr)
     principal_ntoh(&netDumpPtr->dumper, &hostDumpPtr->dumper);
 }
 
+void
 DbHeader_ntoh(netptr, hostptr)
      struct DbHeader *netptr, *hostptr;
 {
@@ -355,6 +402,7 @@ DbHeader_ntoh(netptr, hostptr)
     hostptr->lastTapeId = ntohl(netptr->lastTapeId);
 }
 
+void
 dumpEntry_ntoh(netptr, hostptr)
      struct budb_dumpEntry *netptr, *hostptr;
 {
@@ -381,6 +429,7 @@ principal_hton(hostptr, netptr)
     strcpy(netptr->name, hostptr->name);
     strcpy(netptr->instance, hostptr->instance);
     strcpy(netptr->cell, hostptr->cell);
+    return 0;
 }
 
 principal_ntoh(netptr, hostptr)
@@ -389,8 +438,10 @@ principal_ntoh(netptr, hostptr)
     strcpy(hostptr->name, netptr->name);
     strcpy(hostptr->instance, netptr->instance);
     strcpy(hostptr->cell, netptr->cell);
+    return 0;
 }
 
+void
 structDumpHeader_hton(hostPtr, netPtr)
      struct structDumpHeader *hostPtr, *netPtr;
 {
@@ -399,6 +450,7 @@ structDumpHeader_hton(hostPtr, netPtr)
     netPtr->size = htonl(hostPtr->size);
 }
 
+void
 structDumpHeader_ntoh(netPtr, hostPtr)
      struct structDumpHeader *hostPtr, *netPtr;
 {
@@ -407,6 +459,7 @@ structDumpHeader_ntoh(netPtr, hostPtr)
     hostPtr->size = ntohl(netPtr->size);
 }
 
+void
 tapeEntry_ntoh(netptr, hostptr)
      struct budb_tapeEntry *netptr, *hostptr;
 {
@@ -425,28 +478,33 @@ tapeEntry_ntoh(netptr, hostptr)
     hostptr->dump = ntohl(netptr->dump);
 }
 
+int
 tapeSet_hton(hostptr, netptr)
      struct budb_tapeSet *hostptr, *netptr;
 {
     netptr->id = htonl(hostptr->id);
-    strcpy(netptr->tapeServer, hostptr->tapeServer);    
-    strcpy(netptr->format, hostptr->format);    
+    strcpy(netptr->tapeServer, hostptr->tapeServer);
+    strcpy(netptr->format, hostptr->format);
     netptr->maxTapes = htonl(hostptr->maxTapes);
     netptr->a = htonl(hostptr->a);
     netptr->b = htonl(hostptr->b);
+    return 0;
 }
 
+int
 tapeSet_ntoh(netptr, hostptr)
      struct budb_tapeSet *netptr, *hostptr;
 {
     hostptr->id = ntohl(netptr->id);
-    strcpy(hostptr->tapeServer, netptr->tapeServer);    
-    strcpy(hostptr->format, netptr->format);    
+    strcpy(hostptr->tapeServer, netptr->tapeServer);
+    strcpy(hostptr->format, netptr->format);
     hostptr->maxTapes = ntohl(netptr->maxTapes);
     hostptr->a = ntohl(netptr->a);
     hostptr->b = ntohl(netptr->b);
+    return 0;
 }
 
+void
 textBlock_hton(hostptr, netptr)
      struct textBlock *hostptr, *netptr;
 {
@@ -457,6 +515,7 @@ textBlock_hton(hostptr, netptr)
     netptr->newTextAddr = htonl(hostptr->newTextAddr);
 }
 
+void
 textBlock_ntoh(netptr, hostptr)
      struct textBlock *netptr, *hostptr;
 {
@@ -467,8 +526,9 @@ textBlock_ntoh(netptr, hostptr)
     hostptr->newTextAddr = ntohl(netptr->newTextAddr);
 }
 
+void
 textLock_hton(hostptr, netptr)
-     db_lockP	hostptr, netptr;
+     db_lockP hostptr, netptr;
 {
     netptr->type = htonl(hostptr->type);
     netptr->lockState = htonl(hostptr->lockState);
@@ -478,8 +538,9 @@ textLock_hton(hostptr, netptr)
     netptr->lockHost = htonl(hostptr->lockHost);
 }
 
+void
 textLock_ntoh(netptr, hostptr)
-     db_lockP	netptr, hostptr;
+     db_lockP netptr, hostptr;
 {
     hostptr->type = ntohl(netptr->type);
     hostptr->lockState = ntohl(netptr->lockState);
@@ -489,6 +550,7 @@ textLock_ntoh(netptr, hostptr)
     hostptr->lockHost = ntohl(netptr->lockHost);
 }
 
+void
 volumeEntry_ntoh(netptr, hostptr)
      struct budb_volumeEntry *netptr, *hostptr;
 {
@@ -499,7 +561,7 @@ volumeEntry_ntoh(netptr, hostptr)
     hostptr->partition = ntohl(netptr->partition);
     hostptr->tapeSeq = ntohl(netptr->tapeSeq);
 
-    hostptr->position  = ntohl(netptr->position);
+    hostptr->position = ntohl(netptr->position);
     hostptr->clone = ntohl(netptr->clone);
     hostptr->incTime = ntohl(netptr->incTime);
     hostptr->startByte = ntohl(netptr->startByte);
@@ -515,7 +577,7 @@ volumeEntry_ntoh(netptr, hostptr)
  * -------------------------------------
  */
 
-
+void
 copy_ktcPrincipal_to_budbPrincipal(ktcPtr, budbPtr)
      struct ktc_principal *ktcPtr;
      struct budb_principal *budbPtr;
@@ -548,8 +610,9 @@ dumpToBudbDump(dumpPtr, budbDumpPtr)
     budbDumpPtr->nVolumes = dumpPtr->nVolumes;
 
     memcpy(&budbDumpPtr->tapes, &dumpPtr->tapes, sizeof(struct budb_tapeSet));
-    copy_ktcPrincipal_to_budbPrincipal(&dumpPtr->dumper, &budbDumpPtr->dumper);
-    return(0);
+    copy_ktcPrincipal_to_budbPrincipal(&dumpPtr->dumper,
+				       &budbDumpPtr->dumper);
+    return (0);
 }
 
 tapeToBudbTape(tapePtr, budbTapePtr)
@@ -568,7 +631,7 @@ tapeToBudbTape(tapePtr, budbTapePtr)
     budbTapePtr->labelpos = tapePtr->labelpos;
     budbTapePtr->useCount = tapePtr->useCount;
     budbTapePtr->useKBytes = tapePtr->useKBytes;
-    return(0);
+    return (0);
 }
 
 volsToBudbVol(volFragPtr, volInfoPtr, budbVolPtr)
@@ -581,7 +644,7 @@ volsToBudbVol(volFragPtr, volInfoPtr, budbVolPtr)
     budbVolPtr->id = volInfoPtr->id;
     strcpy(budbVolPtr->server, volInfoPtr->server);
     budbVolPtr->partition = volInfoPtr->partition;
-    budbVolPtr->tapeSeq = 0;      /* Don't know it so mark invalid */
+    budbVolPtr->tapeSeq = 0;	/* Don't know it so mark invalid */
 
     budbVolPtr->position = volFragPtr->position;
     budbVolPtr->clone = volFragPtr->clone;
@@ -589,7 +652,7 @@ volsToBudbVol(volFragPtr, volInfoPtr, budbVolPtr)
     budbVolPtr->startByte = volFragPtr->startByte;
     budbVolPtr->nBytes = volFragPtr->nBytes;
     budbVolPtr->seq = volFragPtr->sequence;
-    return(0);
+    return (0);
 }
 
 /* ----------------------------------
@@ -618,4 +681,5 @@ default_tapeset(tapesetPtr, dumpname)
     tapesetPtr->a = 1;
     tapesetPtr->b = 1;
     tapesetPtr->maxTapes = 0;
+    return 0;
 }
