@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/comerr/error_msg.c,v 1.5 2003/07/15 23:14:53 shadow Exp $");
+    ("$Header: /cvs/openafs/src/comerr/error_msg.c,v 1.6.8.1 2007/04/10 18:39:51 shadow Exp $");
 
 #include "internal.h"
 #include <stdio.h>
@@ -45,7 +45,7 @@ static pthread_once_t et_list_once = PTHREAD_ONCE_INIT;
  * Function to initialize the et_list_mutex
  */
 
-void
+static void
 et_mutex_once(void)
 {
     assert(!pthread_mutex_init
@@ -54,9 +54,11 @@ et_mutex_once(void)
 }
 
 #define LOCK_ET_LIST \
-	(et_list_done || pthread_once(&et_list_once, et_mutex_once)); \
-	assert(pthread_mutex_lock(&et_list_mutex)==0);
-#define UNLOCK_ET_LIST assert(pthread_mutex_unlock(&et_list_mutex)==0);
+	do { \
+	    (et_list_done || pthread_once(&et_list_once, et_mutex_once)); \
+	    assert(pthread_mutex_lock(&et_list_mutex)==0); \
+	} while (0)
+#define UNLOCK_ET_LIST assert(pthread_mutex_unlock(&et_list_mutex)==0)
 #else
 #define LOCK_ET_LIST
 #define UNLOCK_ET_LIST
@@ -108,7 +110,7 @@ volume_message(int code)
 }
 
 const char *
-error_message(afs_int32 code)
+afs_error_message(afs_int32 code)
 {
     int offset;
     struct et_list *et;
@@ -131,18 +133,21 @@ error_message(afs_int32 code)
 	else
 	    goto oops;
     }
-    LOCK_ET_LIST for (et = _et_list; et; et = et->next) {
+    LOCK_ET_LIST;
+    for (et = _et_list; et; et = et->next) {
 	if (et->table->base == table_num) {
 	    /* This is the right table */
 	    if (et->table->n_msgs <= offset)
 		goto oops;
-	    UNLOCK_ET_LIST return (et->table->msgs[offset]);
+	    UNLOCK_ET_LIST;
+	    return (et->table->msgs[offset]);
 	}
     }
   oops:
-    UNLOCK_ET_LIST strcpy(buffer, "Unknown code ");
+    UNLOCK_ET_LIST;
+    strcpy(buffer, "Unknown code ");
     if (table_num) {
-	strcat(buffer, error_table_name(table_num));
+	strcat(buffer, afs_error_table_name(table_num));
 	strcat(buffer, " ");
     }
     for (cp = buffer; *cp; cp++);
@@ -164,20 +169,22 @@ error_message(afs_int32 code)
 }
 
 void
-add_to_error_table(struct et_list *new_table)
+afs_add_to_error_table(struct et_list *new_table)
 {
     struct et_list *et;
 
-    LOCK_ET_LIST
-	/*
-	 * Protect against adding the same error table twice
-	 */
-	for (et = _et_list; et; et = et->next) {
+    LOCK_ET_LIST;
+    /*
+     * Protect against adding the same error table twice
+     */
+    for (et = _et_list; et; et = et->next) {
 	if (et->table->base == new_table->table->base) {
-	    UNLOCK_ET_LIST return;
+	    UNLOCK_ET_LIST;
+	    return;
 	}
     }
 
     new_table->next = _et_list;
     _et_list = new_table;
-UNLOCK_ET_LIST}
+    UNLOCK_ET_LIST;
+}
