@@ -38,7 +38,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/des/crypt.c,v 1.12 2003/11/29 22:08:12 jaltman Exp $");
+    ("$Header: /cvs/openafs/src/des/crypt.c,v 1.14.8.1 2006/07/25 14:47:41 shadow Exp $");
 
 #ifdef AFS_NT40_ENV
 #include <windows.h>
@@ -91,7 +91,7 @@ RCSID
  */
 
 /* XXX shouldn't this be !AFS_64BIT_ENV ? */
-#if !defined(cray) && !defined(AFS_ALPHA_LINUX20_ENV) && !defined(AFS_IA64_LINUX20_ENV) && !defined(AFS_AMD64_LINUX20_ENV)
+#if !defined(cray) && !defined(AFS_ALPHA_LINUX20_ENV) && !defined(AFS_IA64_LINUX20_ENV) && !defined(AFS_AMD64_LINUX20_ENV) && !defined(AFS_PPC64_LINUX20_ENV) & !defined(AFS_S390X_LINUX20_ENV)
 #define	LONG_IS_32_BITS
 #endif
 
@@ -963,6 +963,7 @@ setkey(key)
 }
 #endif
 
+#if 0
 /*
  * "encrypt" routine (for backwards compatibility)
  */
@@ -993,6 +994,7 @@ encrypt(block, flag)
     }
     return (0);
 }
+#endif
 
 #ifdef CRYPT_DEBUG
 STATIC
