@@ -16,7 +16,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/kauth/krb_udp.c,v 1.23.14.2 2007/10/30 15:16:39 shadow Exp $");
+    ("$Header: /cvs/openafs/src/kauth/krb_udp.c,v 1.23.14.4 2008/02/04 03:59:06 jaltman Exp $");
 
 #include <afs/stds.h>
 #include <sys/types.h>
@@ -60,6 +60,9 @@ RCSID
 #define	KDC_GEN_ERR	20
 #endif
 
+#ifndef AFS_NT40_ENV
+#define closesocket close
+#endif
 
 int krb_udp_debug = 0;
 
@@ -817,7 +820,7 @@ process_udp_request(ksoc, pkt)
     return;
 }
 
-static
+static void
 SocketListener()
 {
     fd_set rfds;
@@ -892,11 +895,11 @@ SocketListener()
 	}
     }
     if (sock_kerb >= 0) {
-	close(sock_kerb);
+	closesocket(sock_kerb);
 	sock_kerb = -1;
     }
     if (sock_kerb5 >= 0) {
-	close(sock_kerb5);
+	closesocket(sock_kerb5);
 	sock_kerb5 = -1;
     }
     printf("UDP SocketListener exiting due to error\n");
