@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/ptserver/testpt.c,v 1.14.14.1 2007/04/10 18:39:54 shadow Exp $");
+    ("$Header: /cvs/openafs/src/ptserver/testpt.c,v 1.14.14.4 2007/10/31 04:09:32 shadow Exp $");
 
 #include <ctype.h>
 #include <errno.h>
@@ -24,13 +24,7 @@ RCSID
 #include <netdb.h>
 #include <netinet/in.h>
 #endif
-#ifdef HAVE_STRING_H
 #include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
 #endif
@@ -57,7 +51,7 @@ static char conf_dir[100];
 static char lcell[MAXCELLCHARS];
 
 int
-ListUsedIds(struct cmd_syndesc *as, char *arock)
+ListUsedIds(struct cmd_syndesc *as, void *arock)
 {
     afs_int32 code;
     namelist lnames;
@@ -308,14 +302,14 @@ CreateGroup(int g)
     }
 
     sprintf(name, "%s:%s%d", ownerName, createPrefix, g);
-    code = ubik_Call(PR_NewEntry, pruclient, 0, name, PRGRP, owner, &id);
+    code = ubik_PR_NewEntry(pruclient, 0, name, PRGRP, owner, &id);
     if (code) {
 	if (code == PREXIST) {
 	    code = pr_Delete(name);
 	    if (code == 0) {
 		nGDels++;
 		code =
-		    ubik_Call(PR_NewEntry, pruclient, 0, name, PRGRP, owner,
+		    ubik_PR_NewEntry(pruclient, 0, name, PRGRP, owner,
 			      &id);
 		if (code == 0) {
 		    if (verbose)
@@ -351,7 +345,7 @@ DeleteRandomId(afs_int32 *list)
     for (j = 0; j < number; j++) {	/* find an undeleted id */
 	m = (k + j) % number;
 	if (id = list[m]) {
-	    code = ubik_Call(PR_Delete, pruclient, 0, id);
+	    code = ubik_PR_Delete(pruclient, 0, id);
 	    if (code) {
 		afs_com_err(whoami, code, "Couldn't delete %di", id);
 		exit(22);
@@ -379,7 +373,7 @@ AddUser(int u, int g)
 	CreateGroup(g);
     ui = users[u];
     gi = groups[g];
-    code = ubik_Call(PR_AddToGroup, pruclient, 0, ui, gi);
+    code = ubik_PR_AddToGroup(pruclient, 0, ui, gi);
     if (code) {
 	afs_com_err(whoami, code, "couldn't add %d to %d", ui, gi);
 	exit(14);
@@ -398,7 +392,7 @@ RemUser(int u, int g)
 
     ui = users[u];
     gi = groups[g];
-    code = ubik_Call(PR_RemoveFromGroup, pruclient, 0, ui, gi);
+    code = ubik_PR_RemoveFromGroup(pruclient, 0, ui, gi);
     if (code) {
 	afs_com_err(whoami, code, "couldn't remove %d from %d", ui, gi);
 	exit(14);
@@ -410,7 +404,7 @@ RemUser(int u, int g)
 }
 
 int
-TestManyMembers(struct cmd_syndesc *as, char *arock)
+TestManyMembers(struct cmd_syndesc *as, void *arock)
 {
     char *filled;		/* users filled up */
     char *cleaned;		/* users cleaned up */
@@ -656,7 +650,7 @@ TestManyMembers(struct cmd_syndesc *as, char *arock)
 		}
 #define GETOWNED(xlist,xid) \
   (xlist).prlist_val = 0; (xlist).prlist_len = 0; \
-  code = ubik_Call (PR_ListOwned, pruclient, 0, (xid), &(xlist), &over); \
+  code = ubik_PR_ListOwned(pruclient, 0, (xid), &(xlist), &over); \
   if (code) { \
       afs_com_err (whoami, code, "getting owner list of (%di)", (xid)); \
       exit (23); } \
@@ -767,7 +761,7 @@ ka_ConvertBytes(char *ascii,		/* output buffer */
  */
 
 int
-TestPrServ(struct cmd_syndesc *as, char *arock)
+TestPrServ(struct cmd_syndesc *as, void *arock)
 {
     afs_int32 id;
     char name[PR_MAXNAMELEN + 1];
@@ -913,7 +907,7 @@ static char tmp_cell_file[128] = "";
 static char tmp_noauth_file[128] = "";
 
 static int
-MyAfterProc(struct cmd_syndesc *as, char *arock)
+MyAfterProc(struct cmd_syndesc *as, void *arock)
 {
     if (strlen(tmp_conf_file))
 	unlink(tmp_conf_file);
@@ -927,7 +921,7 @@ MyAfterProc(struct cmd_syndesc *as, char *arock)
 }
 
 static int
-MyBeforeProc(struct cmd_syndesc *as, char *arock)
+MyBeforeProc(struct cmd_syndesc *as, void *arock)
 {
     afs_int32 code;
     int i;
@@ -1109,7 +1103,7 @@ main(int argc, char *argv[])
     cmd_SetBeforeProc(MyBeforeProc, NULL);
     cmd_SetAfterProc(MyAfterProc, NULL);
 
-    ts = cmd_CreateSyntax("usedIds", ListUsedIds, 0,
+    ts = cmd_CreateSyntax("usedIds", ListUsedIds, NULL,
 			  "Find used (or unused) user (or group) ids");
     cmd_AddParm(ts, "-startId", CMD_SINGLE, CMD_OPTIONAL,
 		"id to start checking");
@@ -1118,10 +1112,10 @@ main(int argc, char *argv[])
     cmd_AddParm(ts, "-unused", CMD_FLAG, CMD_OPTIONAL, "print unused ids");
     add_std_args(ts);
 
-    ts = cmd_CreateSyntax("initcmd", TestPrServ, 0, "test the prserver");
+    ts = cmd_CreateSyntax("initcmd", TestPrServ, NULL, "test the prserver");
     add_std_args(ts);
 
-    ts = cmd_CreateSyntax("testmanymembers", TestManyMembers, 0,
+    ts = cmd_CreateSyntax("testmanymembers", TestManyMembers, NULL,
 			  "test creating users and groups w/ many members");
     cmd_AddParm(ts, "-number", CMD_SINGLE, 0,
 		"number of users/groups to create");
