@@ -19,7 +19,7 @@
 #endif
 
 RCSID
-    ("$Header: /cvs/openafs/src/rx/rx_event.c,v 1.14.2.2 2004/12/07 06:10:06 shadow Exp $");
+    ("$Header: /cvs/openafs/src/rx/rx_event.c,v 1.18 2005/11/05 06:48:18 jaltman Exp $");
 
 #ifdef KERNEL
 #ifndef UKERNEL
@@ -198,15 +198,14 @@ _rxevent_Post(struct clock *when, void (*func) (), void *arg, void *arg1,
     int isEarliest = 0;
 
     MUTEX_ENTER(&rxevent_lock);
-    AFS_ASSERT_RXGLOCK();
 #ifdef RXDEBUG
     if (rx_Log_event) {
 	struct clock now;
 	clock_GetTime(&now);
-	fprintf(rx_Log_event, "%d.%d: rxevent_Post(%d.%d, %lx, %lx, %lx, %d)\n",
+	fprintf(rx_Log_event, "%d.%d: rxevent_Post(%d.%d, %lp, %lp, %lp, %d)\n",
 		(int)now.sec, (int)now.usec, (int)when->sec, (int)when->usec,
-		(unsigned long)func, (unsigned long)arg,
-		(unsigned long)arg1, arg2);
+		func, arg,
+		arg1, arg2);
     }
 #endif
 
@@ -328,16 +327,15 @@ rxevent_Cancel_1(register struct rxevent *ev, register struct rx_call *call,
     if (rx_Log_event) {
 	struct clock now;
 	clock_GetTime(&now);
-	fprintf(rx_Log_event, "%d.%d: rxevent_Cancel_1(%d.%d, %lx, %lx)\n",
+	fprintf(rx_Log_event, "%d.%d: rxevent_Cancel_1(%d.%d, %lp, %lp)\n",
 		(int)now.sec, (int)now.usec, (int)ev->eventTime.sec,
-		(int)ev->eventTime.usec, (unsigned long)ev->func,
-		(unsigned long)ev->arg);
+		(int)ev->eventTime.usec, ev->func,
+		ev->arg);
     }
 #endif
     /* Append it to the free list (rather than prepending) to keep the free
      * list hot so nothing pages out
      */
-    AFS_ASSERT_RXGLOCK();
     MUTEX_ENTER(&rxevent_lock);
     if (!ev) {
 	MUTEX_EXIT(&rxevent_lock);
@@ -383,8 +381,6 @@ rxevent_RaiseEvents(struct clock *next)
 
     MUTEX_ENTER(&rxevent_lock);
 
-    AFS_ASSERT_RXGLOCK();
-
     /* Events are sorted by time, so only scan until an event is found that has
      * not yet timed out */
 
