@@ -35,7 +35,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/sys/pioctl_nt.c,v 1.34 2005/11/05 06:48:24 jaltman Exp $");
+    ("$Header: /cvs/openafs/src/sys/pioctl_nt.c,v 1.34.4.8 2007/10/18 22:07:59 jaltman Exp $");
 
 #include <afs/stds.h>
 #include <windows.h>
@@ -53,12 +53,13 @@ RCSID
 #include <osi.h>
 
 #include <cm.h>
-#include <cm_dir.h>
+#include <cm_server.h>
 #include <cm_cell.h>
 #include <cm_user.h>
 #include <cm_conn.h>
 #include <cm_scache.h>
 #include <cm_buf.h>
+#include <cm_dir.h>
 #include <cm_utils.h>
 #include <cm_ioctl.h>
 
@@ -90,12 +91,15 @@ CMtoUNIXerror(int cm_code)
     case CM_ERROR_NOACCESS:
 	return EACCES;
     case CM_ERROR_NOSUCHFILE:
+    case CM_ERROR_NOSUCHPATH:
+    case CM_ERROR_BPLUS_NOMATCH:
 	return ENOENT;
     case CM_ERROR_INVAL:
 	return EINVAL;
     case CM_ERROR_BADFD:
 	return EBADF;
     case CM_ERROR_EXISTS:
+    case CM_ERROR_INEXACT_MATCH:
 	return EEXIST;
     case CM_ERROR_CROSSDEVLINK:
 	return EXDEV;
@@ -115,6 +119,12 @@ CMtoUNIXerror(int cm_code)
 	return EDOM;		/* hack */
     case CM_ERROR_TOOMANYBUFS:
 	return EFBIG;		/* hack */
+    case CM_ERROR_ALLBUSY:
+        return EBUSY;
+    case CM_ERROR_ALLDOWN:
+        return ENOSYS;          /* hack */
+    case CM_ERROR_ALLOFFLINE:
+        return ENXIO;           /* hack */
     default:
 	if (cm_code > 0 && cm_code < EILSEQ)
 	    return cm_code;
@@ -154,6 +164,61 @@ IoctlDebug(void)
     return debug;
 }
 
+static DWORD 
+GetServiceStatus(
+    LPSTR lpszMachineName, 
+    LPSTR lpszServiceName,
+    DWORD *lpdwCurrentState) 
+{ 
+    DWORD           hr               = NOERROR; 
+    SC_HANDLE       schSCManager     = NULL; 
+    SC_HANDLE       schService       = NULL; 
+    DWORD           fdwDesiredAccess = 0; 
+    SERVICE_STATUS  ssServiceStatus  = {0}; 
+    BOOL            fRet             = FALSE; 
+
+    *lpdwCurrentState = 0; 
+ 
+    fdwDesiredAccess = GENERIC_READ; 
+ 
+    schSCManager = OpenSCManager(lpszMachineName,  
+                                 NULL,
+                                 fdwDesiredAccess); 
+ 
+    if(schSCManager == NULL) 
+    { 
+        hr = GetLastError();
+        goto cleanup; 
+    } 
+ 
+    schService = OpenService(schSCManager,
+                             lpszServiceName,
+                             fdwDesiredAccess); 
+ 
+    if(schService == NULL) 
+    { 
+        hr = GetLastError();
+        goto cleanup; 
+    } 
+ 
+    fRet = QueryServiceStatus(schService,
+                              &ssServiceStatus); 
+ 
+    if(fRet == FALSE) 
+    { 
+        hr = GetLastError(); 
+        goto cleanup; 
+    } 
+ 
+    *lpdwCurrentState = ssServiceStatus.dwCurrentState; 
+ 
+cleanup: 
+ 
+    CloseServiceHandle(schService); 
+    CloseServiceHandle(schSCManager); 
+ 
+    return(hr); 
+} 
 
 // krb5 functions
 DECL_FUNC_PTR(krb5_cc_default_name);
@@ -256,8 +321,11 @@ LoadFuncs(
     if (error) return 0;
     return 1;
 }
-
+#if defined(_IA64_) || defined(_AMD64_)
+#define KERB5DLL "krb5_64.dll"
+#else
 #define KERB5DLL "krb5_32.dll"
+#endif
 static BOOL
 IsKrb5Available()
 {
@@ -329,8 +397,12 @@ GetLSAPrincipalName(char * szUser, DWORD *dwSize)
 static long
 GetIoctlHandle(char *fileNamep, HANDLE * handlep)
 {
+#ifndef AFSIFS
     char *drivep;
     char netbiosName[MAX_NB_NAME_LENGTH];
+    DWORD CurrentState = 0;
+    char  HostName[64] = "";
+#endif
     char tbuffer[256]="";
     HANDLE fh;
     HKEY hk;
@@ -344,6 +416,12 @@ GetIoctlHandle(char *fileNamep, HANDLE * handlep)
     DWORD dwSize = sizeof(szUser);
 
 #ifndef AFSIFS
+    memset(HostName, '\0', sizeof(HostName));
+    gethostname(HostName, sizeof(HostName));
+    if (GetServiceStatus(HostName, TEXT("TransarcAFSDaemon"), &CurrentState) == NOERROR &&
+	CurrentState != SERVICE_RUNNING)
+	return -1;
+
     if (fileNamep) {
         drivep = strchr(fileNamep, ':');
         if (drivep && (drivep - fileNamep) >= 1) {
@@ -416,7 +494,7 @@ GetIoctlHandle(char *fileNamep, HANDLE * handlep)
     }
 #endif
 	
-	if (fh == INVALID_HANDLE_VALUE) {
+    if (fh == INVALID_HANDLE_VALUE) {
         int  gonext = 0;
 
         gle = GetLastError();
@@ -637,6 +715,9 @@ Transceive(HANDLE handle, fs_ioctlRequest_t * reqp)
     long rcount;
     long ioCount;
     DWORD gle;
+#ifdef AFSIFS
+    char *data;
+#endif
 
     rcount = (long)(reqp->mp - reqp->data);
     if (rcount <= 0) {
@@ -758,12 +839,11 @@ fs_GetFullPath(char *pathp, char *outPathp, long outSize)
     if (!pathp)
         return CM_ERROR_NOSUCHPATH;
 
-    //sprintf(tpath, "%c:\\", pathp[0]);
     rootDir = CreateFile(pathp, 0, 0, NULL, OPEN_EXISTING, FILE_FLAG_BACKUP_SEMANTICS, NULL);
     if (rootDir == INVALID_HANDLE_VALUE)
         return CM_ERROR_NOSUCHPATH;
 
-    wpath = tpath;
+    wpath = (wchar_t *)tpath;
     length = 0;
     if (!DeviceIoControl(rootDir, IOCTL_AFSRDR_GET_PATH, NULL, 0, wpath, 1000, &length, NULL))
     {
@@ -774,10 +854,8 @@ fs_GetFullPath(char *pathp, char *outPathp, long outSize)
 
     code = WideCharToMultiByte(CP_UTF8, 0/*WC_NO_BEST_FIT_CHARS*/, wpath, length/sizeof(wchar_t), outPathp, outSize/sizeof(wchar_t), NULL, NULL);
 
-//    strcpy(outPathp, tpath);
     return 0;
-#endif
-
+#else
     if (pathp[0] != 0 && pathp[1] == ':') {
 	/* there's a drive letter there */
 	firstp = pathp + 2;
@@ -879,6 +957,7 @@ fs_GetFullPath(char *pathp, char *outPathp, long outSize)
 	    *p = '\\';
     }
     return 0;
+#endif
 }
 
 long
@@ -922,6 +1001,10 @@ pioctl(char *pathp, long opcode, struct ViceIoctl *blobp, int follow)
 
     MarshallString(&preq, fullPath);
     if (blobp->in_size) {
+        if (blobp->in_size > sizeof(preq.data) - (preq.mp - preq.data)*sizeof(char)) {
+            errno = E2BIG;
+            return -1;
+        }
 	memcpy(preq.mp, blobp->in, blobp->in_size);
 	preq.mp += blobp->in_size;
     }
