@@ -354,6 +354,14 @@ extern int ubik_ClientInit(register struct rx_connection **serverconns,
 
 extern afs_int32 ubik_ClientDestroy(struct ubik_client *aclient);
 
+extern afs_int32 ubik_CallIter(int (*aproc) (), struct ubik_client *aclient,
+			       afs_int32 aflags, int *apos, long p1, long p2,
+			       long p3, long p4, long p5, long p6, long p7,
+			       long p8, long p9, long p10, long p11, long p12,
+			       long p13, long p14, long p15, long p16);
+
+extern struct rx_connection *ubik_RefreshConn(struct rx_connection *tc);
+
 /* ubik.c */
 extern int ubik_BeginTrans(register struct ubik_dbase *dbase,
 			   afs_int32 transMode, struct ubik_trans **transPtr);
@@ -366,7 +374,7 @@ extern afs_int32 ugen_ClientInit(int noAuthFlag, char *confDir, char *cellName,
 				 struct ubik_client **uclientp,
 				 int (*secproc) (), char *funcName, 
 				 afs_int32 gen_rxkad_level, 
-				 afs_int32 maxservers, afs_int32 serviceid, 
+				 afs_int32 maxservers, char *serviceid, 
 				 afs_int32 deadtime, afs_uint32 server, 
 				 afs_uint32 port, afs_int32 usrvid);
 
