@@ -22,7 +22,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/viced/viced.c,v 1.75.2.18 2007/11/29 19:26:41 shadow Exp $");
+    ("$Header: /cvs/openafs/src/viced/viced.c,v 1.75.2.20 2008/01/23 04:18:16 shadow Exp $");
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -164,6 +164,7 @@ int debuglevel = 0;
 int printBanner = 0;
 int rxJumbograms = 1;		/* default is to send and receive jumbograms. */
 int rxBind = 0;		/* don't bind */
+int rxkadDisableDotCheck = 0;      /* disable check for dot in principal name */ 
 int rxMaxMTU = -1;
 afs_int32 implicitAdminRights = PRSFS_LOOKUP;	/* The ADMINISTER right is 
 						 * already implied */
@@ -574,11 +575,6 @@ FsyncCheckLWP()
 
     setThreadId("FsyncCheckLWP");
 
-#ifdef AFS_PTHREAD_ENV
-    assert(pthread_cond_init(&fsync_cond, NULL) == 0);
-    assert(pthread_mutex_init(&fsync_glock_mutex, NULL) == 0);
-#endif
-
 #ifdef AFS_DEMAND_ATTACH_FS
     FS_STATE_WRLOCK;
     while (fs_state.mode == FS_MODE_NORMAL) {
@@ -895,6 +891,7 @@ FlagMsg()
     strcat(buffer, "[-rxdbge (enable rxevent debugging)] ");
     strcat(buffer, "[-rxmaxmtu <bytes>] ");
     strcat(buffer, "[-rxbind (bind the Rx socket to one address)] ");
+    strcat(buffer, "[-allow-dotted-principals (disable the rxkad principal name dot check)] ");
 #ifdef AFS_DEMAND_ATTACH_FS
     strcat(buffer, "[-fs-state-dont-save (disable state save during shutdown)] ");
     strcat(buffer, "[-fs-state-dont-restore (disable state restore during startup)] ");
@@ -1274,6 +1271,8 @@ ParseArgs(int argc, char *argv[])
 	    rxJumbograms = 0;
 	} else if (!strcmp(argv[i], "-rxbind")) {
 	    rxBind = 1;
+	} else if (!strcmp(argv[i], "-allow-dotted-principals")) {
+	    rxkadDisableDotCheck = 1;
 	} else if (!strcmp(argv[i], "-rxmaxmtu")) {
 	    if ((i + 1) >= argc) {
 		fprintf(stderr, "missing argument for -rxmaxmtu\n"); 
@@ -2087,6 +2086,11 @@ main(int argc, char *argv[])
 		("Failed to initialize RX, probably two servers running.\n"));
 	exit(-1);
     }
+    if (rxkadDisableDotCheck) {
+        rx_SetSecurityConfiguration(tservice, RXS_CONFIG_FLAGS,
+                                    (void *)RXS_CONFIG_FLAGS_DISABLE_DOTCHECK,
+                                    NULL);
+    }
     rx_SetMinProcs(tservice, 3);
     rx_SetMaxProcs(tservice, lwps);
     rx_SetCheckReach(tservice, 1);
@@ -2137,6 +2141,11 @@ main(int argc, char *argv[])
     /* allow super users to manage RX statistics */
     rx_SetRxStatUserOk(fs_rxstat_userok);
 
+#ifdef AFS_PTHREAD_ENV
+    assert(pthread_cond_init(&fsync_cond, NULL) == 0);
+    assert(pthread_mutex_init(&fsync_glock_mutex, NULL) == 0);
+#endif
+
 #if !defined(AFS_DEMAND_ATTACH_FS)
     /* 
      * For DAFS, we do not start the Rx server threads until after
