@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/vlserver/vlprocs.c,v 1.15.4.1 2007/10/30 15:16:56 shadow Exp $");
+    ("$Header: /cvs/openafs/src/vlserver/vlprocs.c,v 1.15.4.2 2007/11/26 21:08:46 shadow Exp $");
 
 #include <sys/types.h>
 #include <stdio.h>
@@ -55,17 +55,17 @@ static char rxinfo_str[128];	/* Need rxinfo string to be non-local */
 static int put_attributeentry();
 static int put_nattributeentry();
 static int RemoveEntry();
-static ReleaseEntry();
+static void ReleaseEntry();
 static int check_vldbentry();
 static int check_nvldbentry();
 static int vldbentry_to_vlentry();
 static int nvldbentry_to_vlentry();
 static get_vldbupdateentry();
 static int repsite_exists();
-static repsite_compress();
-static vlentry_to_vldbentry();
-static vlentry_to_nvldbentry();
-static vlentry_to_uvldbentry();
+static void repsite_compress();
+static void vlentry_to_vldbentry();
+static void vlentry_to_nvldbentry();
+static void vlentry_to_uvldbentry();
 static int InvalidVolname();
 static int InvalidVoltype();
 static int InvalidOperation();
@@ -2556,7 +2556,7 @@ RemoveEntry(trans, entryptr, tentry)
     return 0;
 }
 
-static
+static void
 ReleaseEntry(tentry, releasetype)
      struct nvlentry *tentry;
      afs_int32 releasetype;
@@ -2861,7 +2861,7 @@ repsite_exists(VlEntry, server, partition)
 
 
 /* Repsite table compression: used when deleting a repsite entry so that all active repsite entries are on the top of the table. */
-static
+static void
 repsite_compress(VlEntry, offset)
      struct nvlentry *VlEntry;
      int offset;
@@ -2882,7 +2882,7 @@ repsite_compress(VlEntry, offset)
 
 
 /* Convert from the internal (compacted) vldb entry to the external representation used by the interface. */
-static
+static void
 vlentry_to_vldbentry(VlEntry, VldbEntry)
      struct nvlentry *VlEntry;
      struct vldbentry *VldbEntry;
@@ -2924,7 +2924,7 @@ vlentry_to_vldbentry(VlEntry, VldbEntry)
 
 
 /* Convert from the internal (compacted) vldb entry to the external representation used by the interface. */
-static
+static void
 vlentry_to_nvldbentry(VlEntry, VldbEntry)
      struct nvlentry *VlEntry;
      struct nvldbentry *VldbEntry;
@@ -2964,7 +2964,7 @@ vlentry_to_nvldbentry(VlEntry, VldbEntry)
     VldbEntry->flags = VlEntry->flags;
 }
 
-static
+static void
 vlentry_to_uvldbentry(VlEntry, VldbEntry)
      struct nvlentry *VlEntry;
      struct uvldbentry *VldbEntry;
